@@ -34,13 +34,13 @@ module Param = struct
     { warm_up_time = 1.0
     ; min_measures = 10
     ; max_confidence = 0.03
-    ; max_duration = 20.
+    ; max_duration = 120.
     ; verbose = false
     }
 
-  let fast x = { x with min_measures = 5; max_confidence = 0.15 }
+  let fast x = { x with min_measures = 5; max_confidence = 0.15; max_duration = 20. }
 
-  let ffast x = { x with min_measures = 2; max_confidence = 42. }
+  let ffast x = { x with min_measures = 2; max_confidence = 42.; max_duration = 20. }
 
   let verbose x = { x with verbose = true }
 end
@@ -112,9 +112,11 @@ let rec measure_rec ~print (param : Param.t) cmd l =
   let l = t :: l in
   if need_more ~print param l then measure_rec ~print param cmd l else l
 
-let measure_one param code meas spec nm cmd =
+let measure_one param code (meas : Measure.t) spec nm cmd =
   let l =
-    if measures_need_update code meas spec nm then [] else read_measures meas spec nm
+    if measures_need_update code meas spec nm
+    then []
+    else read_measures meas.Measure.path spec nm
   in
   if need_more ~print:false param l
   then (
@@ -126,7 +128,7 @@ let measure_one param code meas spec nm cmd =
     l)
   else l
 
-let measure param code meas spec cmd =
+let measure ~param ~code ~(meas : Measure.t) ~spec cmd =
   List.iter (Spec.find_names ~root:code spec) ~f:(fun nm ->
       let cmd = if cmd = "" then cmd else cmd ^ " " in
       let cmd = Format.sprintf "%s%s" cmd (Spec.file ~root:code spec nm) in
@@ -146,17 +148,38 @@ let ml_size param =
         src
         dst)
 
-let file_size param =
+let file_size ?(wasm = false) param =
   compile_no_ext param ~comptime:false (fun ~src ~dst ->
-      Format.sprintf "wc -c < %s > %s" src dst)
+      let files =
+        if wasm then src ^ " " ^ Filename.remove_extension src ^ ".assets/*" else src
+      in
+      Format.sprintf "cat %s | wc -c > %s" files dst)
 
-let compr_file_size param =
+let compr_file_size ?(wasm = false) param =
   compile_no_ext param ~comptime:false (fun ~src ~dst ->
-      Format.sprintf "sed 's/^ *//g' %s | gzip -c | wc -c > %s" src dst)
+      let input =
+        if wasm
+        then
+          Printf.sprintf
+            "echo %s %s | xargs gzip -c"
+            src
+            (Filename.remove_extension src ^ ".assets/*")
+        else Printf.sprintf "sed 's/^ *//g' %s | gzip -c" src
+      in
+      Format.sprintf "%s | wc -c > %s" input dst)
 
-let bzip2_file_size param =
+let bzip2_file_size ?(wasm = false) param =
   compile_no_ext param ~comptime:false (fun ~src ~dst ->
-      Format.sprintf "sed 's/^ *//g' %s | bzip2 -c | wc -c > %s" src dst)
+      let input =
+        if wasm
+        then
+          Printf.sprintf
+            "echo %s %s | xargs bzip2 -c"
+            src
+            (Filename.remove_extension src ^ ".assets/*")
+        else Printf.sprintf "sed 's/^ *//g' %s | bzip2 -c" src
+      in
+      Format.sprintf "%s | wc -c > %s" input dst)
 
 let runtime_size param =
   compile_no_ext param ~comptime:false (fun ~src ~dst ->
@@ -174,100 +197,104 @@ let gen_size param =
 
 (****)
 
-let read_config file =
-  if not (Sys.file_exists file)
-  then (
-    Format.eprintf "Configuration file '%s' not found!@." file;
-    exit 1);
-  let i = ref [] in
-  let ch = open_in file in
-  (try
-     while true do
-       let line = String.trim (input_line ch) in
-       if line.[0] <> '#'
-       then
-         match
-           List.filter
-             ~f:(function
-               | "" -> false
-               | _ -> true)
-             (split_on_char line ~sep:' ')
-         with
-         | "interpreter" :: nm :: rem -> i := (String.concat ~sep:" " rem, nm) :: !i
-         | [ "interpreter" ] ->
-             Format.eprintf "Malformed config option '%s'@." line;
-             exit 1
-         | kind :: _ ->
-             Format.eprintf "Unknown config option '%s'@." kind;
-             exit 1
-         | [] ->
-             Format.eprintf "Bad config line '%s'@." line;
-             exit 1
-     done
-   with End_of_file -> ());
-  close_in ch;
-  List.rev !i
-
 let _ =
   let compile_only = ref false in
   let full = ref false in
-  let effects = ref false in
-  let conf_file = ref "run.config" in
-  let nobyteopt = ref false in
+  let interpreter_config = ref "run.config" in
   let param = ref Param.default in
+  let reports = ref [] in
   let fast_run () = param := Param.fast !param in
   let ffast_run () = param := Param.ffast !param in
   let verbose () = param := Param.verbose !param in
   let options =
     [ "-compile", Arg.Set compile_only, " only compiles"
     ; "-all", Arg.Set full, " run all benchmarks"
-    ; "-effects", Arg.Set effects, " only run with and without effect handler support"
-    ; "-config", Arg.Set_string conf_file, "<file> use <file> as a config file"
+    ; "-config", Arg.Set_string interpreter_config, "<file> use <file> as a config file"
     ; "-fast", Arg.Unit fast_run, " perform less iterations"
     ; "-ffast", Arg.Unit ffast_run, " perform very few iterations"
     ; "-verbose", Arg.Unit verbose, " verbose"
-    ; ( "-nobyteopt"
-      , Arg.Set nobyteopt
-      , " do not run benchs on bytecode and native programs" )
     ]
   in
   Arg.parse
     (Arg.align options)
-    (fun s -> raise (Arg.Bad (Format.sprintf "unknown option `%s'" s)))
-    (Format.sprintf "Usage: %s [options]" Sys.argv.(0));
-  let conf_file = !conf_file in
+    (fun report -> reports := report :: !reports)
+    (Format.sprintf "Usage: %s [options] [REPORTS.config]*" Sys.argv.(0));
   let compile_only = !compile_only in
-  let nobyteopt = !nobyteopt in
   let full = !full in
-  let effects = !effects in
   let param = !param in
-  let interpreters = read_config conf_file in
+  let interpreters = read_interpreter_config !interpreter_config in
+  let filtered, filter_bench =
+    match !reports with
+    | [] -> false, fun (_, _) -> true
+    | reports ->
+        List.map reports ~f:read_report_config
+        |> List.concat_map ~f:(fun l ->
+               List.concat_map l ~f:(fun l ->
+                   List.concat_map l ~f:(function
+                     | None -> []
+                     | Some (p1, p2, _measure, _) -> (
+                         match String.split_on_char ~sep:'/' p1 with
+                         | [ "results"; "times"; _host; interpreter ] ->
+                             [ interpreter, p2 ]
+                         | _ -> []))))
+        |> List.sort_uniq ~cmp:compare
+        |> fun required ->
+        true, fun (interp, suite) -> List.mem (interp, Spec.name suite) ~set:required
+  in
   let compile = compile param ~comptime:true in
-  let compile_jsoo ?(effects = false) opts =
+  let compile_jsoo ?(effects = `None) opts =
     compile
       (Format.sprintf
          "js_of_ocaml -q --target-env browser --debug mark-runtime-gen %s %s"
          opts
-         (if effects then "--enable=effects" else "--disable=effects"))
+         (match effects with
+         | `None -> "--effects=disabled"
+         | `Cps -> "--effects=cps"
+         | `Double_translation -> "--effects=double-translation"))
+  in
+  let compile_wasmoo ?(effects = `None) opts =
+    compile
+      (Format.sprintf
+         "wasm_of_ocaml -q %s %s"
+         opts
+         (match effects with
+         | `None -> ""
+         | `Cps -> "--effects=cps"))
   in
   Format.eprintf "Compile@.";
   compile "ocamlc" src Spec.ml code Spec.byte;
   compile "ocamlopt" src Spec.ml code Spec.opt;
-  compile_jsoo "" code Spec.byte code Spec.js_of_ocaml;
+  compile_wasmoo "--opt=2" code Spec.byte code Spec.wasm_of_ocaml;
+  compile_jsoo "--opt=2" code Spec.byte code Spec.js_of_ocaml;
   compile_jsoo "--opt=3" code Spec.byte code Spec.js_of_ocaml_o3;
   compile_jsoo "--enable=use-js-string" code Spec.byte code Spec.js_of_ocaml_js_string;
   compile_jsoo "--disable inline" code Spec.byte code Spec.js_of_ocaml_inline;
   compile_jsoo "--disable deadcode" code Spec.byte code Spec.js_of_ocaml_deadcode;
   compile_jsoo "--disable compact" code Spec.byte code Spec.js_of_ocaml_compact;
   compile_jsoo "--disable optcall" code Spec.byte code Spec.js_of_ocaml_call;
-  compile_jsoo ~effects:true "" code Spec.byte code Spec.js_of_ocaml_effects;
+  compile_jsoo ~effects:`Cps "" code Spec.byte code Spec.js_of_ocaml_effects_cps;
+  compile_jsoo
+    ~effects:`Double_translation
+    ""
+    code
+    Spec.byte
+    code
+    Spec.js_of_ocaml_effects_double_translation;
   compile "ocamlc -unsafe" src Spec.ml code Spec.byte_unsafe;
   compile "ocamlopt" src Spec.ml code Spec.opt_unsafe;
   compile_jsoo "" code Spec.byte_unsafe code Spec.js_of_ocaml_unsafe;
   Format.eprintf "Sizes@.";
+  let sizes = sizes.Measure.path in
   ml_size param src Spec.ml sizes Spec.ml;
   file_size param code Spec.byte sizes Spec.byte;
   file_size param code Spec.js_of_ocaml sizes (Spec.sub_spec Spec.js_of_ocaml "full");
+  file_size
+    ~wasm:true
+    param
+    code
+    Spec.wasm_of_ocaml
+    sizes
+    (Spec.sub_spec Spec.wasm_of_ocaml "full");
   compr_file_size
     param
     code
@@ -277,21 +304,47 @@ let _ =
   compr_file_size
     param
     code
-    Spec.js_of_ocaml_effects
+    Spec.js_of_ocaml_effects_cps
     sizes
-    (Spec.sub_spec Spec.js_of_ocaml_effects "gzipped");
+    (Spec.sub_spec Spec.js_of_ocaml_effects_cps "gzipped");
+  compr_file_size
+    param
+    code
+    Spec.js_of_ocaml_effects_double_translation
+    sizes
+    (Spec.sub_spec Spec.js_of_ocaml_effects_double_translation "gzipped");
+  compr_file_size
+    ~wasm:true
+    param
+    code
+    Spec.wasm_of_ocaml
+    sizes
+    (Spec.sub_spec Spec.wasm_of_ocaml "gzipped");
   bzip2_file_size
     param
     code
-    Spec.js_of_ocaml_effects
+    Spec.js_of_ocaml_effects_cps
     sizes
-    (Spec.sub_spec Spec.js_of_ocaml_effects "bzip2");
+    (Spec.sub_spec Spec.js_of_ocaml_effects_cps "bzip2");
+  bzip2_file_size
+    param
+    code
+    Spec.js_of_ocaml_effects_double_translation
+    sizes
+    (Spec.sub_spec Spec.js_of_ocaml_effects_double_translation "bzip2");
   bzip2_file_size
     param
     code
     Spec.js_of_ocaml
     sizes
     (Spec.sub_spec Spec.js_of_ocaml "bzip2");
+  bzip2_file_size
+    ~wasm:true
+    param
+    code
+    Spec.wasm_of_ocaml
+    sizes
+    (Spec.sub_spec Spec.wasm_of_ocaml "bzip2");
   runtime_size
     param
     code
@@ -305,41 +358,58 @@ let _ =
   gen_size param code Spec.js_of_ocaml_deadcode sizes Spec.js_of_ocaml_deadcode;
   gen_size param code Spec.js_of_ocaml_compact sizes Spec.js_of_ocaml_compact;
   gen_size param code Spec.js_of_ocaml_call sizes Spec.js_of_ocaml_call;
-  gen_size param code Spec.js_of_ocaml_effects sizes Spec.js_of_ocaml_effects;
+  gen_size param code Spec.js_of_ocaml_effects_cps sizes Spec.js_of_ocaml_effects_cps;
+  gen_size
+    param
+    code
+    Spec.js_of_ocaml_effects_double_translation
+    sizes
+    Spec.js_of_ocaml_effects_double_translation;
   if compile_only then exit 0;
   Format.eprintf "Measure@.";
-  if not nobyteopt
-  then (
-    measure param code times Spec.opt "";
-    measure param code times Spec.byte "");
-  let compilers, suites =
-    if full
-    then
-      ( interpreters
-      , [ Some Spec.js_of_ocaml
-        ; Some Spec.js_of_ocaml_o3
-        ; Some Spec.js_of_ocaml_js_string
-        ; Some Spec.js_of_ocaml_unsafe
-        ; Some Spec.js_of_ocaml_inline
-        ; Some Spec.js_of_ocaml_deadcode
-        ; Some Spec.js_of_ocaml_compact
-        ; Some Spec.js_of_ocaml_call
-        ; Some Spec.js_of_ocaml_effects
-        ] )
-    else if effects
-    then
-      ( (match interpreters with
-        | i :: _ -> [ i ]
-        | [] -> [])
-      , [ Some Spec.js_of_ocaml; Some Spec.js_of_ocaml_effects ] )
+  if filter_bench ("", Spec.opt) then measure ~param ~code ~meas:times ~spec:Spec.opt "";
+  if filter_bench ("", Spec.byte) then measure ~param ~code ~meas:times ~spec:Spec.byte "";
+  let all_spec =
+    [ Some Spec.js_of_ocaml
+    ; Some Spec.js_of_ocaml_o3
+    ; Some Spec.js_of_ocaml_js_string
+    ; Some Spec.js_of_ocaml_unsafe
+    ; Some Spec.js_of_ocaml_inline
+    ; Some Spec.js_of_ocaml_deadcode
+    ; Some Spec.js_of_ocaml_compact
+    ; Some Spec.js_of_ocaml_call
+    ; Some Spec.js_of_ocaml_effects_cps
+    ; Some Spec.js_of_ocaml_effects_double_translation
+    ; Some Spec.wasm_of_ocaml
+    ]
+  in
+  let interpreters, suites =
+    if full || filtered
+    then interpreters, all_spec
     else
       ( (match interpreters with
         | i :: _ -> [ i ]
         | [] -> [])
-      , [ Some Spec.js_of_ocaml ] )
+      , [ Some Spec.js_of_ocaml; Some Spec.wasm_of_ocaml ] )
   in
-  List.iter compilers ~f:(fun (comp, dir) ->
-      measure param src (Filename.concat times dir) Spec.js comp;
+  List.iter interpreters ~f:(fun (comp, dir) ->
+      (* Measure the time taken by an implementation in Javascript for comparison *)
+      if filter_bench (dir, Spec.js)
+      then
+        measure
+          ~param
+          ~code:src
+          ~meas:Measure.{ times with path = Filename.concat times.path dir }
+          ~spec:Spec.js
+          comp;
       List.iter suites ~f:(function
-          | None -> ()
-          | Some suite -> measure param code (Filename.concat times dir) suite comp))
+        | None -> ()
+        | Some suite ->
+            if filter_bench (dir, suite)
+            then
+              measure
+                ~param
+                ~code
+                ~meas:Measure.{ times with path = Filename.concat times.path dir }
+                ~spec:suite
+                comp))
