@@ -39,11 +39,22 @@ let trim_trailing_dir_sep s =
 
 let normalize_include_dirs dirs = List.map dirs ~f:trim_trailing_dir_sep
 
+let normalize_effects (effects : [ `Disabled | `Cps | `Double_translation ] option) common
+    : Config.effects_backend =
+  match effects with
+  | None ->
+      (* For backward compatibility, consider that [--enable effects] alone means
+         [--effects cps] *)
+      if List.mem ~eq:String.equal "effects" common.Jsoo_cmdline.Arg.optim.enable
+      then `Cps
+      else `Disabled
+  | Some ((`Disabled | `Cps | `Double_translation) as e) -> e
+
 type t =
   { common : Jsoo_cmdline.Arg.t
   ; (* compile option *)
-    profile : Driver.profile option
-  ; source_map : (string option * Source_map.Standard.t) option
+    profile : Profile.t option
+  ; source_map : Source_map.Encoding_spec.t option
   ; runtime_files : string list
   ; no_runtime : bool
   ; include_runtime : bool
@@ -53,6 +64,7 @@ type t =
   ; static_env : (string * string) list
   ; wrap_with_fun : [ `Iife | `Named of string | `Anonymous ]
   ; target_env : Target_env.t
+  ; shape_files : string list
   ; (* toplevel *)
     dynlink : bool
   ; linkall : bool
@@ -65,8 +77,29 @@ type t =
   ; fs_output : string option
   ; fs_external : bool
   ; keep_unit_names : bool
+  ; effects : Config.effects_backend
   }
 
+let set_param =
+  let doc = "Set compiler options." in
+  let all = List.map (Config.Param.all ()) ~f:(fun (x, _, _) -> x, x) in
+  let pair = Arg.(pair ~sep:'=' (enum all) string) in
+  let parser s =
+    match Arg.conv_parser pair s with
+    | Ok (k, v) -> (
+        match
+          List.find ~f:(fun (k', _, _) -> String.equal k k') (Config.Param.all ())
+        with
+        | _, _, valid -> (
+            match valid v with
+            | Ok () -> Ok (k, v)
+            | Error msg -> Error (`Msg ("Unexpected VALUE after [=], " ^ msg))))
+    | Error _ as e -> e
+  in
+  let printer = Arg.conv_printer pair in
+  let c = Arg.conv (parser, printer) in
+  Arg.(value & opt_all (list c) [] & info [ "set" ] ~docv:"PARAM=VALUE" ~doc)
+
 let wrap_with_fun_conv =
   let conv s =
     if String.equal s ""
@@ -102,6 +135,10 @@ let options =
     let doc = "Set output file name to [$(docv)]." in
     Arg.(value & opt (some string) None & info [ "o" ] ~docv:"FILE" ~doc)
   in
+  let shape_files =
+    let doc = "load shape file [$(docv)]." in
+    Arg.(value & opt_all string [] & info [ "load-shape" ] ~docv:"FILE" ~doc)
+  in
   let input_file =
     let doc =
       "Compile the bytecode program [$(docv)]. "
@@ -115,7 +152,9 @@ let options =
   in
   let profile =
     let doc = "Set optimization profile : [$(docv)]." in
-    let profile = List.map Driver.profiles ~f:(fun (i, p) -> string_of_int i, p) in
+    let profile =
+      List.map Profile.all ~f:(fun p -> string_of_int (Profile.to_int p), p)
+    in
     Arg.(value & opt (some (enum profile)) None & info [ "opt" ] ~docv:"NUM" ~doc)
   in
   let noruntime =
@@ -146,6 +185,10 @@ let options =
     let doc = "Do not inline sources in source map." in
     Arg.(value & flag & info [ "source-map-no-source" ] ~doc)
   in
+  let sourcemap_empty =
+    let doc = "Always generate empty source maps." in
+    Arg.(value & flag & info [ "empty-sourcemap"; "empty-source-map" ] ~doc)
+  in
   let sourcemap_root =
     let doc = "root dir for source map." in
     Arg.(value & opt (some string) None & info [ "source-map-root" ] ~doc)
@@ -157,14 +200,6 @@ let options =
     in
     Arg.(value & opt wrap_with_fun_conv `Iife & info [ "wrap-with-fun" ] ~doc)
   in
-  let set_param =
-    let doc = "Set compiler options." in
-    let all = List.map (Config.Param.all ()) ~f:(fun (x, _) -> x, x) in
-    Arg.(
-      value
-      & opt_all (list (pair ~sep:'=' (enum all) string)) []
-      & info [ "set" ] ~docv:"PARAM=VALUE" ~doc)
-  in
   let set_env =
     let doc = "Set environment variable statically." in
     Arg.(
@@ -253,6 +288,24 @@ let options =
       & opt (some string) None
       & info [ "ofs" ] ~docs:filesystem_section ~docv:"FILE" ~doc)
   in
+  let effects =
+    let doc =
+      "Select an implementation of effect handlers. [$(docv)] should be one of $(b,cps), \
+       $(b,double-translation) or $(b,disabled) (the default). Effects won't be \
+       supported if unspecified."
+    in
+    Arg.(
+      value
+      & opt
+          (some
+             (enum
+                [ "cps", `Cps
+                ; "double-translation", `Double_translation
+                ; "disabled", `Disabled
+                ]))
+          None
+      & info [ "effects" ] ~docv:"KIND" ~doc)
+  in
   let build_t
       common
       set_param
@@ -274,12 +327,15 @@ let options =
       sourcemap
       sourcemap_inline_in_js
       sourcemap_don't_inline_content
+      sourcemap_empty
       sourcemap_root
       target_env
       output_file
       input_file
       js_files
-      keep_unit_names =
+      keep_unit_names
+      effects
+      shape_files =
     let inline_source_content = not sourcemap_don't_inline_content in
     let chop_extension s = try Filename.chop_extension s with Invalid_argument _ -> s in
     let runtime_files = js_files in
@@ -307,17 +363,25 @@ let options =
           | `Name file, _ -> Some file, Some (chop_extension file ^ ".map")
           | `Stdout, _ -> None, None
         in
-        Some
-          ( sm_output_file
-          , { (Source_map.Standard.empty ~inline_source_content) with
-              file
-            ; sourceroot = sourcemap_root
-            } )
+        let source_map =
+          { (Source_map.Standard.empty ~inline_source_content) with
+            file
+          ; sourceroot = sourcemap_root
+          }
+        in
+        let spec =
+          { Source_map.Encoding_spec.output_file = sm_output_file
+          ; source_map
+          ; keep_empty = sourcemap_empty
+          }
+        in
+        Some spec
       else None
     in
     let params : (string * string) list = List.flatten set_param in
     let static_env : (string * string) list = List.flatten set_env in
     let include_dirs = normalize_include_dirs include_dirs in
+    let effects = normalize_effects effects common in
     `Ok
       { common
       ; params
@@ -341,6 +405,8 @@ let options =
       ; bytecode
       ; source_map
       ; keep_unit_names
+      ; effects
+      ; shape_files
       }
   in
   let t =
@@ -366,12 +432,15 @@ let options =
       $ sourcemap
       $ sourcemap_inline_in_js
       $ sourcemap_don't_inline_content
+      $ sourcemap_empty
       $ sourcemap_root
       $ target_env
       $ output_file
       $ input_file
       $ js_files
-      $ keep_unit_names)
+      $ keep_unit_names
+      $ effects
+      $ shape_files)
   in
   Term.ret t
 
@@ -411,6 +480,10 @@ let options_runtime_only =
     let doc = "Do not inline sources in source map." in
     Arg.(value & flag & info [ "source-map-no-source" ] ~doc)
   in
+  let sourcemap_empty =
+    let doc = "Always generate empty source maps." in
+    Arg.(value & flag & info [ "empty-sourcemap"; "empty-source-map" ] ~doc)
+  in
   let sourcemap_root =
     let doc = "root dir for source map." in
     Arg.(value & opt (some string) None & info [ "source-map-root" ] ~doc)
@@ -441,14 +514,6 @@ let options_runtime_only =
     in
     Arg.(value & opt wrap_with_fun_conv `Iife & info [ "wrap-with-fun" ] ~doc)
   in
-  let set_param =
-    let doc = "Set compiler options." in
-    let all = List.map (Config.Param.all ()) ~f:(fun (x, _) -> x, x) in
-    Arg.(
-      value
-      & opt_all (list (pair ~sep:'=' (enum all) string)) []
-      & info [ "set" ] ~docv:"PARAM=VALUE" ~doc)
-  in
   let set_env =
     let doc = "Set environment variable statically." in
     Arg.(
@@ -496,6 +561,24 @@ let options_runtime_only =
       & opt (some string) None
       & info [ "ofs" ] ~docs:filesystem_section ~docv:"FILE" ~doc)
   in
+  let effects =
+    let doc =
+      "Select an implementation of effect handlers. [$(docv)] should be one of $(b,cps), \
+       $(b,double-translation), or $(b,disabled) (the default). Effects won't be \
+       supported if unspecified."
+    in
+    Arg.(
+      value
+      & opt
+          (some
+             (enum
+                [ "cps", `Cps
+                ; "double-translation", `Double_translation
+                ; "disabled", `Disabled
+                ]))
+          None
+      & info [ "effects" ] ~docv:"KIND" ~doc)
+  in
   let build_t
       common
       toplevel
@@ -512,10 +595,12 @@ let options_runtime_only =
       sourcemap
       sourcemap_inline_in_js
       sourcemap_don't_inline_content
+      sourcemap_empty
       sourcemap_root
       target_env
       output_file
-      js_files =
+      js_files
+      effects =
     let inline_source_content = not sourcemap_don't_inline_content in
     let chop_extension s = try Filename.chop_extension s with Invalid_argument _ -> s in
     let runtime_files = js_files in
@@ -533,17 +618,25 @@ let options_runtime_only =
           | `Name file, _ -> Some file, Some (chop_extension file ^ ".map")
           | `Stdout, _ -> None, None
         in
-        Some
-          ( sm_output_file
-          , { (Source_map.Standard.empty ~inline_source_content) with
-              file
-            ; sourceroot = sourcemap_root
-            } )
+        let source_map =
+          { (Source_map.Standard.empty ~inline_source_content) with
+            file
+          ; sourceroot = sourcemap_root
+          }
+        in
+        let spec =
+          { Source_map.Encoding_spec.output_file = sm_output_file
+          ; source_map
+          ; keep_empty = sourcemap_empty
+          }
+        in
+        Some spec
       else None
     in
     let params : (string * string) list = List.flatten set_param in
     let static_env : (string * string) list = List.flatten set_env in
     let include_dirs = normalize_include_dirs include_dirs in
+    let effects = normalize_effects effects common in
     `Ok
       { common
       ; params
@@ -567,6 +660,8 @@ let options_runtime_only =
       ; bytecode = `None
       ; source_map
       ; keep_unit_names = false
+      ; effects
+      ; shape_files = []
       }
   in
   let t =
@@ -587,9 +682,11 @@ let options_runtime_only =
       $ sourcemap
       $ sourcemap_inline_in_js
       $ sourcemap_don't_inline_content
+      $ sourcemap_empty
       $ sourcemap_root
       $ target_env
       $ output_file
-      $ js_files)
+      $ js_files
+      $ effects)
   in
   Term.ret t
