@@ -23,7 +23,7 @@ open Cmdliner
 
 type t =
   { common : Jsoo_cmdline.Arg.t
-  ; source_map : (string option * Source_map.Standard.t) option
+  ; source_map : Source_map.Encoding_spec.t option
   ; js_files : string list
   ; output_file : string option
   ; resolve_sourcemap_url : bool
@@ -51,6 +51,10 @@ let options =
     let doc = "Inline sourcemap in the generated JavaScript." in
     Arg.(value & flag & info [ "source-map-inline" ] ~doc)
   in
+  let sourcemap_empty =
+    let doc = "Always generate empty source maps." in
+    Arg.(value & flag & info [ "empty-sourcemap"; "empty-source-map" ] ~doc)
+  in
   let sourcemap_root =
     let doc = "root dir for source map." in
     Arg.(value & opt (some string) None & info [ "source-map-root" ] ~doc)
@@ -83,6 +87,7 @@ let options =
       no_sourcemap
       sourcemap
       sourcemap_inline_in_js
+      sourcemap_empty
       sourcemap_root
       output_file
       resolve_sourcemap_url
@@ -100,12 +105,19 @@ let options =
           | Some file -> Some file, Some (chop_extension file ^ ".map")
           | None -> None, None
         in
-        Some
-          ( sm_output_file
-          , { (Source_map.Standard.empty ~inline_source_content:true) with
-              file
-            ; sourceroot = sourcemap_root
-            } )
+        let source_map =
+          { (Source_map.Standard.empty ~inline_source_content:true) with
+            file
+          ; sourceroot = sourcemap_root
+          }
+        in
+        let spec =
+          { Source_map.Encoding_spec.output_file = sm_output_file
+          ; source_map
+          ; keep_empty = sourcemap_empty
+          }
+        in
+        Some spec
       else None
     in
     `Ok
@@ -126,6 +138,7 @@ let options =
       $ no_sourcemap
       $ sourcemap
       $ sourcemap_inline_in_js
+      $ sourcemap_empty
       $ sourcemap_root
       $ output_file
       $ resolve_sourcemap_url
