@@ -34,7 +34,6 @@ let runtime =
     ; hash
     ; ieee_754
     ; int64
-    ; internalMod
     ; ints
     ; io
     ; jslib
@@ -54,9 +53,10 @@ let runtime =
     ; domain
     ; prng
     ; sync
-    ; effect
+    ; effect_
     ; zstd
     ; runtime_events
+    ; blake2
     ]
 
 include Files
