@@ -23,8 +23,7 @@
     Base64 is a group of similar binary-to-text encoding schemes that represent
     binary data in an ASCII string format by translating it into a radix-64
     representation. It is specified in RFC 4648.
-
-    {e Release 5.6.0 - https://ocsigen.org/js_of_ocaml/latest/manual/overview} *)
+ *)
 
 type ('a, 'b) result =
   | Ok of 'a
@@ -65,7 +64,7 @@ val decode_exn :
     [padding] character. If input is not padded correctly, decoder does the
     best-effort but it does not ensure [decode_exn (encode ~pad:false x) = x].
 
-    @raise if Invalid_argument [s] is not a valid Base64 string. *)
+    @raise Stdlib.Invalid_argument if [s] is not a valid Base64 string. *)
 
 val decode_sub :
      ?pad:bool
