@@ -26,29 +26,34 @@ module Flag = struct
 
   let o ~name ~default =
     let state =
-      try List.assoc name !optims
-      with Not_found ->
-        let state = ref default in
-        optims := (name, state) :: !optims;
-        state
+      match List.string_assoc name !optims with
+      | Some x -> x
+      | None ->
+          let state = ref default in
+          optims := (name, state) :: !optims;
+          state
     in
     fun () -> !state
 
   let find s =
-    try !(List.assoc s !optims)
-    with Not_found -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
+    match List.string_assoc s !optims with
+    | Some x -> !x
+    | None -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
 
   let set s b =
-    try List.assoc s !optims := b
-    with Not_found -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
+    match List.string_assoc s !optims with
+    | Some s -> s := b
+    | None -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
 
   let disable s =
-    try List.assoc s !optims := false
-    with Not_found -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
+    match List.string_assoc s !optims with
+    | Some s -> s := false
+    | None -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
 
   let enable s =
-    try List.assoc s !optims := true
-    with Not_found -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
+    match List.string_assoc s !optims with
+    | Some s -> s := true
+    | None -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
 
   let pretty = o ~name:"pretty" ~default:false
 
@@ -84,8 +89,6 @@ module Flag = struct
 
   let improved_stacktrace = o ~name:"with-js-error" ~default:false
 
-  let warn_unused = o ~name:"warn-unused" ~default:false
-
   let inline_callgen = o ~name:"callgen" ~default:false
 
   let safe_string = o ~name:"safestring" ~default:true
@@ -101,33 +104,56 @@ module Flag = struct
   let auto_link = o ~name:"auto-link" ~default:true
 
   let es6 = o ~name:"es6" ~default:false
+
+  let load_shapes_auto = o ~name:"load-shapes-auto" ~default:false
 end
 
 module Param = struct
-  let int default = default, int_of_string
+  let int default =
+    ( default
+    , int_of_string
+    , fun s ->
+        try
+          ignore (int_of_string s : int);
+          Ok ()
+        with _ -> Error "expecting an integer" )
 
   let enum : (string * 'a) list -> _ = function
-    | (_, v) :: _ as l -> v, fun x -> List.assoc x l
+    | (_, v) :: _ as l ->
+        ( v
+        , (fun x ->
+            match List.string_assoc x l with
+            | Some x -> x
+            | None -> assert false)
+        , fun x ->
+            if List.exists ~f:(fun (y, _) -> String.equal x y) l
+            then Ok ()
+            else
+              Error
+                (Printf.sprintf
+                   "expecting one of %s"
+                   (String.concat ~sep:", " (List.map l ~f:fst))) )
     | _ -> assert false
 
   let params : (string * _) list ref = ref []
 
-  let p ~name ~desc (default, convert) =
-    assert (not (List.mem_assoc name ~map:!params));
+  let p ~name ~desc (default, convert, valid) =
+    assert (Option.is_none (List.string_assoc name !params));
     let state = ref default in
     let set : string -> unit =
      fun v ->
       try state := convert v
-      with _ -> warn "Warning: malformed option %s=%s. IGNORE@." name v
+      with _ -> failwith (Printf.sprintf "malformed option %s=%s." name v)
     in
-    params := (name, (set, desc)) :: !params;
+    params := (name, (set, desc, valid)) :: !params;
     fun () -> !state
 
   let set s v =
-    try fst (List.assoc s !params) v
-    with Not_found -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
+    match List.string_assoc s !params with
+    | Some (f, _, _) -> f v
+    | None -> failwith (Printf.sprintf "The option named %S doesn't exist" s)
 
-  let all () = List.map !params ~f:(fun (n, (_, d)) -> n, d)
+  let all () = List.map !params ~f:(fun (n, (_, d, valid)) -> n, d, valid)
 
   (* V8 "optimize" switches with less than 128 case.
      60 seams to perform well. *)
@@ -135,7 +161,7 @@ module Param = struct
     p ~name:"switch_size" ~desc:"set the maximum number of case in a switch" (int 60)
 
   let inlining_limit =
-    p ~name:"inlining-limit" ~desc:"set the size limit for inlining" (int 200)
+    p ~name:"inlining-limit" ~desc:"set the size limit for inlining" (int 150)
 
   let tailcall_max_depth =
     p
@@ -153,12 +179,15 @@ module Param = struct
     | TcNone
     | TcTrampoline
 
+  let tc_equal (a : tc) b = Poly.equal a b
+
   (* | TcWhile *)
 
   let tc_default = TcTrampoline
 
   let _tc_all =
-    tc_default :: List.filter [ TcNone; TcTrampoline ] ~f:(Poly.( <> ) tc_default)
+    tc_default
+    :: List.filter [ TcNone; TcTrampoline ] ~f:(fun x -> not (tc_equal tc_default x))
 
   let tailcall_optim =
     p
@@ -195,3 +224,20 @@ let set_target (t : [ `JavaScript | `Wasm ]) =
   | `JavaScript -> Targetint.set_num_bits 32
   | `Wasm -> Targetint.set_num_bits 31);
   target_ := (t :> [ `JavaScript | `Wasm | `None ])
+
+type effects_backend =
+  [ `Disabled
+  | `Cps
+  | `Double_translation
+  | `Jspi
+  ]
+
+let effects_ : [< `None | effects_backend ] ref = ref `None
+
+let effects () =
+  match !effects_ with
+  | `None -> failwith "effects was not set"
+  | (`Jspi | `Cps | `Disabled | `Double_translation) as b -> b
+
+let set_effects_backend (backend : effects_backend) =
+  effects_ := (backend :> [ `None | effects_backend ])
