@@ -23,10 +23,10 @@ val f :
   -> exported_runtime:bool
   -> live_vars:Deadcode.variable_uses
   -> trampolined_calls:Effects.trampolined_calls
+  -> in_cps:Effects.in_cps
   -> should_export:bool
   -> warn_on_unhandled_effect:bool
   -> deadcode_sentinal:Code.Var.t
-  -> Parse_bytecode.Debug.t
   -> Javascript.program
 
 val init : unit -> unit
