@@ -31,31 +31,17 @@ let times = Debug.find "times"
 
 open Code
 
-(****)
+module VarPairTbl = Hashtbl.Make (struct
+  type t = Var.t * Var.t
 
-(* Compute the list of variables containing the return values of each
-   function *)
-let return_values p =
-  Code.fold_closures
-    p
-    (fun name_opt _ (pc, _) rets ->
-      match name_opt with
-      | None -> rets
-      | Some name ->
-          let s =
-            Code.traverse
-              { fold = fold_children }
-              (fun pc s ->
-                let block = Addr.Map.find pc p.blocks in
-                match block.branch with
-                | Return x -> Var.Set.add x s
-                | _ -> s)
-              pc
-              p.blocks
-              Var.Set.empty
-          in
-          Var.Map.add name s rets)
-    Var.Map.empty
+  let hash (a, b) = Var.idx a + Var.idx b
+
+  let equal (a, b) (c, d) = Var.equal a c && Var.equal b d
+end)
+
+let associated_list h x = try Var.Hashtbl.find h x with Not_found -> []
+
+let add_to_list h x v = Var.Hashtbl.replace h x (v :: associated_list h x)
 
 (****)
 
@@ -67,46 +53,56 @@ type def =
   | Phi of
       { known : Var.Set.t (* Known arguments *)
       ; others : bool (* Can there be other arguments *)
+      ; unit : bool (* Whether we are propagating unit (used for typing) *)
       }
 
-let undefined = Phi { known = Var.Set.empty; others = false }
+let undefined = Phi { known = Var.Set.empty; others = false; unit = false }
 
 let is_undefined d =
   match d with
   | Expr _ -> false
-  | Phi { known; others } -> Var.Set.is_empty known && not others
+  | Phi { known; others; unit } -> Var.Set.is_empty known && (not others) && not unit
 
 type escape_status =
   | Escape
   | Escape_constant (* Escapes but we know the value is not modified *)
   | No
 
+type mutable_fields =
+  | No_field
+  | Some_fields of IntSet.t
+  | All_fields
+
 type state =
   { vars : Var.ISet.t (* Set of all veriables considered *)
-  ; deps : Var.t Var.Tbl.DataSet.t Var.Tbl.t (* Dependency between variables *)
+  ; deps : Var.t list Var.Tbl.t (* Dependency between variables *)
   ; defs : def array (* Definition of each variable *)
   ; variable_may_escape : escape_status array
         (* Any value bound to this variable may escape *)
-  ; variable_possibly_mutable : Var.ISet.t
+  ; variable_mutable_fields : mutable_fields array
         (* Any value bound to this variable may be mutable *)
   ; may_escape : escape_status array (* This value may escape *)
-  ; possibly_mutable : Var.ISet.t (* This value may be mutable *)
+  ; mutable_fields : mutable_fields array (* This value may be mutable *)
   ; return_values : Var.Set.t Var.Map.t
         (* Set of variables holding return values of each function *)
-  ; known_cases : (Var.t, int list) Hashtbl.t
+  ; functions_from_returned_value : Var.t list Var.Hashtbl.t
+        (* Functions associated to each return value *)
+  ; known_cases : int list Var.Hashtbl.t
         (* Possible tags for a block after a [switch]. This is used to
            get a more precise approximation of the effect of a field
            access [Field] *)
-  ; applied_functions : (Var.t * Var.t, unit) Hashtbl.t
+  ; applied_functions : unit VarPairTbl.t
         (* Functions that have been already considered at a call site.
            This is to avoid repeated computations *)
+  ; function_call_sites : Var.t list Var.Hashtbl.t
+        (* Known call sites of each functions *)
   ; fast : bool
   }
 
 let add_var st x = Var.ISet.add st.vars x
 
 (* x depends on y *)
-let add_dep st x y = Var.Tbl.add_set st.deps y x
+let add_dep st x y = Var.Tbl.set st.deps y (x :: Var.Tbl.get st.deps y)
 
 let add_expr_def st x e =
   add_var st x;
@@ -120,13 +116,22 @@ let add_assign_def st x y =
   let idx = Var.idx x in
   match st.defs.(idx) with
   | Expr _ -> assert false
-  | Phi { known; others } -> st.defs.(idx) <- Phi { known = Var.Set.add y known; others }
+  | Phi { known; others; unit } ->
+      st.defs.(idx) <- Phi { known = Var.Set.add y known; others; unit }
+
+let add_unit_def st x =
+  add_var st x;
+  let idx = Var.idx x in
+  match st.defs.(idx) with
+  | Expr _ -> assert false
+  | Phi { known; others; _ } -> st.defs.(idx) <- Phi { known; others; unit = true }
 
 let add_param_def st x =
   add_var st x;
   let idx = Var.idx x in
   assert (is_undefined st.defs.(idx));
-  if st.fast then st.defs.(idx) <- Phi { known = Var.Set.empty; others = true }
+  if st.fast
+  then st.defs.(idx) <- Phi { known = Var.Set.empty; others = true; unit = false }
 
 let rec arg_deps st ?ignore params args =
   match params, args with
@@ -134,7 +139,7 @@ let rec arg_deps st ?ignore params args =
       (* This is to deal with the [else] clause of a conditional,
          where we know that the value of the tested variable is 0. *)
       (match ignore with
-      | Some y' when Var.equal y y' -> ()
+      | Some y' when Var.equal y y' -> add_unit_def st x
       | _ -> add_assign_def st x y);
       arg_deps st params args
   | [], [] -> ()
@@ -146,7 +151,14 @@ let cont_deps blocks st ?ignore (pc, args) =
 
 let do_escape st level x = st.variable_may_escape.(Var.idx x) <- level
 
-let possibly_mutable st x = Var.ISet.add st.variable_possibly_mutable x
+let possibly_mutable st x = st.variable_mutable_fields.(Var.idx x) <- All_fields
+
+let field_possibly_mutable st x n =
+  match st.variable_mutable_fields.(Var.idx x) with
+  | No_field -> st.variable_mutable_fields.(Var.idx x) <- Some_fields (IntSet.singleton n)
+  | Some_fields s ->
+      st.variable_mutable_fields.(Var.idx x) <- Some_fields (IntSet.add n s)
+  | All_fields -> ()
 
 let expr_deps blocks st x e =
   match e with
@@ -221,14 +233,14 @@ let expr_deps blocks st x e =
          dependencies right now. This speeds up the analysis
          significantly. *)
       match st.defs.(Var.idx f) with
-      | Expr (Closure (params, _)) when List.length args = List.length params ->
-          Hashtbl.add st.applied_functions (x, f) ();
+      | Expr (Closure (params, _, _)) when List.compare_lengths args params = 0 ->
+          VarPairTbl.add st.applied_functions (x, f) ();
+          add_to_list st.function_call_sites f x;
           if st.fast
           then List.iter ~f:(fun a -> do_escape st Escape a) args
-          else List.iter2 ~f:(fun p a -> add_assign_def st p a) params args;
-          Var.Set.iter (fun y -> add_dep st x y) (Var.Map.find f st.return_values)
+          else List.iter2 ~f:(fun p a -> add_assign_def st p a) params args
       | _ -> ())
-  | Closure (l, cont) ->
+  | Closure (l, cont, _) ->
       List.iter l ~f:(fun x -> add_param_def st x);
       cont_deps blocks st cont
   | Field (y, _, _) -> add_dep st x y
@@ -251,7 +263,10 @@ let program_deps st { start; blocks; _ } =
               add_expr_def st x e;
               expr_deps blocks st x e
           | Assign (x, y) -> add_assign_def st x y
-          | Set_field (x, _, _, y) | Array_set (x, _, y) ->
+          | Set_field (x, n, _, y) ->
+              field_possibly_mutable st x n;
+              do_escape st Escape y
+          | Array_set (x, _, y) ->
               possibly_mutable st x;
               do_escape st Escape y
           | Event _ | Offset_ref _ -> ());
@@ -272,27 +287,28 @@ let program_deps st { start; blocks; _ } =
                increasing order *)
             match st.defs.(Code.Var.idx x) with
             | Expr (Prim (Extern "%direct_obj_tag", [ Pv b ])) ->
-                let h = Hashtbl.create 16 in
+                let h = Addr.Hashtbl.create 16 in
                 Array.iteri a1 ~f:(fun i (pc, _) ->
-                    Hashtbl.replace
+                    Addr.Hashtbl.replace
                       h
                       pc
-                      (i :: (try Hashtbl.find h pc with Not_found -> [])));
-                Hashtbl.iter
+                      (i :: (try Addr.Hashtbl.find h pc with Not_found -> [])));
+                Addr.Hashtbl.iter
                   (fun pc tags ->
                     let block = Addr.Map.find pc blocks in
                     List.iter
                       ~f:(fun i ->
                         match i with
                         | Let (y, Field (x', _, _)) when Var.equal b x' ->
-                            Hashtbl.add st.known_cases y tags
+                            Var.Hashtbl.add st.known_cases y tags
                         | _ -> ())
                       block.body)
                   h
             | Expr _ | Phi _ -> ())
       | Pushtrap (cont, x, cont_h) ->
           add_var st x;
-          st.defs.(Var.idx x) <- Phi { known = Var.Set.empty; others = true };
+          st.defs.(Var.idx x) <-
+            Phi { known = Var.Set.empty; others = true; unit = false };
           cont_deps blocks st cont_h;
           cont_deps blocks st cont)
     blocks
@@ -344,14 +360,15 @@ module Domain = struct
           Array.iter ~f:(fun y -> variable_escape ~update ~st ~approx s y) a;
           match s, mut with
           | Escape, Maybe_mutable ->
-              Var.ISet.add st.possibly_mutable x;
+              st.mutable_fields.(Var.idx x) <- All_fields;
               update ~children:true x
           | (Escape_constant | No), _ | Escape, Immutable -> ())
-      | Expr (Closure (params, _)) ->
+      | Expr (Closure (params, _, _)) ->
           List.iter
             ~f:(fun y ->
               (match st.defs.(Var.idx y) with
-              | Phi { known; _ } -> st.defs.(Var.idx y) <- Phi { known; others = true }
+              | Phi { known; _ } ->
+                  st.defs.(Var.idx y) <- Phi { known; others = true; unit = false }
               | Expr _ -> assert false);
               update ~children:false y)
             params;
@@ -389,18 +406,28 @@ module Domain = struct
       s
       (if o then others else bot)
 
-  let mark_mutable ~update ~st a =
+  let mark_mutable ~update ~st a mutable_fields =
     match a with
     | Top -> ()
     | Values { known; _ } ->
         Var.Set.iter
           (fun x ->
             match st.defs.(Var.idx x) with
-            | Expr (Block (_, _, _, Maybe_mutable)) ->
-                if not (Var.ISet.mem st.possibly_mutable x)
-                then (
-                  Var.ISet.add st.possibly_mutable x;
-                  update ~children:true x)
+            | Expr (Block (_, _, _, Maybe_mutable)) -> (
+                match st.mutable_fields.(Var.idx x), mutable_fields with
+                | _, No_field -> ()
+                | No_field, _ ->
+                    st.mutable_fields.(Var.idx x) <- mutable_fields;
+                    update ~children:true x
+                | Some_fields s, Some_fields s' ->
+                    if IntSet.exists (fun i -> not (IntSet.mem i s)) s'
+                    then (
+                      st.mutable_fields.(Var.idx x) <- Some_fields (IntSet.union s s');
+                      update ~children:true x)
+                | Some_fields _, All_fields ->
+                    st.mutable_fields.(Var.idx x) <- All_fields;
+                    update ~children:true x
+                | All_fields, _ -> ())
             | Expr (Block (_, _, _, Immutable)) | Expr (Closure _) -> ()
             | Phi _ | Expr _ -> assert false)
           known
@@ -408,7 +435,7 @@ end
 
 let propagate st ~update approx x =
   match st.defs.(Var.idx x) with
-  | Phi { known; others } ->
+  | Phi { known; others; _ } ->
       Domain.join_set ~update ~st ~approx ~others (fun y -> Var.Tbl.get approx y) known
   | Expr e -> (
       match e with
@@ -420,7 +447,7 @@ let propagate st ~update approx x =
           match Var.Tbl.get approx y with
           | Values { known; others } ->
               let tags =
-                try Some (Hashtbl.find st.known_cases x) with Not_found -> None
+                try Some (Var.Hashtbl.find st.known_cases x) with Not_found -> None
               in
               Domain.join_set
                 ~others
@@ -433,14 +460,19 @@ let propagate st ~update approx x =
                     when n < Array.length a
                          &&
                          match tags with
-                         | Some tags -> List.memq t ~set:tags
+                         | Some tags -> List.mem ~eq:Int.equal t tags
                          | None -> true ->
                       let t = a.(n) in
+                      let m =
+                        match st.mutable_fields.(Var.idx z) with
+                        | No_field -> false
+                        | Some_fields s -> IntSet.mem n s
+                        | All_fields -> true
+                      in
+                      if not m then add_dep st x z;
                       add_dep st x t;
                       let a = Var.Tbl.get approx t in
-                      if Var.ISet.mem st.possibly_mutable z
-                      then Domain.join ~update ~st ~approx Domain.others a
-                      else a
+                      if m then Domain.join ~update ~st ~approx Domain.others a else a
                   | Expr (Block _ | Closure _) -> Domain.bot
                   | Phi _ | Expr _ -> assert false)
                 known
@@ -464,6 +496,12 @@ let propagate st ~update approx x =
                   (fun z ->
                     match st.defs.(Var.idx z) with
                     | Expr (Block (_, lst, _, _)) ->
+                        let m =
+                          match st.mutable_fields.(Var.idx z) with
+                          | No_field -> false
+                          | Some_fields _ | All_fields -> true
+                        in
+                        if not m then add_dep st x z;
                         Array.iter ~f:(fun t -> add_dep st x t) lst;
                         let a =
                           Array.fold_left
@@ -472,9 +510,7 @@ let propagate st ~update approx x =
                             ~init:Domain.bot
                             lst
                         in
-                        if Var.ISet.mem st.possibly_mutable z
-                        then Domain.join ~update ~st ~approx Domain.others a
-                        else a
+                        if m then Domain.join ~update ~st ~approx Domain.others a else a
                     | Expr (Closure _) -> Domain.bot
                     | Phi _ | Expr _ -> assert false)
                   known
@@ -501,11 +537,12 @@ let propagate st ~update approx x =
                 ~others
                 (fun g ->
                   match st.defs.(Var.idx g) with
-                  | Expr (Closure (params, _)) when List.length args = List.length params
-                    ->
-                      if not (Hashtbl.mem st.applied_functions (x, g))
+                  | Expr (Closure (params, _, _))
+                    when List.compare_lengths args params = 0 ->
+                      if not (VarPairTbl.mem st.applied_functions (x, g))
                       then (
-                        Hashtbl.add st.applied_functions (x, g) ();
+                        VarPairTbl.add st.applied_functions (x, g) ();
+                        add_to_list st.function_call_sites g x;
                         if st.fast
                         then
                           List.iter
@@ -518,18 +555,15 @@ let propagate st ~update approx x =
                               add_assign_def st p a;
                               update ~children:false p)
                             params
-                            args;
-                        Var.Set.iter
-                          (fun y -> add_dep st x y)
-                          (Var.Map.find g st.return_values));
+                            args);
                       Domain.join_set
                         ~update
                         ~st
                         ~approx
                         (fun y -> Var.Tbl.get approx y)
                         (Var.Map.find g st.return_values)
-                  | Expr (Closure (_, _)) ->
-                      (* The funciton is partially applied or over applied *)
+                  | Expr (Closure (_, _, _)) ->
+                      (* The function is partially applied or over applied *)
                       List.iter
                         ~f:(fun y -> Domain.variable_escape ~update ~st ~approx Escape y)
                         args;
@@ -560,22 +594,66 @@ let propagate st ~update approx x =
       (match st.variable_may_escape.(Var.idx x) with
       | (Escape | Escape_constant) as s -> Domain.approx_escape ~update ~st ~approx s res
       | No -> ());
-      if Var.ISet.mem st.variable_possibly_mutable x
-      then Domain.mark_mutable ~update ~st res;
+      (match st.variable_mutable_fields.(Var.idx x) with
+      | No_field -> ()
+      | (Some_fields _ | All_fields) as s -> Domain.mark_mutable ~update ~st res s);
       res
   | Top -> Top
 
 module G = Dgraph.Make_Imperative (Var) (Var.ISet) (Var.Tbl)
 module Solver = G.Solver (Domain)
 
+let print_approx st f a =
+  match a with
+  | Top -> Format.fprintf f "top"
+  | Values { known; others } ->
+      Format.fprintf
+        f
+        "{%a/%b}"
+        (Format.pp_print_list
+           ~pp_sep:(fun f () -> Format.fprintf f ", ")
+           (fun f x ->
+             Format.fprintf
+               f
+               "%a(%s)"
+               Var.print
+               x
+               (match st.defs.(Var.idx x) with
+               | Expr (Closure _) -> "C"
+               | Expr (Block _) -> (
+                   "B"
+                   ^
+                   match st.may_escape.(Var.idx x) with
+                   | Escape -> "X"
+                   | _ -> "")
+               | _ -> "O")))
+        (Var.Set.elements known)
+        others
+
 let solver st =
   let g =
     { G.domain = st.vars
     ; G.iter_children =
-        (fun f x -> Var.Tbl.DataSet.iter (fun k -> f k) (Var.Tbl.get st.deps x))
+        (fun f x ->
+          List.iter ~f (Var.Tbl.get st.deps x);
+          List.iter
+            ~f:(fun g -> List.iter ~f (associated_list st.function_call_sites g))
+            (associated_list st.functions_from_returned_value x))
     }
   in
-  Solver.f' () g (propagate st)
+  let res = Solver.f' () g (propagate st) in
+  if debug ()
+  then
+    Solver.check g res (propagate st) (fun x a b ->
+        Format.eprintf
+          "Incorrect value: %a: %a -> %a@."
+          Var.print
+          x
+          (print_approx st)
+          a
+          (print_approx st)
+          b);
+  res
 
 (****)
 
@@ -593,34 +671,39 @@ let f ~fast p =
   let rets = return_values p in
   let nv = Var.count () in
   let vars = Var.ISet.empty () in
-  let deps = Var.Tbl.make_set () in
+  let deps = Var.Tbl.make () [] in
   let defs = Array.make nv undefined in
   let variable_may_escape = Array.make nv No in
-  let variable_possibly_mutable = Var.ISet.empty () in
+  let variable_mutable_fields = Array.make nv No_field in
   let may_escape = Array.make nv No in
-  let possibly_mutable = Var.ISet.empty () in
+  let mutable_fields = Array.make nv No_field in
+  let functions_from_returned_value = Var.Hashtbl.create 128 in
+  Var.Map.iter
+    (fun f s -> Var.Set.iter (fun x -> add_to_list functions_from_returned_value x f) s)
+    rets;
   let st =
     { vars
     ; deps
     ; defs
     ; return_values = rets
+    ; functions_from_returned_value
     ; variable_may_escape
-    ; variable_possibly_mutable
+    ; variable_mutable_fields
     ; may_escape
-    ; possibly_mutable
-    ; known_cases = Hashtbl.create 16
-    ; applied_functions = Hashtbl.create 16
+    ; mutable_fields
+    ; known_cases = Var.Hashtbl.create 16
+    ; applied_functions = VarPairTbl.create 16
     ; fast
+    ; function_call_sites = Var.Hashtbl.create 128
     }
   in
   program_deps st p;
   if times ()
-  then Format.eprintf "      global flow analysis (initialize): %a@." Timer.print t1;
+  then Format.eprintf "    global flow analysis (initialize): %a@." Timer.print t1;
   let t2 = Timer.make () in
   let approximation = solver st in
-  if times ()
-  then Format.eprintf "      global flow analysis (solve): %a@." Timer.print t2;
-  if times () then Format.eprintf "    global flow analysis: %a@." Timer.print t;
+  if times () then Format.eprintf "    global flow analysis (solve): %a@." Timer.print t2;
+  if times () then Format.eprintf "  global flow analysis: %a@." Timer.print t;
   if debug ()
   then
     Var.ISet.iter
@@ -635,31 +718,29 @@ let f ~fast p =
             (fun f a ->
               match a with
               | Top -> Format.fprintf f "top"
-              | Values { known; others } ->
+              | Values _ ->
+                  let print_mutable_fields f s =
+                    match s with
+                    | No_field -> Format.fprintf f "no"
+                    | Some_fields s ->
+                        Format.fprintf
+                          f
+                          "{%a}"
+                          (Format.pp_print_list
+                             ~pp_sep:(fun f () -> Format.fprintf f ", ")
+                             (fun f i -> Format.fprintf f "%d" i))
+                          (IntSet.elements s)
+                    | All_fields -> Format.fprintf f "yes"
+                  in
                   Format.fprintf
                     f
-                    "{%a/%b} mut:%b vmut:%b vesc:%s esc:%s"
-                    (Format.pp_print_list
-                       ~pp_sep:(fun f () -> Format.fprintf f ", ")
-                       (fun f x ->
-                         Format.fprintf
-                           f
-                           "%a(%s)"
-                           Var.print
-                           x
-                           (match st.defs.(Var.idx x) with
-                           | Expr (Closure _) -> "C"
-                           | Expr (Block _) -> (
-                               "B"
-                               ^
-                               match st.may_escape.(Var.idx x) with
-                               | Escape -> "X"
-                               | _ -> "")
-                           | _ -> "O")))
-                    (Var.Set.elements known)
-                    others
-                    (Var.ISet.mem st.possibly_mutable x)
-                    (Var.ISet.mem st.variable_possibly_mutable x)
+                    "%a mut:%a vmut:%a vesc:%s esc:%s"
+                    (print_approx st)
+                    a
+                    print_mutable_fields
+                    st.mutable_fields.(Var.idx x)
+                    print_mutable_fields
+                    st.variable_mutable_fields.(Var.idx x)
                     (match st.variable_may_escape.(Var.idx x) with
                     | Escape -> "Y"
                     | Escape_constant -> "y"
@@ -673,14 +754,18 @@ let f ~fast p =
   let info_variable_may_escape = variable_may_escape in
   let info_may_escape = Var.ISet.empty () in
   Array.iteri
-    ~f:(fun i s -> if Poly.(s <> No) then Var.ISet.add info_may_escape (Var.of_idx i))
+    ~f:(fun i s ->
+      match s with
+      | Escape_constant | Escape -> Var.ISet.add info_may_escape (Var.of_idx i)
+      | No -> ())
     may_escape;
-  { info_defs = defs
-  ; info_approximation = approximation
-  ; info_variable_may_escape
-  ; info_may_escape
-  ; info_return_vals = rets
-  }
+  ( st
+  , { info_defs = defs
+    ; info_approximation = approximation
+    ; info_variable_may_escape
+    ; info_may_escape
+    ; info_return_vals = rets
+    } )
 
 let exact_call info f n =
   match Var.Tbl.get info.info_approximation f with
@@ -689,11 +774,40 @@ let exact_call info f n =
       Var.Set.for_all
         (fun g ->
           match info.info_defs.(Var.idx g) with
-          | Expr (Closure (params, _)) -> List.length params = n
+          | Expr (Closure (params, _, _)) -> List.compare_length_with params ~len:n = 0
           | Expr (Block _) -> true
           | Expr _ | Phi _ -> assert false)
         known
 
+let get_unique_closure info f =
+  (* The specialize pass can create knew functions *)
+  if Var.idx f >= Var.Tbl.length info.info_approximation
+  then None
+  else
+    match Var.Tbl.get info.info_approximation f with
+    | Top | Values { others = true; _ } -> None
+    | Values { known; others = false } -> (
+        match
+          Var.Set.fold
+            (fun g acc ->
+              match info.info_defs.(Var.idx g) with
+              | Expr (Closure _) -> (
+                  match acc with
+                  | None -> Some (Some g)
+                  | Some (Some _) -> Some None
+                  | Some None -> acc)
+              | Expr (Block _) -> acc
+              | Expr _ | Phi _ -> assert false)
+            known
+            None
+        with
+        | None -> None
+        | Some kind -> kind)
+
+let update_def info x expr =
+  let idx = Code.Var.idx x in
+  info.info_defs.(idx) <- Expr expr
+
 let function_arity info f =
   match Var.Tbl.get info.info_approximation f with
   | Top | Values { others = true; _ } -> None
@@ -702,7 +816,7 @@ let function_arity info f =
         Var.Set.fold
           (fun g acc ->
             match info.info_defs.(Var.idx g) with
-            | Expr (Closure (params, _)) -> (
+            | Expr (Closure (params, _, _)) -> (
                 let n = List.length params in
                 match acc with
                 | None -> Some (Some n)
