@@ -34,6 +34,8 @@ module Num : sig
 
   val to_targetint : t -> Targetint.t
 
+  val hash : t -> int
+
   (** Predicates *)
 
   val is_zero : t -> bool
@@ -42,6 +44,8 @@ module Num : sig
 
   val is_neg : t -> bool
 
+  val equal : t -> t -> bool
+
   (** Arithmetic *)
 
   val add : t -> t -> t
@@ -55,11 +59,12 @@ end = struct
   let to_string s = s
 
   let to_targetint s =
-    if String.is_prefix s ~prefix:"0"
-       && String.length s > 1
-       && String.for_all s ~f:(function
-              | '0' .. '7' -> true
-              | _ -> false)
+    if
+      String.starts_with s ~prefix:"0"
+      && String.length s > 1
+      && String.for_all s ~f:(function
+           | '0' .. '7' -> true
+           | _ -> false)
     then (* legacy octal notation *)
       Targetint.of_string_exn ("0o" ^ s)
     else Targetint.of_string_exn s
@@ -133,6 +138,10 @@ end = struct
 
   let is_neg s = Char.equal s.[0] '-'
 
+  let equal a b = String.equal a b
+
+  let hash a = String.hash a
+
   let neg s =
     match String.drop_prefix s ~prefix:"-" with
     | None -> "-" ^ s
@@ -149,6 +158,12 @@ module Label = struct
   let fresh () = L (Code.Var.fresh ())
 
   let of_string s = S s
+
+  let equal a b =
+    match a, b with
+    | L x, L y -> Code.Var.equal x y
+    | S s, S t -> Utf8_string.equal s t
+    | L _, S _ | S _, L _ -> false
 end
 
 type location =
@@ -508,6 +523,8 @@ let ident ?(loc = N) ?var (Utf8_string.Utf8 n as name) =
   if not (is_ident' name) then failwith (Printf.sprintf "%s not a valid ident" n);
   S { name; var; loc }
 
+let ident_equal (a : ident) b = Poly.equal a b
+
 let param' id = BindingIdent id, None
 
 let param ?loc ?var name = param' (ident ?loc ?var name)
@@ -530,16 +547,16 @@ and bound_idents_of_pattern p =
   match p with
   | ObjectBinding { list; rest } -> (
       List.concat_map list ~f:(function
-          | Prop_ident (Prop_and_ident i, _) -> [ i ]
-          | Prop_binding (_, e) -> bound_idents_of_element e)
+        | Prop_ident (Prop_and_ident i, _) -> [ i ]
+        | Prop_binding (_, e) -> bound_idents_of_element e)
       @
       match rest with
       | None -> []
       | Some x -> [ x ])
   | ArrayBinding { list; rest } -> (
       List.concat_map list ~f:(function
-          | None -> []
-          | Some e -> bound_idents_of_element e)
+        | None -> []
+        | Some e -> bound_idents_of_element e)
       @
       match rest with
       | None -> []
@@ -587,30 +604,29 @@ let rec assignment_target_of_expr' x =
   | EObj l ->
       let list =
         List.map l ~f:(function
-            | Property (PNI n, EVar (S { name = n'; _ } as id))
-              when Utf8_string.equal n n' -> TargetPropertyId (Prop_and_ident id, None)
-            | Property (n, e) ->
-                let e, i =
-                  match e with
-                  | EBin (Eq, e, i) -> e, Some (i, N)
-                  | _ -> e, None
-                in
-                TargetProperty (n, assignment_target_of_expr' e, i)
-            | CoverInitializedName (_, i, (e, loc)) ->
-                TargetPropertyId
-                  (Prop_and_ident i, Some (assignment_target_of_expr' e, loc))
-            | PropertySpread e -> TargetPropertySpread (assignment_target_of_expr' e)
-            | PropertyMethod (n, m) -> TargetPropertyMethod (n, m))
+          | Property (PNI n, EVar (S { name = n'; _ } as id)) when Utf8_string.equal n n'
+            -> TargetPropertyId (Prop_and_ident id, None)
+          | Property (n, e) ->
+              let e, i =
+                match e with
+                | EBin (Eq, e, i) -> e, Some (i, N)
+                | _ -> e, None
+              in
+              TargetProperty (n, assignment_target_of_expr' e, i)
+          | CoverInitializedName (_, i, (e, loc)) ->
+              TargetPropertyId (Prop_and_ident i, Some (assignment_target_of_expr' e, loc))
+          | PropertySpread e -> TargetPropertySpread (assignment_target_of_expr' e)
+          | PropertyMethod (n, m) -> TargetPropertyMethod (n, m))
       in
       EAssignTarget (ObjectTarget list)
   | EArr l ->
       let list =
         List.map l ~f:(function
-            | ElementHole -> TargetElementHole
-            | Element (EVar x) -> TargetElementId (x, None)
-            | Element (EBin (Eq, EVar x, rhs)) -> TargetElementId (x, Some (rhs, N))
-            | Element e -> TargetElement (assignment_target_of_expr' e)
-            | ElementSpread e -> TargetElementSpread (assignment_target_of_expr' e))
+          | ElementHole -> TargetElementHole
+          | Element (EVar x) -> TargetElementId (x, None)
+          | Element (EBin (Eq, EVar x, rhs)) -> TargetElementId (x, Some (rhs, N))
+          | Element e -> TargetElement (assignment_target_of_expr' e)
+          | ElementSpread e -> TargetElementSpread (assignment_target_of_expr' e))
       in
       EAssignTarget (ArrayTarget list)
   | _ -> x
@@ -619,3 +635,5 @@ and assignment_target_of_expr op x =
   match op with
   | None | Some Eq -> assignment_target_of_expr' x
   | _ -> x
+
+let location_equal (a : location) b = Poly.equal a b
