@@ -35,6 +35,8 @@ module Num : sig
 
   val to_targetint : t -> Targetint.t
 
+  val hash : t -> int
+
   (** Predicates *)
 
   val is_zero : t -> bool
@@ -43,6 +45,8 @@ module Num : sig
 
   val is_neg : t -> bool
 
+  val equal : t -> t -> bool
+
   (** Arithmetic *)
 
   val add : t -> t -> t
@@ -58,6 +62,8 @@ module Label : sig
   val fresh : unit -> t
 
   val of_string : Utf8_string.t -> t
+
+  val equal : t -> t -> bool
 end
 
 type location =
@@ -413,6 +419,8 @@ val is_ident : string -> bool
 
 val is_ident' : Utf8_string.t -> bool
 
+val ident_equal : ident -> ident -> bool
+
 val ident : ?loc:location -> ?var:Code.Var.t -> identifier -> ident
 
 val param : ?loc:location -> ?var:Code.Var.t -> identifier -> formal_parameter
@@ -449,3 +457,5 @@ val early_error : ?reason:string -> Parse_info.t -> early_error
 val fun_ : ident list -> statement_list -> location -> function_declaration
 
 val assignment_target_of_expr : binop option -> expression -> expression
+
+val location_equal : location -> location -> bool
