@@ -34,7 +34,7 @@ module type Strategy = sig
 
   val record_block : t -> Js_traverse.t -> Js_traverse.block -> unit
 
-  val allocate_variables : t -> count:int Javascript.IdentMap.t -> string array
+  val allocate_variables : t -> count:int array -> string array
 end
 
 module Min : Strategy = struct
@@ -83,9 +83,11 @@ while compiling the OCaml toplevel:
   type alloc =
     { mutable first_free : int
     ; used : BitSet.t
+    ; existing_names : StringSet.t
     }
 
-  let make_alloc_table () = { first_free = 0; used = BitSet.create () }
+  let make_alloc_table existing_names =
+    { first_free = 0; used = BitSet.create (); existing_names }
 
   let next_available a i = BitSet.next_free a.used (max i a.first_free)
 
@@ -93,7 +95,9 @@ while compiling the OCaml toplevel:
     BitSet.set a.used i;
     if a.first_free = i then a.first_free <- BitSet.next_free a.used a.first_free
 
-  let is_available l i = List.for_all l ~f:(fun a -> not (BitSet.mem a.used i))
+  let is_available l name i =
+    List.for_all l ~f:(fun a ->
+        (not (BitSet.mem a.used i)) && not (StringSet.mem name a.existing_names))
 
   let first_available l =
     let rec find_rec n l =
@@ -105,19 +109,19 @@ while compiling the OCaml toplevel:
   let mark_allocated l i = List.iter l ~f:(fun a -> allocate a i)
 
   type t =
-    { constr : alloc list array
-    ; (* Constraints on variables *)
-      mutable parameters : Var.t list array
-    ; (* Function parameters *)
-      mutable constraints : S.t list
+    { constr : alloc list array (* Constraints on variables *)
+    ; mutable parameters : Var.t list array (* Function parameters *)
+    ; printer : Var_printer.t
     }
 
-  (* For debugging *)
-
-  let create nv = { constr = Array.make nv []; parameters = [| [] |]; constraints = [] }
+  let create nv =
+    { constr = Array.make nv []
+    ; parameters = [| [] |]
+    ; printer = Var_printer.create Var_printer.Alphabet.javascript
+    }
 
   let allocate_variables t ~count =
-    let weight v = try IdentMap.find (V (Var.of_idx v)) count with Not_found -> 0 in
+    let weight v = count.(v) in
     let constr = t.constr in
     let len = Array.length constr in
     let idx = Array.make len 0 in
@@ -125,7 +129,7 @@ while compiling the OCaml toplevel:
       idx.(i) <- i
     done;
     Array.stable_sort idx ~cmp:(fun i j -> compare (weight j) (weight i));
-    let name = Array.make len "" in
+    let names = Array.make len "" in
     let n0 = ref 0 in
     let n1 = ref 0 in
     let n2 = ref 0 in
@@ -138,22 +142,19 @@ while compiling the OCaml toplevel:
         n2 := !n2 + weight i);
       n3 := !n3 + weight i
     in
-    let nm ~origin n =
-      let n = Var.to_string ~origin:(Var.of_idx origin) (Var.of_idx n) in
-      name.(origin) <- n
-    in
     let total = ref 0 in
     let bad = ref 0 in
     for i = 0 to Array.length t.parameters - 1 do
+      let name = Var_printer.to_string t.printer i in
       List.iter
         (List.rev t.parameters.(i))
         ~f:(fun x ->
           incr total;
           let idx = Var.idx x in
           let l = constr.(idx) in
-          if is_available l i
+          if is_available l name i
           then (
-            nm ~origin:idx i;
+            names.(idx) <- name;
             mark_allocated l i;
             stats idx i)
           else incr bad)
@@ -165,29 +166,49 @@ while compiling the OCaml toplevel:
         (!total - !bad)
         !total;
     for i = 0 to len - 1 do
-      let l = constr.(idx.(i)) in
-      if (not (List.is_empty l)) && String.length name.(idx.(i)) = 0
-      then (
-        let n = first_available l in
-        let idx = idx.(i) in
-        nm ~origin:idx n;
-        mark_allocated l n;
-        stats idx n);
-      if List.is_empty l then assert (weight idx.(i) = 0)
+      let idx = idx.(i) in
+      let l = constr.(idx) in
+      (if (not (List.is_empty l)) && String.length names.(idx) = 0
+       then
+         let rec loop () =
+           let n = first_available l in
+           let name = Var_printer.to_string t.printer n in
+           if List.for_all l ~f:(fun a -> not (StringSet.mem name a.existing_names))
+           then (
+             names.(idx) <- name;
+             mark_allocated l n;
+             stats idx n)
+           else (
+             mark_allocated l n;
+             loop ())
+         in
+         loop ());
+      if List.is_empty l then assert (weight idx = 0)
     done;
     if debug_shortvar ()
     then (
       Format.eprintf "short variable count: %d/%d@." !n1 !n0;
       Format.eprintf "short variable occurrences: %d/%d@." !n2 !n3);
-    name
+    names
 
   let add_constraints global u ~offset (params : ident list) =
     let constr = global.constr in
-    let c = make_alloc_table () in
-    S.iter
-      (fun v ->
-        let i = Var.idx v in
-        constr.(i) <- c :: constr.(i))
+    let existing_names =
+      Javascript.IdentSet.fold
+        (fun x acc ->
+          match x with
+          | V _ -> acc
+          | S { name = Utf8 name; _ } -> StringSet.add name acc)
+        u
+        StringSet.empty
+    in
+    let c = make_alloc_table existing_names in
+    Javascript.IdentSet.iter
+      (function
+        | S _ -> ()
+        | V v ->
+            let i = Var.idx v in
+            constr.(i) <- c :: constr.(i))
       u;
     let params = Array.of_list params in
     let len = Array.length params in
@@ -206,8 +227,7 @@ while compiling the OCaml toplevel:
       match params.(i) with
       | V x -> global.parameters.(i + offset) <- x :: global.parameters.(i + offset)
       | _ -> ()
-    done;
-    global.constraints <- u :: global.constraints
+    done
 
   let record_block state scope (block : Js_traverse.block) =
     let all =
@@ -223,15 +243,6 @@ while compiling the OCaml toplevel:
           let ids = bound_idents_of_binding p in
           List.fold_left ids ~init:all ~f:(fun all i -> Javascript.IdentSet.add i all)
     in
-    let all =
-      Javascript.IdentSet.fold
-        (fun x acc ->
-          match x with
-          | V i -> S.add i acc
-          | S _ -> acc)
-        all
-        S.empty
-    in
     match block with
     | Normal -> add_constraints state all ~offset:0 []
     | Catch (v, _) -> add_constraints state all ~offset:5 (bound_idents_of_binding v)
@@ -249,7 +260,7 @@ module Preserve : Strategy = struct
 
   type t =
     { size : int
-    ; mutable scopes : (S.t * Js_traverse.t) list
+    ; mutable scopes : (S.t * Javascript.IdentSet.t) list
     }
 
   let create size = { size; scopes = [] }
@@ -265,6 +276,11 @@ module Preserve : Strategy = struct
           Javascript.IdentSet.elements
             (IdentSet.union scope.Js_traverse.def_var scope.Js_traverse.def_local)
     in
+    let all =
+      Javascript.IdentSet.union
+        (Javascript.IdentSet.union scope.Js_traverse.def_var scope.Js_traverse.def_local)
+        scope.Js_traverse.use
+    in
     let defs =
       List.fold_left
         ~init:S.empty
@@ -274,13 +290,23 @@ module Preserve : Strategy = struct
           | S _ -> acc)
         defs
     in
+    t.scopes <- (defs, all) :: t.scopes
 
-    t.scopes <- (defs, scope) :: t.scopes
+  let uniq_var = Debug.find "var"
 
-  let allocate_variables t ~count:_ =
+  let allocate_variables t ~count =
     let names = Array.make t.size "" in
-    List.iter t.scopes ~f:(fun (defs, state) ->
-        let assigned =
+    let uniq_var = uniq_var () in
+    let unamed = ref 0 in
+    let create_unamed =
+      if uniq_var
+      then
+        fun i ->
+          "_" ^ Var_printer.Alphabet.to_string Var_printer.Alphabet.javascript i ^ "_"
+      else fun i -> Var_printer.Alphabet.to_string Var_printer.Alphabet.javascript i
+    in
+    List.iter t.scopes ~f:(fun (defs, all) ->
+        let reserved =
           IdentSet.fold
             (fun var acc ->
               match var with
@@ -288,35 +314,61 @@ module Preserve : Strategy = struct
               | V v ->
                   let name = names.(Var.idx v) in
                   if not (String.is_empty name) then StringSet.add name acc else acc)
-            (IdentSet.union
-               state.Js_traverse.use
-               (IdentSet.union state.Js_traverse.def_var state.Js_traverse.def_local))
-            Reserved.keyword
+            all
+            StringSet.empty
         in
-        let _assigned =
+        (* We first allocate variable that have a name *)
+        let reserved, others =
           S.fold
-            (fun var assigned ->
+            (fun var (reserved, others) ->
               assert (String.is_empty names.(Var.idx var));
-              let name =
-                match Var.get_name var with
-                | Some expected_name ->
-                    assert (not (String.is_empty expected_name));
-                    if not (StringSet.mem expected_name assigned)
+              match Var.get_name var with
+              | Some expected_name ->
+                  assert (not (String.is_empty expected_name));
+                  assert (not (StringSet.mem expected_name Reserved.keyword));
+                  let name =
+                    if not (StringSet.mem expected_name reserved)
                     then expected_name
                     else
+                      let expected_name =
+                        if Char.equal expected_name.[String.length expected_name - 1] '$'
+                        then expected_name
+                        else expected_name ^ "$"
+                      in
                       let i = ref 0 in
                       while
-                        StringSet.mem (Printf.sprintf "%s$%d" expected_name !i) assigned
+                        StringSet.mem (Printf.sprintf "%s%d" expected_name !i) reserved
                       do
                         incr i
                       done;
-                      Printf.sprintf "%s$%d" expected_name !i
-                | None -> Var.to_string var
-              in
-              names.(Var.idx var) <- name;
-              StringSet.add name assigned)
+                      Printf.sprintf "%s%d" expected_name !i
+                  in
+                  names.(Var.idx var) <- name;
+                  StringSet.add name reserved, others
+              | None -> reserved, var :: others)
             defs
-            assigned
+            (reserved, [])
+        in
+        (* We then allocate variables without names *)
+        if not uniq_var then unamed := 0;
+        let _reserved =
+          let weight v = count.(Var.idx v) in
+          List.stable_sort others ~cmp:(fun i j ->
+              match compare (weight j) (weight i) with
+              | 0 -> Var.compare i j
+              | c -> c)
+          |> List.fold_left ~init:reserved ~f:(fun reserved var ->
+                 let name =
+                   while
+                     let name = create_unamed !unamed in
+                     StringSet.mem name reserved || StringSet.mem name Reserved.keyword
+                   do
+                     incr unamed
+                   done;
+                   create_unamed !unamed
+                 in
+                 names.(Var.idx var) <- name;
+                 StringSet.add name reserved)
         in
         ());
     names
@@ -331,7 +383,7 @@ class traverse record_block =
       super#record_block b
   end
 
-class traverse_labels h =
+class traverse_idents_and_labels ~idents ~labels =
   object
     inherit Js_traverse.iter as super
 
@@ -345,14 +397,23 @@ class traverse_labels h =
       function
       | Labelled_statement (L l, (s, _)) ->
           let m = {<ldepth = ldepth + 1>} in
-          Hashtbl.add h l ldepth;
+          Var.Hashtbl.add labels l ldepth;
           m#statement s
       | s -> super#statement s
+
+    method ident i =
+      match i with
+      | S _ -> ()
+      | V v ->
+          let idx = Code.Var.idx v in
+          idents.(idx) <- idents.(idx) + 1
   end
 
 class name ident label =
   object (m)
-    inherit Js_traverse.subst ident as super
+    inherit Js_traverse.map as super
+
+    method ident x = ident x
 
     method statement =
       function
@@ -366,53 +427,57 @@ class name ident label =
 let program' (module Strategy : Strategy) p =
   let nv = Var.count () in
   let state = Strategy.create nv in
-  let labels = Hashtbl.create 20 in
+  let labels = Var.Hashtbl.create 20 in
   let mapper = new traverse (Strategy.record_block state) in
   let p = mapper#program p in
+  let count = Array.make nv 0 in
   let () =
-    let o = new traverse_labels labels in
+    let o = new traverse_idents_and_labels ~idents:count ~labels in
     o#program p
   in
   mapper#record_block Normal;
-  let free =
-    IdentSet.filter
-      (function
-        | V _ -> true
-        | S _ -> false)
+  let freevar =
+    IdentSet.fold
+      (fun ident acc ->
+        match ident with
+        | V v -> Var.Set.add v acc
+        | S _ -> acc)
       mapper#get_free
+      Var.Set.empty
   in
-  let has_free_var = IdentSet.cardinal free <> 0 in
+  let has_free_var = not (Var.Set.is_empty freevar) in
   let unallocated_names = ref Var.Set.empty in
-  let names = Strategy.allocate_variables state ~count:mapper#get_count in
+  let names = Strategy.allocate_variables state ~count in
   (* ignore the choosen name for escaping/free [V _] variables *)
-  IdentSet.iter
-    (function
-      | S _ -> ()
-      | V x -> names.(Var.idx x) <- "")
-    free;
-  let ident = function
-    | V v -> (
-        if Config.Flag.stable_var ()
-        then
-          ident ~var:v (Utf8_string.of_string_exn (Printf.sprintf "v%d" (Code.Var.idx v)))
-        else
+  Var.Set.iter (fun x -> names.(Var.idx x) <- "") freevar;
+  let ident =
+    if Config.Flag.stable_var ()
+    then
+      function
+      | S _ as x -> x
+      | V v ->
+          let name = Printf.sprintf "v%d" (Code.Var.idx v) in
+          ident ~var:v (Utf8_string.of_string_exn name)
+    else
+      function
+      | S _ as x -> x
+      | V v as x -> (
           let name = names.(Var.idx v) in
           match name with
           | "" ->
               unallocated_names := Var.Set.add v !unallocated_names;
-              V v
+              x
           | _ -> ident ~var:v (Utf8_string.of_string_exn name))
-    | x -> x
   in
   let label_printer = Var_printer.create Var_printer.Alphabet.javascript in
-  let max_label_depth = Hashtbl.fold (fun _ d acc -> max d acc) labels 0 in
+  let max_label_depth = Var.Hashtbl.fold (fun _ d acc -> max d acc) labels 0 in
   let lname_per_depth =
     Array.init (max_label_depth + 1) ~f:(fun i -> Var_printer.to_string label_printer i)
   in
   let label = function
     | Label.S _ as l -> l
     | L v ->
-        let i = Hashtbl.find labels v in
+        let i = Var.Hashtbl.find labels v in
         S (Utf8_string.of_string_exn lname_per_depth.(i))
   in
   let p = (new name ident label)#program p in
@@ -423,7 +488,7 @@ let program' (module Strategy : Strategy) p =
        then
          Format.eprintf
            "Some variables escaped (#%d). Use [--debug js_assign] for more info.@."
-           (IdentSet.cardinal free)
+           (Var.Set.cardinal freevar)
        else
          let (_ : Source_map.info) =
            Js_output.program
@@ -432,11 +497,7 @@ let program' (module Strategy : Strategy) p =
              p
          in
          Format.eprintf "Some variables escaped:";
-         IdentSet.iter
-           (function
-             | S _ -> ()
-             | V v -> Format.eprintf " <%s>" (Var.to_string v))
-           free;
+         Var.Set.iter (fun v -> Format.eprintf " <%a>" Var.print v) freevar;
          Format.eprintf "@."
      in
      assert false);
