@@ -164,7 +164,7 @@ struct
     match loc with
     | N -> ()
     | _ ->
-        let location_changed = Poly.(loc <> !current_loc) in
+        let location_changed = not (location_equal loc !current_loc) in
         (if source_map_enabled && (!last_mapping_has_a_name || location_changed)
          then
            match loc with
@@ -242,7 +242,7 @@ struct
     | S { name = Utf8 name; var = None; _ } -> PP.string f name
     | V v ->
         assert accept_unnamed_var;
-        PP.string f ("<" ^ Code.Var.to_string v ^ ">")
+        PP.string f (Format.asprintf "<%a>" Code.Var.print v)
 
   let opt_identifier f ~kind i =
     match i with
@@ -467,7 +467,12 @@ struct
       | EAssignTarget (ArrayTarget _) -> false
       | EBin (op, e1, e2) ->
           let out, lft, rght = op_prec op in
-          Prec.(l <= out) && (Poly.(op = In && in_) || traverse lft e1 || traverse rght e2)
+          Prec.(l <= out)
+          && ((match op with
+              | In -> in_
+              | _ -> false)
+             || traverse lft e1
+             || traverse rght e2)
       | EUn ((IncrA | DecrA | IncrB | DecrB), e) ->
           Prec.(l <= UpdateExpression) && traverse LeftHandSideExpression e
       | EUn (_, e) -> Prec.(l <= UnaryExpression) && traverse UnaryExpression e
@@ -542,17 +547,16 @@ struct
     for i = 0 to l - 1 do
       let c = s.[i] in
       match c with
-      | '\000' when i = l - 1 || not (Char.is_num s.[i + 1]) -> Buffer.add_string b "\\0"
-      | '\b' -> Buffer.add_string b "\\b"
-      | '\t' -> Buffer.add_string b "\\t"
-      | '\n' -> Buffer.add_string b "\\n"
-      (* This escape sequence is not supported by IE < 9
-         | '\011' -> "\\v"
-      *)
+      | '\000' when i = l - 1 || not (Char.is_digit s.[i + 1]) ->
+          Buffer.add_string b "\\0"
+      | '\b' (* 008 *) -> Buffer.add_string b "\\b"
+      | '\t' (* 009 *) -> Buffer.add_string b "\\t"
+      | '\n' (* 010 *) -> Buffer.add_string b "\\n"
+      | '\011' -> Buffer.add_string b "\\v"
       | '\012' -> Buffer.add_string b "\\f"
+      | '\r' (* 013 *) -> Buffer.add_string b "\\r"
       (* https://github.com/ocsigen/js_of_ocaml/issues/898 *)
       | '/' when i > 0 && Char.equal s.[i - 1] '<' -> Buffer.add_string b "\\/"
-      | '\r' -> Buffer.add_string b "\\r"
       | '\000' .. '\031' | '\127' ->
           Buffer.add_string b "\\x";
           Buffer.add_char_hex b c
@@ -766,7 +770,10 @@ struct
         then (
           PP.start_group f 1;
           PP.string f "(");
-        if Poly.(op = IncrB) then PP.string f "++" else PP.string f "--";
+        (match op with
+        | IncrB -> PP.string f "++"
+        | DecrB -> PP.string f "--"
+        | _ -> assert false);
         expression UnaryExpression f e;
         if Prec.(l > p)
         then (
@@ -779,7 +786,10 @@ struct
           PP.start_group f 1;
           PP.string f "(");
         expression LeftHandSideExpression f e;
-        if Poly.(op = IncrA) then PP.string f "++" else PP.string f "--";
+        (match op with
+        | IncrA -> PP.string f "++"
+        | DecrA -> PP.string f "--"
+        | _ -> assert false);
         if Prec.(l > p)
         then (
           PP.string f ")";
@@ -1093,7 +1103,8 @@ struct
               PP.end_group f;
               PP.string f ")"
               (* There MUST be a space between the yield and its
-                 argument. A line return will not work *)))
+                 argument. A line return will not work *))
+        )
     | EPrivName (Utf8 i) ->
         PP.string f "#";
         PP.string f i
@@ -1103,11 +1114,11 @@ struct
   and template f l =
     PP.string f "`";
     List.iter l ~f:(function
-        | TStr (Utf8 s) -> PP.string f s
-        | TExp e ->
-            PP.string f "${";
-            expression AssignementExpression f e;
-            PP.string f "}");
+      | TStr (Utf8 s) -> PP.string f s
+      | TExp e ->
+          PP.string f "${";
+          expression AssignementExpression f e;
+          PP.string f "}");
     PP.string f "`"
 
   and property_name f n =
@@ -1656,7 +1667,8 @@ struct
             PP.end_group f;
             PP.end_group f
             (* There MUST be a space between the return and its
-               argument. A line return will not work *))
+               argument. A line return will not work *)
+        )
     | Labelled_statement (i, s) ->
         let (Utf8 l) = nane_of_label i in
         PP.string f l;
@@ -1790,9 +1802,10 @@ struct
               f
               ~force_last_comma:(fun _ -> false)
               (fun f (s, i) ->
-                if match i with
-                   | S { name; _ } when Stdlib.Utf8_string.equal name s -> true
-                   | _ -> false
+                if
+                  match i with
+                  | S { name; _ } when Stdlib.Utf8_string.equal name s -> true
+                  | _ -> false
                 then ident f ~kind:`Binding i
                 else (
                   pp_ident_or_string_lit f s;
@@ -1822,9 +1835,10 @@ struct
               ~force_last_comma:(fun _ -> false)
               f
               (fun f (i, s) ->
-                if match i with
-                   | S { name; _ } when Stdlib.Utf8_string.equal name s -> true
-                   | _ -> false
+                if
+                  match i with
+                  | S { name; _ } when Stdlib.Utf8_string.equal name s -> true
+                  | _ -> false
                 then ident f ~kind:`Reference i
                 else (
                   ident f ~kind:`Reference i;
@@ -1916,8 +1930,8 @@ struct
     PP.string f "}";
     PP.end_group f
 
-  and function_declaration :
-      type a. 'pp -> string -> ('pp -> a -> unit) -> a option -> _ -> _ -> _ -> unit =
+  and function_declaration : type a.
+      'pp -> string -> ('pp -> a -> unit) -> a option -> _ -> _ -> _ -> unit =
    fun f prefix (pp_name : _ -> a -> unit) (name : a option) l body loc ->
     PP.start_group f 0;
     PP.start_group f 0;
@@ -2049,7 +2063,7 @@ let need_space a b =
   | _, _ -> false
 
 let hashtbl_to_list htb =
-  Hashtbl.fold (fun k v l -> (k, v) :: l) htb []
+  String.Hashtbl.fold (fun k v l -> (k, v) :: l) htb []
   |> List.sort ~cmp:(fun (_, a) (_, b) -> compare a b)
   |> List.map ~f:fst
 
@@ -2057,21 +2071,21 @@ let blackbox_filename = "/builtin/blackbox.ml"
 
 let program ?(accept_unnamed_var = false) ?(source_map = false) f p =
   let temp_mappings = ref [] in
-  let files = Hashtbl.create 17 in
-  let names = Hashtbl.create 17 in
+  let files = String.Hashtbl.create 17 in
+  let names = String.Hashtbl.create 17 in
   let push_mapping, get_file_index, get_name_index =
     ( (fun pos m -> temp_mappings := (pos, m) :: !temp_mappings)
     , (fun file ->
-        try Hashtbl.find files file
+        try String.Hashtbl.find files file
         with Not_found ->
-          let pos = Hashtbl.length files in
-          Hashtbl.add files file pos;
+          let pos = String.Hashtbl.length files in
+          String.Hashtbl.add files file pos;
           pos)
     , fun name ->
-        try Hashtbl.find names name
+        try String.Hashtbl.find names name
         with Not_found ->
-          let pos = Hashtbl.length names in
-          Hashtbl.add names name pos;
+          let pos = String.Hashtbl.length names in
+          String.Hashtbl.add names name pos;
           pos )
   in
   let hidden_location =
@@ -2097,7 +2111,9 @@ let program ?(accept_unnamed_var = false) ?(source_map = false) f p =
     let accept_unnamed_var = accept_unnamed_var
   end) in
   PP.set_needed_space_function f need_space;
-  if Config.Flag.effects () then PP.set_adjust_indentation_function f (fun n -> n mod 40);
+  (match Config.effects () with
+  | `Cps | `Double_translation -> PP.set_adjust_indentation_function f (fun n -> n mod 40)
+  | `Disabled | `Jspi | (exception Failure _) -> ());
   PP.start_group f 0;
   O.program f p;
   PP.end_group f;
@@ -2125,8 +2141,9 @@ let program ?(accept_unnamed_var = false) ?(source_map = false) f p =
               (* Firefox assumes that a mapping stops at the end of a
                  line, which is inconvenient. When this happens, we
                  repeat the mapping on the next line. *)
-              if pos'.PP.p_line = pos.PP.p_line
-                 || (pos'.p_line = pos.p_line - 1 && pos.p_col = 0)
+              if
+                pos'.PP.p_line = pos.PP.p_line
+                || (pos'.p_line = pos.p_line - 1 && pos.p_col = 0)
               then build_mappings pos' rem (relocate pos' m :: prev_mappings)
               else if pos.p_col > 0
               then
