@@ -47,6 +47,8 @@ class type mapper = object
     -> Javascript.for_binding
     -> Javascript.for_binding
 
+  method binding_property : Javascript.binding_property -> Javascript.binding_property
+
   method variable_declaration :
        Javascript.variable_declaration_kind
     -> Javascript.variable_declaration
@@ -77,6 +79,8 @@ class type iterator = object
 
   method class_decl : Javascript.class_declaration -> unit
 
+  method class_element : Javascript.class_element -> unit
+
   method early_error : Javascript.early_error -> unit
 
   method expression : Javascript.expression -> unit
@@ -103,6 +107,8 @@ class type iterator = object
 
   method statements : Javascript.statement_list -> unit
 
+  method formal_parameter_list : Javascript.formal_parameter_list -> unit
+
   method ident : Javascript.ident -> unit
 
   method program : Javascript.program -> unit
@@ -118,10 +124,6 @@ class map : mapper
 
 class iter : iterator
 
-class subst : (ident -> ident) -> object
-  inherit mapper
-end
-
 type t =
   { use : IdentSet.t
   ; def_var : IdentSet.t
@@ -150,8 +152,6 @@ class type freevar = object ('a)
 
   method state : t
 
-  method get_count : int IdentMap.t
-
   method get_free : IdentSet.t
 
   method get_def : IdentSet.t
@@ -161,8 +161,13 @@ end
 
 class free : freevar
 
+val declared_names : program -> StringSet.t
+
+class fast_freevar : (string -> unit) -> iterator
+
 type scope =
   | Module
+  | Script
   | Lexical_block
   | Fun_block of ident option
 
@@ -172,12 +177,12 @@ class rename_variable : esm:bool -> object ('a)
   method update_state : scope -> Javascript.ident list -> Javascript.statement_list -> 'a
 end
 
-class share_constant : mapper
+val share_constant : Javascript.program -> Javascript.program
 
 class compact_vardecl : object ('a)
-  inherit free
+  inherit map
 
-  method exc : IdentSet.t
+  method pack : Javascript.statement_list -> Javascript.statement_list
 end
 
 class clean : mapper
