@@ -101,7 +101,7 @@ module Check = struct
             (fun x acc ->
               match x with
               | S { name = Utf8_string.Utf8 s; _ } ->
-                  if String.is_prefix s ~prefix:"_" || String.equal s name
+                  if String.starts_with s ~prefix:"_" || String.equal s name
                   then acc
                   else s :: acc
               | V _ -> acc)
@@ -112,8 +112,9 @@ module Check = struct
         (match diff with
         | [] -> ()
         | l ->
-            warn
-              "WARN unused for primitive %s at %s:@. %s@."
+            Warning.warn
+              `Unused_js_variable
+              "unused variable for primitive %s at %s:@. %s@."
               name
               (loc pi)
               (String.concat ~sep:", " l));
@@ -121,13 +122,18 @@ module Check = struct
     end
 
   let primitive ~name pi ~code ~requires ~has_flags =
-    let free =
-      if Config.Flag.warn_unused ()
-      then new check_and_warn name pi
-      else new Js_traverse.free
+    let freename =
+      if Warning.enabled `Unused_js_variable
+      then
+        let o = new check_and_warn name pi in
+        let _code = o#program code in
+        to_stringset o#get_free
+      else
+        let free = ref StringSet.empty in
+        let o = new Js_traverse.fast_freevar (fun s -> free := StringSet.add s !free) in
+        o#program code;
+        !free
     in
-    let _code = free#program code in
-    let freename = to_stringset free#get_free in
     let freename =
       List.fold_left requires ~init:freename ~f:(fun freename x ->
           StringSet.remove x freename)
@@ -135,25 +141,36 @@ module Check = struct
     let freename = StringSet.diff freename Reserved.keyword in
     let freename = StringSet.diff freename Reserved.provided in
     let freename = StringSet.remove Global_constant.global_object freename in
-    let freename = if has_flags then StringSet.remove "FLAG" freename else freename in
+    let freename =
+      if has_flags
+      then StringSet.(diff freename (of_list [ "FLAG"; "CONFIG" ]))
+      else freename
+    in
     if StringSet.mem Global_constant.old_global_object freename
     then
-      warn
-        "warning: %s: 'joo_global_object' is being deprecated, please use `globalThis` \
-         instead@."
+      Warning.warn
+        `Deprecated_joo_global_object
+        "%s: 'joo_global_object' is being deprecated, please use `globalThis` instead@."
         (loc pi);
     let freename = StringSet.remove Global_constant.old_global_object freename in
-    let defname = to_stringset free#get_def in
-    if not (StringSet.mem name defname)
+    if not (StringSet.mem name (Js_traverse.declared_names code))
     then
-      warn
-        "warning: primitive code does not define value with the expected name: %s (%s)@."
+      Warning.warn
+        `Missing_define
+        "primitive code does not define value with the expected name: %s (%s)@."
         name
         (loc pi);
     if not (StringSet.is_empty freename)
-    then (
-      warn "warning: free variables in primitive code %S (%s)@." name (loc pi);
-      warn "vars: %s@." (String.concat ~sep:", " (StringSet.elements freename)))
+    then
+      Warning.warn
+        `Free_variables_in_primitive
+        "free variables in primitive code %S (%s)@.vars: %a@."
+        name
+        (loc pi)
+        (Format.pp_print_list
+           ~pp_sep:(fun fmt () -> Format.pp_print_string fmt ", ")
+           Format.pp_print_string)
+        (StringSet.elements freename)
 end
 
 module Fragment = struct
@@ -185,7 +202,16 @@ module Fragment = struct
       ~f:(fun m (k, v) -> StringMap.add k v m)
       ~init:StringMap.empty
       [ "js-string", Config.Flag.use_js_string
-      ; "effects", Config.Flag.effects
+      ; ( "effects"
+        , fun () ->
+            match Config.effects () with
+            | `Disabled | `Jspi -> false
+            | `Cps | `Double_translation -> true )
+      ; ( "doubletranslate"
+        , fun () ->
+            match Config.effects () with
+            | `Double_translation -> true
+            | `Jspi | `Cps | `Disabled -> false )
       ; ( "wasm"
         , fun () ->
             match Config.target () with
@@ -413,21 +439,22 @@ type provided =
   ; filename : string
   ; weakdef : bool
   ; target_env : Target_env.t
+  ; aliases : StringSet.t
   }
 
 let always_included = ref []
 
-let provided = Hashtbl.create 31
+let provided = String.Hashtbl.create 31
 
-let provided_rev = Hashtbl.create 31
+let provided_rev = Int.Hashtbl.create 31
 
-let code_pieces = Hashtbl.create 31
+let code_pieces = Int.Hashtbl.create 31
 
 let reset () =
   always_included := [];
-  Hashtbl.clear provided;
-  Hashtbl.clear provided_rev;
-  Hashtbl.clear code_pieces;
+  String.Hashtbl.clear provided;
+  Int.Hashtbl.clear provided_rev;
+  Int.Hashtbl.clear code_pieces;
   Primitive.reset ();
   Generate.init ()
 
@@ -435,13 +462,25 @@ let list_all ?from () =
   let include_ =
     match from with
     | None -> fun _ _ -> true
-    | Some l -> fun fn _nm -> List.mem fn ~set:l
+    | Some l -> fun fn _nm -> List.mem ~eq:String.equal fn l
   in
-  Hashtbl.fold
+  String.Hashtbl.fold
     (fun nm p set -> if include_ p.filename nm then StringSet.add nm set else set)
     provided
     StringSet.empty
 
+let list_all_with_aliases ?from () =
+  let include_ =
+    match from with
+    | None -> fun _ _ -> true
+    | Some l -> fun fn _nm -> List.mem ~eq:String.equal fn l
+  in
+  String.Hashtbl.fold
+    (fun nm p map ->
+      if include_ p.filename nm then StringMap.add nm p.aliases map else map)
+    provided
+    StringMap.empty
+
 let load_fragment ~target_env ~filename (f : Fragment.t) =
   match f with
   | Always_include code ->
@@ -495,7 +534,7 @@ let load_fragment ~target_env ~filename (f : Fragment.t) =
               Option.value ~default:Target_env.Isomorphic fragment_target
             in
             let exists =
-              try `Exists (Hashtbl.find provided name) with Not_found -> `New
+              try `Exists (String.Hashtbl.find provided name) with Not_found -> `New
             in
             let is_updating =
               match
@@ -529,8 +568,9 @@ let load_fragment ~target_env ~filename (f : Fragment.t) =
                   if p.weakdef
                   then true
                   else (
-                    warn
-                      "warning: overriding primitive %S\n  old: %s\n  new: %s@."
+                    Warning.warn
+                      `Overriding_primitive
+                      "overriding primitive %S\n  old: %s\n  new: %s@."
                       name
                       (loc p.pi)
                       (loc pi);
@@ -540,27 +580,30 @@ let load_fragment ~target_env ~filename (f : Fragment.t) =
             then `Ignored
             else
               let () = () in
-              let id = Hashtbl.length provided in
+              let id = String.Hashtbl.length provided in
               Primitive.register name kind ka arity;
               StringSet.iter Primitive.register_named_value named_values;
-              Hashtbl.add
+              String.Hashtbl.add
                 provided
                 name
-                { id; pi; filename; weakdef; target_env = fragment_target };
-              Hashtbl.add provided_rev id (name, pi);
-              Hashtbl.add code_pieces id (code, has_macro, requires, deprecated);
+                { id; pi; filename; weakdef; target_env = fragment_target; aliases };
+              Int.Hashtbl.add provided_rev id (name, pi);
+              Int.Hashtbl.add code_pieces id (code, has_macro, requires, deprecated);
               StringSet.iter (fun alias -> Primitive.alias alias name) aliases;
               `Ok)
 
 let check_deps () =
   let provided = list_all () in
-  Hashtbl.iter
+  Int.Hashtbl.iter
     (fun id (code, _has_macro, requires, _deprecated) ->
       match code with
       | Ok code -> (
-          let traverse = new Js_traverse.free in
-          let _js = traverse#program code in
-          let free = to_stringset traverse#get_free in
+          let free = ref StringSet.empty in
+          let traverse =
+            new Js_traverse.fast_freevar (fun s -> free := StringSet.add s !free)
+          in
+          traverse#program code;
+          let free = !free in
           let requires =
             List.fold_right requires ~init:StringSet.empty ~f:StringSet.add
           in
@@ -569,8 +612,9 @@ let check_deps () =
           if not (StringSet.is_empty missing)
           then
             try
-              let name, ploc = Hashtbl.find provided_rev id in
-              warn
+              let name, ploc = Int.Hashtbl.find provided_rev id in
+              Warning.warn
+                `Missing_deps
                 "code providing %s (%s) may miss dependencies: %s\n"
                 name
                 (loc ploc)
@@ -602,7 +646,7 @@ let load_files ~target_env l =
 
 (* resolve *)
 let rec resolve_dep_name_rev state path nm =
-  match Hashtbl.find provided nm with
+  match String.Hashtbl.find provided nm with
   | x ->
       if state.include_ x.filename
       then resolve_dep_id_rev state path x.id
@@ -612,17 +656,17 @@ let rec resolve_dep_name_rev state path nm =
 and resolve_dep_id_rev state path id =
   if IntSet.mem id state.ids
   then (
-    if List.memq id ~set:path
+    if List.mem ~eq:Int.equal id path
     then
       error
         "circular dependency: %s"
         (String.concat
            ~sep:", "
-           (List.map path ~f:(fun id -> fst (Hashtbl.find provided_rev id))));
+           (List.map path ~f:(fun id -> fst (Int.Hashtbl.find provided_rev id))));
     state)
   else
     let path = id :: path in
-    let code, has_macro, req, deprecated = Hashtbl.find code_pieces id in
+    let code, has_macro, req, deprecated = Int.Hashtbl.find code_pieces id in
     let state = { state with ids = IntSet.add id state.ids } in
     let state =
       List.fold_left req ~init:state ~f:(fun state nm ->
@@ -643,7 +687,7 @@ let init ?from () =
   let include_ =
     match from with
     | None -> fun _ -> true
-    | Some l -> fun fn -> List.mem fn ~set:l
+    | Some l -> fun fn -> List.mem ~eq:String.equal fn l
   in
   { ids = IntSet.empty
   ; always_required_codes =
@@ -665,7 +709,7 @@ let resolve_deps ?(check_missing = true) state used =
   let missing, state =
     StringSet.fold
       (fun nm (missing, visited) ->
-        if Hashtbl.mem provided nm
+        if String.Hashtbl.mem provided nm
         then missing, resolve_dep_name_rev visited [] nm
         else StringSet.add nm missing, visited)
       used
@@ -698,18 +742,23 @@ let link ?(check_missing = true) program (state : state) =
       | [ x ] ->
           if false
           then
-            let name = fst (Hashtbl.find provided_rev x) in
-            warn "The runtime primitive [%s] is deprecated. %s\n" name txt
+            let name = fst (Int.Hashtbl.find provided_rev x) in
+            Warning.warn
+              `Deprecated_primitive
+              "The runtime primitive [%s] is deprecated. %s\n"
+              name
+              txt
       | x :: path ->
-          let name = fst (Hashtbl.find provided_rev x) in
+          let name = fst (Int.Hashtbl.find provided_rev x) in
           let path =
             String.concat
               ~sep:"\n"
               (List.map path ~f:(fun id ->
-                   let nm, loc = Hashtbl.find provided_rev id in
+                   let nm, loc = Int.Hashtbl.find provided_rev id in
                    Printf.sprintf "-> %s:%s" nm (Parse_info.to_string loc)))
           in
-          warn
+          Warning.warn
+            `Deprecated_primitive
             "The runtime primitive [%s] is deprecated. %s.  Used by:\n%s\n"
             name
             txt
@@ -731,7 +780,7 @@ let all state =
   IntSet.fold
     (fun id acc ->
       try
-        let name, _ = Hashtbl.find provided_rev id in
+        let name, _ = Int.Hashtbl.find provided_rev id in
         name :: acc
       with Not_found -> acc)
     state.ids
@@ -741,13 +790,13 @@ let missing state = StringSet.elements state.missing
 
 let origin ~name =
   try
-    let x = Hashtbl.find provided name in
+    let x = String.Hashtbl.find provided name in
     x.pi.Parse_info.src
   with Not_found -> None
 
 let deprecated ~name =
   try
-    let x = Hashtbl.find provided name in
-    let _, _, _, deprecated = Hashtbl.find code_pieces x.id in
+    let x = String.Hashtbl.find provided name in
+    let _, _, _, deprecated = Int.Hashtbl.find code_pieces x.id in
     Option.is_some deprecated
   with Not_found -> false
