@@ -40,15 +40,25 @@ class macro_mapper ~flags =
               | Count count ->
                   incr count;
                   super#expression x)
+          | "CONFIG", [ J.Arg (J.EStr (Utf8 "effects")) ] -> (
+              match flags with
+              | Replace ->
+                  let s = Build_info.string_of_effects_backend (Config.effects ()) in
+                  J.EStr (Utf8_string.of_string_exn s)
+              | Count count ->
+                  incr count;
+                  super#expression x)
+          | "CONFIG", [ J.Arg (J.EStr (Utf8 s)) ] ->
+              failwith ("unsupported CONFIG parameter " ^ s)
           | "BLOCK", J.Arg (J.ENum tag) :: (_ :: _ as args)
             when List.for_all args ~f:(function
-                     | J.Arg _ -> true
-                     | J.ArgSpread _ -> false) ->
+                   | J.Arg _ -> true
+                   | J.ArgSpread _ -> false) ->
               let tag = Targetint.to_int_exn (J.Num.to_targetint tag) in
               let args =
                 List.map args ~f:(function
-                    | J.Arg e -> J.Element (m#expression e)
-                    | J.ArgSpread _ -> assert false)
+                  | J.Arg e -> J.Element (m#expression e)
+                  | J.ArgSpread _ -> assert false)
               in
               Mlvalue.Block.make ~tag ~args
           | "TAG", [ J.Arg e ] -> Mlvalue.Block.tag (m#expression e)
