@@ -23,16 +23,3 @@ val current : t
 val compare : t -> t -> int
 
 val split : string -> t
-
-val v :
-  [ `V4_08 (* OCaml 4.08 *)
-  | `V4_09 (* OCaml 4.09 *)
-  | `V4_10 (* OCaml 4.10 *)
-  | `V4_11 (* OCaml 4.11 *)
-  | `V4_12 (* OCaml 4.12 *)
-  | `V4_13 (* OCaml 4.13 *)
-  | `V4_14 (* OCaml 4.14 *)
-  | `V5_00
-  | `V5_01
-  | `V5_02
-  ]
