@@ -21,25 +21,19 @@
 open Stdlib
 
 module Debug : sig
-  type t
+  type summary
 
-  type position =
-    | Before
-    | After
+  val is_empty : summary -> bool
 
-  val create : include_cmis:bool -> bool -> t
+  val default_summary : summary
 
-  val find_loc : t -> position:position -> Code.Addr.t -> Parse_info.t option
-
-  val is_empty : t -> bool
-
-  val paths : t -> units:StringSet.t -> StringSet.t
+  val paths : summary -> units:StringSet.t -> StringSet.t
 end
 
 type one =
   { code : Code.program
   ; cmis : StringSet.t
-  ; debug : Debug.t
+  ; debug : Debug.summary
   }
 
 module Toc : sig
@@ -81,10 +75,7 @@ val from_channel :
   -> [ `Cmo of Cmo_format.compilation_unit | `Cma of Cmo_format.library | `Exe ]
 
 val from_string :
-     prims:string array
-  -> debug:Instruct.debug_event list array
-  -> string
-  -> Code.program * Debug.t
+  prims:string array -> debug:Instruct.debug_event list array -> string -> Code.program
 
 val predefined_exceptions : unit -> Code.program * Unit_info.t
 
