@@ -48,6 +48,7 @@ type t =
   | `Weakdef
   | `Always
   | `Alias of string
+  | `Deprecated of string
   | condition
   ]
 
@@ -63,6 +64,8 @@ val has_arity : string -> int -> bool
 
 val alias : string -> string -> unit
 
+val aliases : unit -> (string * string) list
+
 val resolve : string -> string
 
 val add_external : string -> unit
