@@ -21,6 +21,8 @@ module Source_content : sig
   type t
 
   val create : string -> t
+
+  val of_stringlit : [ `Stringlit of string ] -> t
 end
 
 type map =
@@ -60,9 +62,12 @@ module Mappings : sig
   val empty : t
   (** The empty mapping. *)
 
+  val is_empty : t -> bool
+  (** Test whether the mapping is empty. *)
+
   val of_string_unsafe : string -> t
   (** [of_string_unsafe] does not perform any
-      validation of its argument, unlike {!val:decode}. It is guaranteed that
+      validation of its argument, unlike {!val:decode_exn}. It is guaranteed that
       {!val:of_string_unsafe} and {!val:to_string} are inverse functions.
       Time complexity O(1) *)
 
@@ -111,6 +116,10 @@ module Standard : sig
     linear in function of the size of the input mappings. *)
 
   val empty : inline_source_content:bool -> t
+
+  val of_string : ?tmp_buf:Buffer.t -> string -> t
+
+  val of_file : ?tmp_buf:Buffer.t -> string -> t
 end
 
 module Index : sig
@@ -132,16 +141,27 @@ type t =
 
 val to_string : t -> string
 
-val to_file : t -> string -> unit
+val to_file : ?rewrite_paths:bool -> t -> string -> unit
 
-val of_string : string -> t
+val of_string : ?tmp_buf:Buffer.t -> string -> t
 
-val of_file : string -> t
+val of_file : ?tmp_buf:Buffer.t -> string -> t
 
 val invariant : t -> unit
 
+val find_in_js_file : string -> t option
+
 type info =
   { mappings : Mappings.decoded
   ; sources : string list
   ; names : string list
   }
+
+module Encoding_spec : sig
+  type t =
+    { output_file : string option  (** Source map file ([None] means generate inline. *)
+    ; source_map : Standard.t  (** Source map to extend. *)
+    ; keep_empty : bool
+          (** Don't add anything to the source map (for js_of_ocaml's "empty sourcemap" option. *)
+    }
+end
