@@ -16,22 +16,28 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
+let open_in_text = open_in
+
+let open_out_text = open_out
+
+module Deprecated : sig
+  val open_in : string -> in_channel [@@deprecated "use open_int_text/open_int_bin"]
+
+  val open_out : string -> out_channel [@@deprecated "use open_out_text/open_out_bin"]
+end = struct
+  let open_in = open_in
+
+  let open_out = open_out
+end
+
+include Deprecated
+
 module Poly = struct
-  external ( < ) : 'a -> 'a -> bool = "%lessthan"
-
-  external ( <= ) : 'a -> 'a -> bool = "%lessequal"
-
-  external ( <> ) : 'a -> 'a -> bool = "%notequal"
-
-  external ( = ) : 'a -> 'a -> bool = "%equal"
-
-  external ( > ) : 'a -> 'a -> bool = "%greaterthan"
-
-  external ( >= ) : 'a -> 'a -> bool = "%greaterequal"
-
   external compare : 'a -> 'a -> int = "%compare"
 
   external equal : 'a -> 'a -> bool = "%equal"
+
+  module Hashtbl = Hashtbl
 end
 
 module Int_replace_polymorphic_compare = struct
@@ -64,19 +70,6 @@ let ( != ) = `use_phys_equal
 
 include Int_replace_polymorphic_compare
 
-let quiet = ref false
-
-let werror = ref false
-
-let warnings = ref 0
-
-let warn fmt =
-  Format.ksprintf
-    (fun s ->
-      incr warnings;
-      if not !quiet then Format.eprintf "%s%!" s)
-    fmt
-
 let fail = ref true
 
 let failwith_ fmt =
@@ -85,37 +78,32 @@ let failwith_ fmt =
 let raise_ exn =
   if !fail then raise exn else Format.eprintf "%s@." (Printexc.to_string exn)
 
-let int_num_bits = Sys.int_size
-
 module List = struct
   include ListLabels
 
-  let rec equal ~eq a b =
-    match a, b with
-    | [], [] -> true
-    | x :: xs, y :: ys -> eq x y && equal ~eq xs ys
-    | [], _ :: _ | _ :: _, [] -> false
+  let (mem_assoc [@deprecated "use List.exists"]) = List.mem
 
-  let rec find_map ~f = function
-    | [] -> None
-    | x :: l -> (
-        match f x with
-        | Some _ as result -> result
-        | None -> find_map ~f l)
+  let (assoc [@deprecated "use List.find_map"]) = List.assoc
 
-  let rec find_map_value ~f ~default = function
-    | [] -> default
-    | x :: l -> (
-        match f x with
-        | Some result -> result
-        | None -> find_map_value ~f ~default l)
+  let (assoc_opt [@deprecated "use List.find_map"]) = List.assoc_opt
+
+  let (remove_assoc [@deprecated "use List.filter"]) = List.remove_assoc
+
+  let rec mem ~eq x = function
+    | [] -> false
+    | a :: l -> eq a x || mem ~eq x l
+
+  let string_assoc name l =
+    List.find_map
+      (fun (name', state) -> if String.equal name name' then Some state else None)
+      l
 
   let rec rev_append_map ~f l acc =
     match l with
     | [] -> acc
     | x :: xs -> rev_append_map ~f xs (f x :: acc)
 
-  let slow_map l ~f = rev (rev_map ~f l)
+  let slow_map l ~f = rev (rev_map ~f l) [@@if ocaml_version < (4, 14, 0)]
 
   let max_non_tailcall =
     match Sys.backend_type with
@@ -155,8 +143,15 @@ module List = struct
         :: f4
         :: f5
         :: (if ctr > max_non_tailcall then slow_map ~f tl else count_map ~f tl (ctr + 1))
+  [@@if ocaml_version < (4, 14, 0)]
 
-  let map l ~f = count_map ~f l 0
+  let map l ~f = count_map ~f l 0 [@@if ocaml_version < (4, 14, 0)]
+
+  let[@tail_mod_cons] rec map l ~f =
+    match l with
+    | [] -> []
+    | x :: tl -> f x :: (map [@tailcall]) tl ~f
+  [@@if ocaml_version >= (4, 14, 0)]
 
   let rec take' acc n l =
     if n = 0
@@ -175,34 +170,12 @@ module List = struct
     | [ x ] -> Some x
     | _ :: xs -> last xs
 
-  let sort_uniq ~compare l =
-    let l = List.sort compare l in
-    match l with
-    | ([] | [ _ ]) as l -> l
-    | x :: xs ->
-        let rec loop prev = function
-          | [] -> [ prev ]
-          | x :: rest when compare x prev = 0 -> loop prev rest
-          | x :: rest -> prev :: loop x rest
-        in
-        loop x xs
-
   let is_empty = function
     | [] -> true
     | _ -> false
+  [@@if ocaml_version < (5, 1, 0)]
 
-  let partition_map t ~f =
-    let rec loop t fst snd =
-      match t with
-      | [] -> rev fst, rev snd
-      | x :: t -> (
-          match f x with
-          | `Fst y -> loop t (y :: fst) snd
-          | `Snd y -> loop t fst (y :: snd))
-    in
-    loop t [] []
-
-  let tail_append l1 l2 = rev_append (rev l1) l2
+  let tail_append l1 l2 = rev_append (rev l1) l2 [@@if ocaml_version < (5, 1, 0)]
 
   let rec count_append l1 l2 count =
     match l2 with
@@ -224,8 +197,9 @@ module List = struct
             (if count > max_non_tailcall
              then tail_append tl l2
              else count_append tl l2 (count + 1)))
+  [@@if ocaml_version < (5, 1, 0)]
 
-  let append l1 l2 = count_append l1 l2 0
+  let append l1 l2 = count_append l1 l2 0 [@@if ocaml_version < (5, 1, 0)]
 
   let group l ~f =
     let rec loop (l : 'a list) (this_group : 'a list) (acc : 'a list list) : 'a list list
@@ -242,15 +216,6 @@ module List = struct
     | [] -> []
     | x :: xs -> loop xs [ x ] []
 
-  let concat_map ~f l =
-    let rec aux f acc = function
-      | [] -> rev acc
-      | x :: l ->
-          let xs = f x in
-          aux f (rev_append xs acc) l
-    in
-    aux f [] l
-
   let split_last xs =
     let rec aux acc = function
       | [] -> None
@@ -282,12 +247,6 @@ end
 
 let ( @ ) = List.append
 
-module Nativeint = struct
-  include Nativeint
-
-  external equal : nativeint -> nativeint -> bool = "%equal"
-end
-
 module Int32 = struct
   include Int32
 
@@ -303,14 +262,10 @@ module Int32 = struct
 
   external ( >= ) : int32 -> int32 -> bool = "%greaterequal"
 
-  external compare : int32 -> int32 -> int = "%compare"
-
-  external equal : int32 -> int32 -> bool = "%equal"
-
   let warn_overflow name ~to_dec ~to_hex i i32 =
-    warn
-      "Warning: integer overflow: %s 0x%s (%s) truncated to 0x%lx (%ld); the generated \
-       code might be incorrect.@."
+    Warning.warn
+      `Integer_overflow
+      "%s 0x%s (%s) truncated to 0x%lx (%ld); the generated code might be incorrect.@."
       name
       (to_hex i)
       (to_dec i)
@@ -334,16 +289,30 @@ module Int32 = struct
       n
 end
 
+module Int64 = struct
+  include Int64
+
+  external ( < ) : int64 -> int64 -> bool = "%lessthan"
+
+  external ( <= ) : int64 -> int64 -> bool = "%lessequal"
+
+  external ( <> ) : int64 -> int64 -> bool = "%notequal"
+
+  external ( = ) : int64 -> int64 -> bool = "%equal"
+
+  external ( > ) : int64 -> int64 -> bool = "%greaterthan"
+
+  external ( >= ) : int64 -> int64 -> bool = "%greaterequal"
+end
+
 module Option = struct
+  include Option
+
   let map ~f x =
     match x with
     | None -> None
     | Some v -> Some (f v)
 
-  let to_list = function
-    | None -> []
-    | Some x -> [ x ]
-
   let bind ~f x =
     match x with
     | None -> None
@@ -359,40 +328,13 @@ module Option = struct
     | None -> None
     | Some v -> if f v then Some v else None
 
-  let compare compare_elt a b =
-    match a, b with
-    | None, None -> 0
-    | None, Some _ -> -1
-    | Some _, None -> 1
-    | Some a, Some b -> compare_elt a b
-
-  let equal equal_elt a b =
-    match a, b with
-    | None, None -> true
-    | Some a, Some b -> equal_elt a b
-    | Some _, None | None, Some _ -> false
-
-  let is_none = function
-    | None -> true
-    | Some _ -> false
-
-  let is_some = function
-    | None -> false
-    | Some _ -> true
-
   let value ~default = function
     | None -> default
     | Some s -> s
 end
 
-module Int64 = struct
-  include Int64
-
-  let equal (a : int64) (b : int64) = Poly.(a = b)
-end
-
 module Float = struct
-  type t = float
+  include Float
 
   let equal (_ : float) (_ : float) = `Use_ieee_equal_or_bitwise_equal
 
@@ -401,9 +343,6 @@ module Float = struct
   let bitwise_equal (a : float) (b : float) =
     Int64.equal (Int64.bits_of_float a) (Int64.bits_of_float b)
 
-  (* Re-defined here to stay compatible with OCaml 4.02 *)
-  external classify_float : float -> fpclass = "caml_classify_float"
-
   external ( < ) : t -> t -> bool = "%lessthan"
 
   external ( <= ) : t -> t -> bool = "%lessequal"
@@ -418,13 +357,13 @@ module Float = struct
 end
 
 module Bool = struct
+  include Bool
+
   external ( <> ) : bool -> bool -> bool = "%notequal"
 
   external ( = ) : bool -> bool -> bool = "%equal"
 
   external ( > ) : bool -> bool -> bool = "%greaterthan"
-
-  external equal : bool -> bool -> bool = "%equal"
 end
 
 module Char = struct
@@ -442,27 +381,13 @@ module Char = struct
 
   external ( >= ) : char -> char -> bool = "%greaterequal"
 
-  external compare : char -> char -> int = "%compare"
-
-  external equal : char -> char -> bool = "%equal"
-
-  let is_alpha = function
+  let is_letter = function
     | 'a' .. 'z' | 'A' .. 'Z' -> true
     | _ -> false
 
-  let is_num = function
+  let is_digit = function
     | '0' .. '9' -> true
     | _ -> false
-
-  let lowercase_ascii c =
-    match c with
-    | 'A' .. 'Z' as c -> Char.unsafe_chr (Char.code c + 32)
-    | _ -> c
-
-  let uppercase_ascii c =
-    match c with
-    | 'a' .. 'z' as c -> Char.unsafe_chr (Char.code c - 32)
-    | _ -> c
 end
 
 module Uchar = struct
@@ -564,72 +489,23 @@ module Buffer = struct
     Buffer.add_char b (Array.unsafe_get array_conv (c land 0xf))
 end
 
-module Bytes = struct
-  include BytesLabels
-
-  let sub_string b ~pos:ofs ~len = unsafe_to_string (Bytes.sub b ofs len)
-
-  let fold_left ~f ~init b =
-    let r = ref init in
-    for i = 0 to length b - 1 do
-      r := f !r (unsafe_get b i)
-    done;
-    !r
-
-  let fold_right ~f b ~init =
-    let r = ref init in
-    for i = length b - 1 downto 0 do
-      r := f (unsafe_get b i) !r
-    done;
-    !r
-end
+module Bytes = BytesLabels
 
 module String = struct
   include StringLabels
 
-  let equal (a : string) (b : string) = Poly.(a = b)
+  let hash (a : string) = Hashtbl.hash a [@@if ocaml_version < (5, 0, 0)]
 
-  let hash (a : string) = Hashtbl.hash a
+  module Hashtbl = Hashtbl.Make (struct
+    include String
+
+    let hash = hash
+  end)
 
   let is_empty = function
     | "" -> true
     | _ -> false
 
-  let is_prefix ~prefix s =
-    let len_a = length prefix in
-    let len_s = length s in
-    if len_a > len_s
-    then false
-    else
-      let max_idx_a = len_a - 1 in
-      let rec loop i =
-        if i > max_idx_a
-        then true
-        else if not (Char.equal (unsafe_get prefix i) (unsafe_get s i))
-        then false
-        else loop (i + 1)
-      in
-      loop 0
-
-  let is_suffix ~suffix s =
-    let len_a = length suffix in
-    let len_s = length s in
-    if len_a > len_s
-    then false
-    else
-      let max_idx_a = len_a - 1 in
-      let rec loop i =
-        if i > max_idx_a
-        then true
-        else if not
-                  (Char.equal
-                     (unsafe_get suffix (len_a - 1 - i))
-                     (unsafe_get s (len_s - 1 - i)))
-        then false
-        else loop (i + 1)
-      in
-      loop 0
-
   let drop_prefix ~prefix s =
     let plen = String.length prefix in
     if plen > String.length s
@@ -642,16 +518,6 @@ module String = struct
         Some (String.sub s plen (String.length s - plen))
       with Exit -> None
 
-  let for_all =
-    let rec loop s ~f ~last i =
-      if i > last
-      then true
-      else if f (String.unsafe_get s i)
-      then loop s ~f ~last (i + 1)
-      else false
-    in
-    fun s ~f -> loop s ~f ~last:(String.length s - 1) 0
-
   let is_ascii s =
     let res = ref true in
     for i = 0 to String.length s - 1 do
@@ -668,69 +534,6 @@ module String = struct
     done;
     !res
 
-  let split_char ~sep p = String.split_on_char sep p
-
-  (* copied from https://github.com/ocaml/ocaml/pull/10 *)
-  let split ~sep s =
-    let sep_len = String.length sep in
-    if sep_len = 1
-    then split_char ~sep:sep.[0] s
-    else
-      let sep_max = sep_len - 1 in
-      if sep_max < 0
-      then invalid_arg "String.split: empty separator"
-      else
-        let s_max = String.length s - 1 in
-        if s_max < 0
-        then [ "" ]
-        else
-          let acc = ref [] in
-          let sub_start = ref 0 in
-          let k = ref 0 in
-          let i = ref 0 in
-          (* We build the substrings by running from the start of [s] to the
-             end with [i] trying to match the first character of [sep] in
-             [s]. If this matches, we verify that the whole [sep] is matched
-             using [k]. If this matches we extract a substring from the start
-             of the current substring [sub_start] to [!i - 1] (the position
-             before the [sep] we found).  We then continue to try to match
-             with [i] by starting after the [sep] we just found, this is also
-             becomes the start position of the next substring. If [i] is such
-             that no separator can be found we exit the loop and make a
-             substring from [sub_start] until the end of the string. *)
-          while !i + sep_max <= s_max do
-            if not (Char.equal (String.unsafe_get s !i) (String.unsafe_get sep 0))
-            then incr i
-            else (
-              (* Check remaining [sep] chars match, access to unsafe s (!i + !k) is
-                   guaranteed by loop invariant. *)
-              k := 1;
-              while
-                !k <= sep_max
-                && Char.equal (String.unsafe_get s (!i + !k)) (String.unsafe_get sep !k)
-              do
-                incr k
-              done;
-              if !k <= sep_max
-              then (* no match *) incr i
-              else
-                let new_sub_start = !i + sep_max + 1 in
-                let sub_end = !i - 1 in
-                let sub_len = sub_end - !sub_start + 1 in
-                acc := String.sub s !sub_start sub_len :: !acc;
-                sub_start := new_sub_start;
-                i := new_sub_start)
-          done;
-          List.rev (String.sub s !sub_start (s_max - !sub_start + 1) :: !acc)
-
-  let apply1 f (s : string) : string =
-    let b = Bytes.of_string s in
-    if Bytes.length b = 0
-    then s
-    else (
-      Bytes.unsafe_set b 0 (f (Bytes.unsafe_get b 0));
-      Bytes.to_string b)
-
   let lsplit2 line ~on:delim =
     try
       let pos = index line delim in
@@ -743,10 +546,6 @@ module String = struct
       Some (sub line ~pos:0 ~len:pos, sub line ~pos:(pos + 1) ~len:(length line - pos - 1))
     with Not_found -> None
 
-  let capitalize_ascii s = apply1 Char.uppercase_ascii s
-
-  let uncapitalize_ascii s = apply1 Char.lowercase_ascii s
-
   let[@inline] not_in_x80_to_xBF b = b lsr 6 <> 0b10
 
   let[@inline] not_in_xA0_to_xBF b = b lsr 5 <> 0b101
@@ -951,66 +750,58 @@ module String = struct
             else loop max b (last + 1)
         | '\xE0' ->
             let last = i + 2 in
-            if last > max
-               || not_in_xA0_to_xBF (get b (i + 1))
-               || not_in_x80_to_xBF (get b last)
+            if
+              last > max
+              || not_in_xA0_to_xBF (get b (i + 1))
+              || not_in_x80_to_xBF (get b last)
             then false
             else loop max b (last + 1)
         | '\xE1' .. '\xEC' | '\xEE' .. '\xEF' ->
             let last = i + 2 in
-            if last > max
-               || not_in_x80_to_xBF (get b (i + 1))
-               || not_in_x80_to_xBF (get b last)
+            if
+              last > max
+              || not_in_x80_to_xBF (get b (i + 1))
+              || not_in_x80_to_xBF (get b last)
             then false
             else loop max b (last + 1)
         | '\xED' ->
             let last = i + 2 in
-            if last > max
-               || not_in_x80_to_x9F (get b (i + 1))
-               || not_in_x80_to_xBF (get b last)
+            if
+              last > max
+              || not_in_x80_to_x9F (get b (i + 1))
+              || not_in_x80_to_xBF (get b last)
             then false
             else loop max b (last + 1)
         | '\xF0' ->
             let last = i + 3 in
-            if last > max
-               || not_in_x90_to_xBF (get b (i + 1))
-               || not_in_x80_to_xBF (get b (i + 2))
-               || not_in_x80_to_xBF (get b last)
+            if
+              last > max
+              || not_in_x90_to_xBF (get b (i + 1))
+              || not_in_x80_to_xBF (get b (i + 2))
+              || not_in_x80_to_xBF (get b last)
             then false
             else loop max b (last + 1)
         | '\xF1' .. '\xF3' ->
             let last = i + 3 in
-            if last > max
-               || not_in_x80_to_xBF (get b (i + 1))
-               || not_in_x80_to_xBF (get b (i + 2))
-               || not_in_x80_to_xBF (get b last)
+            if
+              last > max
+              || not_in_x80_to_xBF (get b (i + 1))
+              || not_in_x80_to_xBF (get b (i + 2))
+              || not_in_x80_to_xBF (get b last)
             then false
             else loop max b (last + 1)
         | '\xF4' ->
             let last = i + 3 in
-            if last > max
-               || not_in_x80_to_x8F (get b (i + 1))
-               || not_in_x80_to_xBF (get b (i + 2))
-               || not_in_x80_to_xBF (get b last)
+            if
+              last > max
+              || not_in_x80_to_x8F (get b (i + 1))
+              || not_in_x80_to_xBF (get b (i + 2))
+              || not_in_x80_to_xBF (get b last)
             then false
             else loop max b (last + 1)
         | _ -> false
     in
     loop (length b - 1) b 0
-
-  let fold_left ~f ~init s =
-    let r = ref init in
-    for i = 0 to length s - 1 do
-      r := f !r (unsafe_get s i)
-    done;
-    !r
-
-  let fold_right ~f s ~init =
-    let r = ref init in
-    for i = length s - 1 downto 0 do
-      r := f (unsafe_get s i) !r
-    done;
-    !r
 end
 
 module Utf8_string : sig
@@ -1035,13 +826,15 @@ end = struct
 end
 
 module Int = struct
-  type t = int
+  include Int
 
-  let compare (x : int) y = compare x y
+  let hash (x : t) = x
 
-  let equal (x : t) y = x = y
+  module Hashtbl = Hashtbl.Make (struct
+    include Int
 
-  let hash (x : t) = Hashtbl.hash x
+    let hash x = x
+  end)
 end
 
 module IntSet = Set.Make (Int)
@@ -1073,19 +866,23 @@ module BitSet : sig
   val next_free : t -> int -> int
 
   val next_mem : t -> int -> int
+
+  val clear : t -> unit
 end = struct
   type t = { mutable arr : int array }
 
   let create () = { arr = Array.make 1 0 }
 
-  let create' n = { arr = Array.make ((n / int_num_bits) + 1) 0 }
+  let create' n = { arr = Array.make ((n / Sys.int_size) + 1) 0 }
 
-  let size t = Array.length t.arr * int_num_bits
+  let clear t = Array.fill t.arr 0 (Array.length t.arr) 0
+
+  let size t = Array.length t.arr * Sys.int_size
 
   let mem t i =
     let arr = t.arr in
-    let idx = i / int_num_bits in
-    let off = i mod int_num_bits in
+    let idx = i / Sys.int_size in
+    let off = i mod Sys.int_size in
     idx < Array.length arr
     &&
     let x = Array.unsafe_get arr idx in
@@ -1102,15 +899,15 @@ end = struct
     t.arr <- a
 
   let set t i =
-    let idx = i / int_num_bits in
-    let off = i mod int_num_bits in
+    let idx = i / Sys.int_size in
+    let off = i mod Sys.int_size in
     let size = Array.length t.arr in
     if idx >= size then resize t idx;
     Array.unsafe_set t.arr idx (Array.unsafe_get t.arr idx lor (1 lsl off))
 
   let unset t i =
-    let idx = i / int_num_bits in
-    let off = i mod int_num_bits in
+    let idx = i / Sys.int_size in
+    let off = i mod Sys.int_size in
     let size = Array.length t.arr in
     if idx >= size
     then ()
@@ -1144,17 +941,6 @@ end
 module Array = struct
   include ArrayLabels
 
-  let find_opt ~f:p a =
-    let n = length a in
-    let rec loop i =
-      if i = n
-      then None
-      else
-        let x = unsafe_get a i in
-        if p x then Some x else loop (succ i)
-    in
-    loop 0
-
   let fold_right_i a ~f ~init:x =
     let r = ref x in
     for i = Array.length a - 1 downto 0 do
@@ -1219,6 +1005,202 @@ module Fun = struct
         r
 end
 
+module In_channel = struct
+  let stdlib_input_line = input_line
+
+  (* Read up to [len] bytes into [buf], starting at [ofs]. Return total bytes
+     read. *)
+  let read_upto ic buf ofs len =
+    let rec loop ofs len =
+      if len = 0
+      then ofs
+      else
+        let r = input ic buf ofs len in
+        if r = 0 then ofs else loop (ofs + r) (len - r)
+    in
+    loop ofs len - ofs
+
+  (* Best effort attempt to return a buffer with >= (ofs + n) bytes of storage,
+     and such that it coincides with [buf] at indices < [ofs].
+
+     The returned buffer is equal to [buf] itself if it already has sufficient
+     free space.
+
+     The returned buffer may have *fewer* than [ofs + n] bytes of storage if this
+     number is > [Sys.max_string_length]. However the returned buffer will
+     *always* have > [ofs] bytes of storage. In the limiting case when [ofs = len
+     = Sys.max_string_length] (so that it is not possible to resize the buffer at
+     all), an exception is raised. *)
+
+  let ensure buf ofs n =
+    let len = Bytes.length buf in
+    if len >= ofs + n
+    then buf
+    else
+      let new_len = ref len in
+      while !new_len < ofs + n do
+        new_len := (2 * !new_len) + 1
+      done;
+      let new_len = !new_len in
+      let new_len =
+        if new_len <= Sys.max_string_length
+        then new_len
+        else if ofs < Sys.max_string_length
+        then Sys.max_string_length
+        else
+          failwith
+            "In_channel.input_all: channel content is larger than maximum string length"
+      in
+      let new_buf = Bytes.create new_len in
+      Bytes.blit ~src:buf ~src_pos:0 ~dst:new_buf ~dst_pos:0 ~len:ofs;
+      new_buf
+
+  let input_all ic =
+    let chunk_size = 65536 in
+    (* IO_BUFFER_SIZE *)
+    let initial_size = try in_channel_length ic - pos_in ic with Sys_error _ -> -1 in
+    let initial_size = if initial_size < 0 then chunk_size else initial_size in
+    let initial_size =
+      if initial_size <= Sys.max_string_length
+      then initial_size
+      else Sys.max_string_length
+    in
+    let buf = Bytes.create initial_size in
+    let nread = read_upto ic buf 0 initial_size in
+    if nread < initial_size
+    then (* EOF reached, buffer partially filled *)
+      Bytes.sub_string buf ~pos:0 ~len:nread
+    else
+      (* nread = initial_size, maybe EOF reached *)
+      match input_char ic with
+      | exception End_of_file ->
+          (* EOF reached, buffer is completely filled *)
+          Bytes.unsafe_to_string buf
+      | c ->
+          (* EOF not reached *)
+          let rec loop buf ofs =
+            let buf = ensure buf ofs chunk_size in
+            let rem = Bytes.length buf - ofs in
+            (* [rem] can be < [chunk_size] if buffer size close to
+               [Sys.max_string_length] *)
+            let r = read_upto ic buf ofs rem in
+            if r < rem
+            then (* EOF reached *)
+              Bytes.sub_string buf ~pos:0 ~len:(ofs + r)
+            else (* r = rem *)
+              loop buf (ofs + rem)
+          in
+          let buf = ensure buf nread (chunk_size + 1) in
+          Bytes.set buf nread c;
+          loop buf (nread + 1)
+
+  let input_lines ic =
+    let rec aux acc =
+      match input_line ic with
+      | line -> aux (line :: acc)
+      | exception End_of_file -> acc
+    in
+    List.rev (aux [])
+
+  let input_line_exn = stdlib_input_line
+end
+[@@if ocaml_version < (4, 14, 0)]
+
+module In_channel = struct
+  let stdlib_input_line = input_line
+
+  include In_channel
+
+  (* [In_channel.input_lines] only exists in the stdlib since 5.1. *)
+  let[@tail_mod_cons] rec input_lines ic =
+    match stdlib_input_line ic with
+    | line -> line :: input_lines ic
+    | exception End_of_file -> []
+  [@@if ocaml_version < (5, 1, 0)]
+
+  let input_line_exn = stdlib_input_line
+end
+[@@if ocaml_version >= (4, 14, 0)]
+
+module Seq = struct
+  include Seq
+
+  let rec mapi_aux f i xs () =
+    match xs () with
+    | Nil -> Nil
+    | Cons (x, xs) -> Cons (f i x, mapi_aux f (i + 1) xs)
+
+  (* Available since OCaml 4.14 *)
+  let[@inline] mapi f xs = mapi_aux f 0 xs
+end
+
+let split_lines s =
+  if String.equal s ""
+  then []
+  else
+    let sep = '\n' in
+    let r = ref [] in
+    let j = ref (String.length s) in
+    (* ignore trailing new line *)
+    if Char.equal (String.unsafe_get s (!j - 1)) sep then decr j;
+    for i = !j - 1 downto 0 do
+      if Char.equal (String.unsafe_get s i) sep
+      then (
+        r := String.sub s ~pos:(i + 1) ~len:(!j - i - 1) :: !r;
+        j := i)
+    done;
+    String.sub s ~pos:0 ~len:!j :: !r
+
+let input_lines_read_once ic len = really_input_string ic len |> split_lines
+
+let file_lines_bin fname =
+  (* If possible, read the entire file and split it in lines.
+     This is faster than reading it line by line.
+     Otherwise, we fall back to a line-by-line read. *)
+  let ic = open_in_bin fname in
+  let len = in_channel_length ic in
+  let x =
+    if len < Sys.max_string_length
+    then input_lines_read_once ic len
+    else In_channel.input_lines ic
+  in
+  close_in ic;
+  x
+
+let file_lines_text file =
+  let ic = open_in_text file in
+  let c = In_channel.input_lines ic in
+  close_in ic;
+  c
+
 let generated_name = function
   | "param" | "match" | "switcher" -> true
-  | s -> String.is_prefix ~prefix:"cst_" s
+  | s -> String.starts_with ~prefix:"cst_" s
+
+module Hashtbl = struct
+  include Hashtbl
+
+  let (create [@deprecated "Use Int.Hashtbl, String.Hashtbl, Var.Hashtbl, Addr.Hashtbl"])
+      =
+    Hashtbl.create
+
+  let (of_seq [@deprecated "Use Int.Hashtbl, String.Hashtbl, Var.Hashtbl, Addr.Hashtbl"])
+      =
+    Hashtbl.of_seq
+end
+
+module Lexing = struct
+  include Lexing
+
+  let range_to_string (pos1, pos2) =
+    if phys_equal pos1 dummy_pos || phys_equal pos2 dummy_pos
+    then "At an unknown location:\n"
+    else
+      let file = pos1.pos_fname in
+      let line = pos1.pos_lnum in
+      let char1 = pos1.pos_cnum - pos1.pos_bol in
+      let char2 = pos2.pos_cnum - pos1.pos_bol in
+      (* yes, [pos1.pos_bol] *)
+      Printf.sprintf "File \"%s\", line %d, characters %d-%d:\n" file line char1 char2
+  (* use [char1 + 1] and [char2 + 1] if *not* using Caml mode *)
+end
