@@ -24,6 +24,8 @@ val of_int_exn : int -> t
 
 val of_int32_exn : int32 -> t
 
+val of_int32_truncate : int32 -> t
+
 val of_int32_warning_on_overflow : int32 -> t
 
 val of_nativeint_warning_on_overflow : nativeint -> t
@@ -91,3 +93,5 @@ val ( > ) : t -> t -> bool
 val ( = ) : t -> t -> bool
 
 val ( <> ) : t -> t -> bool
+
+val unsigned_lt : t -> t -> bool
