@@ -17,32 +17,18 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *)
 
-open Stdlib
+open! Stdlib
 
 module Alphabet : sig
   type t
 
   val javascript : t
+
+  val to_string : t -> int -> string
 end
 
 type t
 
-val add_reserved : string -> unit
+val create : Alphabet.t -> t
 
-val get_reserved : unit -> StringSet.t
-
-val create : ?pretty:bool -> ?stable:bool -> Alphabet.t -> t
-
-val reset : t -> unit
-
-val to_string : t -> ?origin:int -> int -> string
-
-val name : t -> int -> string -> unit
-
-val get_name : t -> int -> string option
-
-val propagate_name : t -> int -> int -> unit
-
-val set_pretty : t -> bool -> unit
-
-val set_stable : t -> bool -> unit
+val to_string : t -> int -> string
