@@ -22,7 +22,7 @@ open Util
 let%expect_test "test-compiler/lib-effects/test1.ml" =
   let code =
     compile_and_parse
-      ~effects:true
+      ~effects:`Cps
       {|
          let exceptions s =
            (* Compiled using 'try ... catch',
@@ -55,59 +55,62 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
   print_fun_decl code (Some "exceptions");
   [%expect
     {|
-
     function exceptions(s, cont){
-     try{var _k_ = runtime.caml_int_of_string(s), n = _k_;}
-     catch(_o_){
-      var _g_ = caml_wrap_exception(_o_);
-      if(_g_[1] !== Stdlib[7]){
-       var raise$1 = caml_pop_trap();
-       return raise$1(caml_maybe_attach_backtrace(_g_, 0));
+     try{var _d_ = runtime.caml_int_of_string(s), n = _d_;}
+     catch(exn$0){
+      var exn = caml_wrap_exception(exn$0), tag = exn[1];
+      if(tag !== Stdlib[7]){
+       var raise$1 = caml_pop_trap(), exn$2 = caml_maybe_attach_backtrace(exn, 0);
+       return raise$1(exn$2);
       }
       var n = 0;
      }
      try{
       if(caml_string_equal(s, cst$0))
        throw caml_maybe_attach_backtrace(Stdlib[8], 1);
-      var _j_ = 7, m = _j_;
+      var _c_ = 7, m = _c_;
      }
-     catch(_n_){
-      var _h_ = caml_wrap_exception(_n_);
-      if(_h_ !== Stdlib[8]){
-       var raise$0 = caml_pop_trap();
-       return raise$0(caml_maybe_attach_backtrace(_h_, 0));
+     catch(exn){
+      var exn$0 = caml_wrap_exception(exn);
+      if(exn$0 !== Stdlib[8]){
+       var
+        raise$0 = caml_pop_trap(),
+        exn$1 = caml_maybe_attach_backtrace(exn$0, 0);
+       return raise$0(exn$1);
       }
       var m = 0;
      }
      caml_push_trap
-      (function(_m_){
-        if(_m_ === Stdlib[8]) return cont(0);
-        var raise = caml_pop_trap();
-        return raise(caml_maybe_attach_backtrace(_m_, 0));
+      (function(exn){
+        if(exn === Stdlib[8]) return cont(0);
+        var raise = caml_pop_trap(), exn$0 = caml_maybe_attach_backtrace(exn, 0);
+        return raise(exn$0);
        });
      if(! caml_string_equal(s, cst))
-      return caml_cps_call2
+      return caml_trampoline_cps_call2
               (Stdlib[79],
                cst_toto,
-               function(_l_){caml_pop_trap(); return cont([0, [0, _l_, n, m]]);});
-     var _i_ = Stdlib[8], raise = caml_pop_trap();
-     return raise(caml_maybe_attach_backtrace(_i_, 1));
+               function(_d_){caml_pop_trap(); return cont([0, [0, _d_, n, m]]);});
+     var _b_ = Stdlib[8], raise = caml_pop_trap();
+     return raise(caml_maybe_attach_backtrace(_b_, 1));
     }
-    //end |}];
+    //end
+    |}];
   print_fun_decl code (Some "handler_is_loop");
   [%expect
     {|
     function handler_is_loop(f, g, l, cont){
      caml_push_trap
-      (function(_e_){
-        function _f_(l){
-         return caml_cps_call2
+      (function(exn){
+        function _b_(l){
+         return caml_trampoline_cps_call2
                  (g,
                   l,
                   function(match){
-                   if(72330306 <= match[1]){
+                   var variant = match[1];
+                   if(72330306 <= variant){
                     var l = match[2];
-                    return caml_cps_exact_call1(_f_, l);
+                    return caml_exact_trampoline_call1(_b_, l);
                    }
                    var
                     exn = match[2],
@@ -116,18 +119,23 @@ let%expect_test "test-compiler/lib-effects/test1.ml" =
                    return raise(exn$0);
                   });
         }
-        return _f_(l);
+        return _b_(l);
        });
-     return caml_cps_call2
-             (f, 0, function(_d_){caml_pop_trap(); return cont(_d_);});
+     return caml_trampoline_cps_call2
+             (f, 0, function(_b_){caml_pop_trap(); return cont(_b_);});
     }
-    //end |}];
+    //end
+    |}];
   print_fun_decl code (Some "handler_is_merge_node");
   [%expect
     {|
     function handler_is_merge_node(g, cont){
-     function _a_(s){return caml_cps_call3(Stdlib[28], s, cst_aaa, cont);}
-     caml_push_trap(function(_c_){return _a_(cst$1);});
-     return caml_cps_call2(g, 0, function(_b_){caml_pop_trap(); return _a_(_b_);});
+     function _a_(s){
+      return caml_trampoline_cps_call3(Stdlib[28], s, cst_aaa, cont);
+     }
+     caml_push_trap(function(exn){return _a_(cst$1);});
+     return caml_trampoline_cps_call2
+             (g, 0, function(_b_){caml_pop_trap(); return _a_(_b_);});
     }
-    //end |}]
+    //end
+    |}]
