@@ -27,8 +27,8 @@ let%expect_test "Eliminates unused functions from functor" =
                  ( Some
                      (EArr
                         (List.filter return ~f:(function
-                            | Javascript.ElementHole -> false
-                            | _ -> true)))
+                          | Javascript.ElementHole -> false
+                          | _ -> true)))
                  , loc )
              , loc' )
            in
@@ -41,17 +41,17 @@ let%expect_test "Eliminates unused functions from functor" =
     function create(l, v, r){
      if(l) var h = l[4], hl = h; else var hl = 0;
      if(r) var h$0 = r[4], hr = h$0; else var hr = 0;
-     var _k_ = hr <= hl ? hl + 1 | 0 : hr + 1 | 0;
-     return [0, l, v, r, _k_];
+     var _j_ = hr <= hl ? hl + 1 | 0 : hr + 1 | 0;
+     return [0, l, v, r, _j_];
     }
     function bal(l, v, r){
      if(l) var h = l[4], hl = h; else var hl = 0;
      if(r) var h$0 = r[4], hr = h$0; else var hr = 0;
      if((hr + 2 | 0) < hl){
-      if(! l) return invalid_arg(_b_);
+      if(! l) return invalid_arg(_c_);
       var lr = l[3], lv = l[2], ll = l[1], _f_ = height(lr);
       if(_f_ <= height(ll)) return create(ll, lv, create(lr, v, r));
-      if(! lr) return invalid_arg(_a_);
+      if(! lr) return invalid_arg(_b_);
       var lrr = lr[3], lrv = lr[2], lrl = lr[1], _g_ = create(lrr, v, r);
       return create(create(ll, lv, lrl), lrv, _g_);
      }
@@ -59,10 +59,10 @@ let%expect_test "Eliminates unused functions from functor" =
       var _j_ = hr <= hl ? hl + 1 | 0 : hr + 1 | 0;
       return [0, l, v, r, _j_];
      }
-     if(! r) return invalid_arg(_d_);
+     if(! r) return invalid_arg(_e_);
      var rr = r[3], rv = r[2], rl = r[1], _h_ = height(rl);
      if(_h_ <= height(rr)) return create(create(l, v, rl), rv, rr);
-     if(! rl) return invalid_arg(_c_);
+     if(! rl) return invalid_arg(_d_);
      var rlr = rl[3], rlv = rl[2], rll = rl[1], _i_ = create(rlr, rv, rr);
      return create(create(l, v, rll), rlv, _i_);
     }
@@ -75,32 +75,32 @@ let%expect_test "Eliminates unused functions from functor" =
      return l === ll ? t : bal(ll, v, r);
     }
     function singleton(x){return [0, 0, x, 0, 1];}
-    function find(x, param){
-     var param$0 = param;
+    function find(x, param$0){
+     var param = param$0;
      for(;;){
-      if(! param$0) throw caml_maybe_attach_backtrace(Not_found, 1);
-      var
-       r = param$0[3],
-       v = param$0[2],
-       l = param$0[1],
-       c = caml_call2(Ord[1], x, v);
+      if(! param) throw caml_maybe_attach_backtrace(Not_found, 1);
+      var r = param[3], v = param[2], l = param[1], c = caml_call2(Ord[1], x, v);
       if(0 === c) return v;
-      var r$0 = 0 <= c ? r : l;
-      param$0 = r$0;
+      param = 0 <= c ? r : l;
      }
     }
     return [0, 0, add, singleton, find];
-    //end |}]
+    //end
+    |}]
 
 let%expect_test "Omit unused fields" =
   let program =
     compile_and_parse
       {|
+      let l = ref []
+       
       let f b x =
+        l := (fun y -> x + y) :: !l; (* Prevent inlining *)
         let t = if b then (1, 2, x) else (3, x, 4) in
         let (u, _, v) = t in
         (u, v)
-      in print_int (fst (f true 1) + snd (f false 2))
+
+      let () = print_int (fst (f true 1) + snd (f false 2))
       |}
   in
   (* Expect second field in each triple to be omitted. *)
@@ -108,10 +108,12 @@ let%expect_test "Omit unused fields" =
   [%expect
     {|
     function f(b, x){
+     l[1] = [0, function(y){return x + y | 0;}, l[1]];
      var t = b ? [0, 1, , x] : [0, 3, , 4], v = t[3], u = t[1];
      return [0, u, v];
     }
-    //end |}]
+    //end
+    |}]
 
 let%expect_test "Omit unused return expressions" =
   let program =
