@@ -75,7 +75,7 @@ let print ?(debuginfo = true) ?(report = false) ?(invalid = false) ~compact sour
 
 let%expect_test "spread operator survives round-trip" =
   print ~compact:true "f(...[1, 2, 3])";
-  [%expect {| /*<<fake:1:0>>*/ /*<<fake:1:0>>*/f(...[1,2,3]); |}]
+  [%expect {| /*<<fake:1:0>>*/f(...[1,2,3]); |}]
 
 let%expect_test "no postfix addition coalesce" =
   print ~compact:true "a + +b";
@@ -104,14 +104,14 @@ let%expect_test "reserved words as fields" =
   |};
   [%expect
     {|
-    /*<<fake:2:4>>*/ x.debugger;
-    /*<<fake:3:4>>*/ x.catch;
-    /*<<fake:4:4>>*/ x.for;
-    /*<<fake:5:4>>*/ x.continue;
-    /*<<fake:6:4>>*/  /*<<fake:6:10>>*/ var y = {debugger: 2};
-    /*<<fake:7:4>>*/  /*<<fake:7:10>>*/ var y = {catch: 2};
-    /*<<fake:8:4>>*/  /*<<fake:8:10>>*/ var y = {for: 2};
-    /*<<fake:9:4>>*/  /*<<fake:9:10>>*/ var y = {continue: 2}; |}]
+     /*<<fake:2:4>>*/ x.debugger;
+     /*<<fake:3:4>>*/ x.catch;
+     /*<<fake:4:4>>*/ x.for;
+     /*<<fake:5:4>>*/ x.continue;
+    var y =  /*<<fake:6:10>>*/ {debugger: 2};
+    var y =  /*<<fake:7:10>>*/ {catch: 2};
+    var y =  /*<<fake:8:10>>*/ {for: 2};
+    var y =  /*<<fake:9:10>>*/ {continue: 2}; |}]
 
 let%expect_test "preserve number literals" =
   print
@@ -133,20 +133,20 @@ let%expect_test "preserve number literals" =
      var t = 1E+3; |};
   [%expect
     {|
-    /*<<fake:2:5>>*/  /*<<fake:2:11>>*/ var x = 0xf_fff;
-    /*<<fake:3:5>>*/  /*<<fake:3:11>>*/ var x = 0Xffff;
-    /*<<fake:4:5>>*/  /*<<fake:4:11>>*/ var y = 0o7_1223;
-    /*<<fake:5:5>>*/  /*<<fake:5:11>>*/ var y = 0O7123;
-    /*<<fake:6:5>>*/  /*<<fake:6:11>>*/ var y = 0b1_1001;
-    /*<<fake:7:5>>*/  /*<<fake:7:11>>*/ var y = 0B11001;
-    /*<<fake:8:5>>*/  /*<<fake:8:11>>*/ var y = 071923;
-    /*<<fake:9:5>>*/  /*<<fake:9:11>>*/ var y = 07123;
-    /*<<fake:10:5>>*/  /*<<fake:10:11>>*/ var z = 0.0;
-    /*<<fake:11:5>>*/  /*<<fake:11:11>>*/ var z = 0.;
-    /*<<fake:12:5>>*/  /*<<fake:12:11>>*/ var t = 1.0e-3;
-    /*<<fake:13:5>>*/  /*<<fake:13:11>>*/ var t = 1.0E+3;
-    /*<<fake:14:5>>*/  /*<<fake:14:11>>*/ var t = 1e-3;
-    /*<<fake:15:5>>*/  /*<<fake:15:11>>*/ var t = 1E+3; |}]
+    var x =  /*<<fake:2:11>>*/ 0xf_fff;
+    var x =  /*<<fake:3:11>>*/ 0Xffff;
+    var y =  /*<<fake:4:11>>*/ 0o7_1223;
+    var y =  /*<<fake:5:11>>*/ 0O7123;
+    var y =  /*<<fake:6:11>>*/ 0b1_1001;
+    var y =  /*<<fake:7:11>>*/ 0B11001;
+    var y =  /*<<fake:8:11>>*/ 071923;
+    var y =  /*<<fake:9:11>>*/ 07123;
+    var z =  /*<<fake:10:11>>*/ 0.0;
+    var z =  /*<<fake:11:11>>*/ 0.;
+    var t =  /*<<fake:12:11>>*/ 1.0e-3;
+    var t =  /*<<fake:13:11>>*/ 1.0E+3;
+    var t =  /*<<fake:14:11>>*/ 1e-3;
+    var t =  /*<<fake:15:11>>*/ 1E+3; |}]
 
 let%expect_test "preserve number literals in property_name" =
   print
@@ -157,10 +157,8 @@ let%expect_test "preserve number literals in property_name" =
  |};
   [%expect
     {|
-    /*<<fake:2:4>>*/  /*<<fake:2:22>>*/ var
-    number_as_key = {100000000000000000000: 2};
-    /*<<fake:3:4>>*/  /*<<fake:3:22>>*/ var
-    number_as_key = {100000000000000000000n: 2}; |}]
+    var number_as_key =  /*<<fake:2:22>>*/ {100000000000000000000: 2};
+    var number_as_key =  /*<<fake:3:22>>*/ {100000000000000000000n: 2}; |}]
 
 let%expect_test "ops" =
   print
@@ -204,7 +202,7 @@ let%expect_test "ops" =
     /*<<fake:15:4>>*/ y = a?.b?.s?.[a] ?? c ?? d;
     /*<<fake:17:4>>*/ a?.b;
     /*<<fake:18:4>>*/ a?.[b];
-    /*<<fake:19:4>>*/  /*<<fake:19:4>>*/ a?.(b); |}]
+    /*<<fake:19:4>>*/ a?.(b); |}]
 
 let%expect_test "arrow" =
   print
@@ -234,27 +232,34 @@ let%expect_test "arrow" =
 
   [%expect
     {|
-    /*<<fake:2:4>>*/  /*<<fake:2:10>>*/ var a = x=> /*<<fake:2:18>>*/ x + 2;
-    /*<<fake:3:4>>*/  /*<<fake:3:10>>*/ var a = ()=> /*<<fake:3:19>>*/ 2;
-    /*<<fake:4:4>>*/  /*<<fake:4:10>>*/ var a = x=> /*<<fake:4:20>>*/ x + 2;
-    /*<<fake:5:4>>*/  /*<<fake:5:10>>*/ var a = (x, y)=> /*<<fake:5:22>>*/ x + y;
-    /*<<fake:7:4>>*/  /*<<fake:7:10>>*/ var
-    a = x=>{ /*<<fake:7:20>>*/ x + 2; /*<<fake:7:13>>*/ };
-    /*<<fake:8:4>>*/  /*<<fake:8:10>>*/ var
-    a = ()=>{ /*<<fake:8:21>>*/ 2; /*<<fake:8:13>>*/ };
-    /*<<fake:9:4>>*/  /*<<fake:9:10>>*/ var
-    a = x=>{ /*<<fake:9:22>>*/ x + 2; /*<<fake:9:13>>*/ };
-    /*<<fake:11:4>>*/  /*<<fake:11:10>>*/ var
-    a = ( /*<<fake:11:16>>*/ x = 1 / 2)=> /*<<fake:11:28>>*/ x + 10;
-    /*<<fake:13:4>>*/  /*<<fake:13:10>>*/ var
-    a = ( /*<<fake:13:16>>*/ x = /qwe/g)=> /*<<fake:13:30>>*/ x + 10;
-    /*<<fake:16:4>>*/  /*<<fake:16:10>>*/ var
-    a = x=> /*<<fake:16:17>>*/ y=> /*<<fake:16:22>>*/ x + y;
-    /*<<fake:17:4>>*/  /*<<fake:17:10>>*/ var
-    a = x=> /*<<fake:17:17>>*/ y=> /*<<fake:17:23>>*/ x + y;
-    /*<<fake:19:4>>*/  /*<<fake:19:10>>*/ var a = async x=> /*<<fake:19:23>>*/ y;
-    /*<<fake:20:4>>*/  /*<<fake:20:10>>*/ var
-    a = async (a, b)=> /*<<fake:20:27>>*/ a + b; |}]
+    var a =  /*<<fake:2:10>>*/ x=> /*<<fake:2:18>>*/ x + 2 /*<<fake:2:23>>*/ ;
+    var a =  /*<<fake:3:10>>*/ ()=> /*<<fake:3:19>>*/ 2 /*<<fake:3:20>>*/ ;
+    var a =  /*<<fake:4:10>>*/ x=> /*<<fake:4:20>>*/ x + 2 /*<<fake:4:25>>*/ ;
+    var
+     a =  /*<<fake:5:10>>*/ (x, y)=> /*<<fake:5:22>>*/ x + y /*<<fake:5:27>>*/ ;
+    var a =  /*<<fake:7:10>>*/ x=>{ /*<<fake:7:20>>*/ x + 2; /*<<fake:7:13>>*/ };
+    var a =  /*<<fake:8:10>>*/ ()=>{ /*<<fake:8:21>>*/ 2; /*<<fake:8:13>>*/ };
+    var a =  /*<<fake:9:10>>*/ x=>{ /*<<fake:9:22>>*/ x + 2; /*<<fake:9:13>>*/ };
+    var
+     a =
+        /*<<fake:11:10>>*/ (x =  /*<<fake:11:16>>*/ 1 / 2)=> /*<<fake:11:28>>*/ x + 10 /*<<fake:11:34>>*/ ;
+    var
+     a =
+        /*<<fake:13:10>>*/ (x =  /*<<fake:13:16>>*/ /qwe/g)=> /*<<fake:13:30>>*/ x + 10 /*<<fake:13:36>>*/ ;
+    var
+     a =
+        /*<<fake:16:10>>*/ x=>
+           /*<<fake:16:17>>*/ y=> /*<<fake:16:22>>*/ x + y /*<<fake:16:27>>*/ ;
+    var
+     a =
+        /*<<fake:17:10>>*/ x=>
+           /*<<fake:17:17>>*/ y=> /*<<fake:17:23>>*/ x + y /*<<fake:17:28>>*/  /*<<fake:17:29>>*/ ;
+    var
+     a =  /*<<fake:19:10>>*/ async x=> /*<<fake:19:23>>*/ y /*<<fake:19:24>>*/ ;
+    var
+     a =
+        /*<<fake:20:10>>*/ async (a, b)=>
+           /*<<fake:20:27>>*/ a + b /*<<fake:20:32>>*/ ; |}]
 
 let%expect_test "trailing comma" =
   (* GH#989 *)
@@ -286,21 +291,20 @@ function rehb_shape(_face /*: fk_face */, text /*: string */) {
 
   [%expect
     {|
-    /*<<fake:4:0>>*/ function rehb_new_face(_fontName){
-     /*<<fake:7:2>>*/ return undefined;
-    /*<<fake:8:0>>*/ }
-    /*<<fake:12:0>>*/ function rehb_shape(_face, text){
-     /*<<fake:13:2>>*/  /*<<fake:13:10>>*/ var
-     str =  /*<<fake:13:12>>*/ caml_to_js_string(text);
-     /*<<fake:14:2>>*/  /*<<fake:14:10>>*/ var
-     ret =
-        /*<<fake:14:12>>*/  /*<<fake:14:12>>*/ str.split("").map
-        (function mapper(_char){
-           /*<<fake:15:6>>*/ return [0, 0, 0];
-          /*<<fake:14:30>>*/ });
-     /*<<fake:19:2>>*/  /*<<fake:19:2>>*/ ret.unshift(0);
-     /*<<fake:20:2>>*/ return ret;
-    /*<<fake:21:0>>*/ } |}]
+    function rehb_new_face(_fontName){
+      /*<<fake:7:2>>*/ return undefined /*<<fake:7:18>>*/ ;
+     /*<<fake:8:0>>*/ }
+    function rehb_shape(_face, text){
+     var str =  /*<<fake:13:10>>*/  /*<<fake:13:12>>*/ caml_to_js_string(text);
+     var
+      ret =
+         /*<<fake:14:10>>*/  /*<<fake:14:12>>*/ str.split("").map
+         (function mapper(_char){
+            /*<<fake:15:6>>*/ return [0, 0, 0] /*<<fake:15:69>>*/ ;
+           /*<<fake:14:30>>*/ });
+      /*<<fake:19:2>>*/ ret.unshift(0);
+      /*<<fake:20:2>>*/ return ret /*<<fake:20:12>>*/ ;
+     /*<<fake:21:0>>*/ } |}]
 
 let%expect_test "rest parameters" =
   (* GH#1031 *)
@@ -319,12 +323,11 @@ let%expect_test "rest parameters" =
     {|
     /*<<fake:2:6>>*/ api_obj[key_module][key_func] =
     function(...args){
-      /*<<fake:3:8>>*/ return  /*<<fake:3:15>>*/  /*<<fake:3:15>>*/ checkIfInitialized
-              ().then
+      /*<<fake:3:8>>*/ return  /*<<fake:3:15>>*/ checkIfInitialized().then
              (function(){
                 /*<<fake:4:10>>*/ return  /*<<fake:4:17>>*/ callWithProto
-                       (api_json[key_module][key_func], args);
-               /*<<fake:3:41>>*/ });
+                       (api_json[key_module][key_func], args) /*<<fake:4:68>>*/ ;
+               /*<<fake:3:41>>*/ }) /*<<fake:5:10>>*/ ;
      /*<<fake:2:38>>*/ }; |}]
 
 let%expect_test "async/await" =
@@ -359,24 +362,25 @@ let%expect_test "async/await" =
 
   [%expect
     {|
-    /*<<fake:2:9>>*/ async function compile(src){
-     /*<<fake:4:11>>*/  /*<<fake:4:31>>*/ const
-     glslangModule =
-       await
-        /*<<fake:4:39>>*/ import
-        ("https://unpkg.com/@webgpu/glslang@0.0.7/web/glslang.js");
-     /*<<fake:7:11>>*/  /*<<fake:7:25>>*/ const
-     glslang = await  /*<<fake:7:33>>*/ glslangModule.default();
-     /*<<fake:8:11>>*/ return  /*<<fake:8:18>>*/ glslang.compileGLSL
-            (src, "compute");
-    /*<<fake:2:9>>*/ }
-    /*<<fake:12:4>>*/ async;
-    /*<<fake:13:4>>*/ function test(){ /*<<fake:13:22>>*/ }
-    /*<<fake:15:4>>*/ async function test(){ /*<<fake:15:4>>*/ }
-    /*<<fake:17:4>>*/ async;
-    /*<<fake:18:4>>*/ function* test(){ /*<<fake:18:4>>*/ }
-    /*<<fake:20:4>>*/ async function* test(){ /*<<fake:20:4>>*/ }
-    /*<<fake:22:4>>*/ 1 + async function* test(){ /*<<fake:22:8>>*/ }; |}]
+    async function compile(src){
+     const
+      glslangModule =
+         /*<<fake:4:31>>*/ await
+         /*<<fake:4:39>>*/ import
+         ("https://unpkg.com/@webgpu/glslang@0.0.7/web/glslang.js");
+     const
+      glslang =
+         /*<<fake:7:25>>*/ await  /*<<fake:7:33>>*/ glslangModule.default();
+      /*<<fake:8:11>>*/ return  /*<<fake:8:18>>*/ glslang.compileGLSL
+             (src, "compute") /*<<fake:8:53>>*/ ;
+     /*<<fake:2:9>>*/ }
+     /*<<fake:12:4>>*/ async;
+    function test(){ /*<<fake:13:22>>*/ }
+    async function test(){ /*<<fake:15:4>>*/ }
+     /*<<fake:17:4>>*/ async;
+    function* test(){ /*<<fake:18:4>>*/ }
+    async function* test(){ /*<<fake:20:4>>*/ }
+     /*<<fake:22:4>>*/ 1 + async function* test(){ /*<<fake:22:8>>*/ }; |}]
 
 let%expect_test "get/set property" =
   (* GH#1017 *)
@@ -399,16 +403,24 @@ let%expect_test "get/set property" =
 
   [%expect
     {|
-    /*<<fake:2:5>>*/  /*<<fake:2:11>>*/ var
-    x =
-      {get prop(){ /*<<fake:3:20>>*/ return 3; /*<<fake:3:7>>*/ },
-       set prop(x){ /*<<fake:4:21>>*/ return x == 2; /*<<fake:4:7>>*/ },
-       a: 4,
-       b(){ /*<<fake:6:13>>*/ return 5; /*<<fake:6:7>>*/ },
-       * e(){ /*<<fake:7:14>>*/ return 5; /*<<fake:7:7>>*/ },
-       async e(){ /*<<fake:8:19>>*/ return 5; /*<<fake:8:7>>*/ },
-       async* e(){ /*<<fake:9:20>>*/ return 5; /*<<fake:9:7>>*/ },
-       ["field" + 1]: 3}; |}]
+    var
+     x =
+        /*<<fake:2:11>>*/ {get prop(){
+          /*<<fake:3:20>>*/ return 3 /*<<fake:3:28>>*/ ;
+         /*<<fake:3:7>>*/ },
+        set prop(x){
+          /*<<fake:4:21>>*/ return x == 2 /*<<fake:4:34>>*/ ;
+         /*<<fake:4:7>>*/ },
+        a: 4,
+        b(){ /*<<fake:6:13>>*/ return 5 /*<<fake:6:21>>*/ ; /*<<fake:6:7>>*/ },
+        * e(){ /*<<fake:7:14>>*/ return 5 /*<<fake:7:22>>*/ ; /*<<fake:7:7>>*/ },
+        async e(){
+          /*<<fake:8:19>>*/ return 5 /*<<fake:8:27>>*/ ;
+         /*<<fake:8:7>>*/ },
+        async* e(){
+          /*<<fake:9:20>>*/ return 5 /*<<fake:9:28>>*/ ;
+         /*<<fake:9:7>>*/ },
+        ["field" + 1]: 3}; |}]
 
 let%expect_test "assignment pattern" =
   (* GH#1017 *)
@@ -437,19 +449,17 @@ let%expect_test "assignment pattern" =
 
   [%expect
     {|
-    /*<<fake:2:4>>*/ var x, y, rest;
-    /*<<fake:3:4>>*/  /*<<fake:3:14>>*/ var [x, y] = [1, 2];
-    /*<<fake:4:4>>*/  /*<<fake:4:22>>*/ var [x, y, ...rest] = [1, 2, ...o];
-    /*<<fake:6:4>>*/  /*<<fake:6:14>>*/ var {x, y} = {x: 1, y: 2};
-    /*<<fake:7:4>>*/  /*<<fake:7:22>>*/ var {x, y, ...rest} = {x: 1, y: 2, ...o};
-    /*<<fake:9:4>>*/ [x, y] = [1, 2];
-    /*<<fake:10:4>>*/ [x, y, ...rest] = [1, 2];
-    /*<<fake:12:4>>*/ ({x, y} = {x: 1, y: 2});
-    /*<<fake:13:4>>*/ ({x, y, ...rest} = {x: 1, y: 2});
-    /*<<fake:15:4>>*/ for([a, b, {c, d = e, [f]: [g, h, a, i, j]}] in 3)
-     /*<<fake:15:43>>*/ ;
-    /*<<fake:17:4>>*/ for([a, b, {c, d = e, [f]: [g, h, a, i, j]}] of 3)
-     /*<<fake:17:43>>*/ ; |}]
+    var x, y, rest;
+    var [x, y] = /*<<fake:3:14>>*/  [1, 2];
+    var [x, y, ...rest] = /*<<fake:4:22>>*/  [1, 2, ...o];
+    var {x, y} = /*<<fake:6:14>>*/  {x: 1, y: 2};
+    var {x, y, ...rest} = /*<<fake:7:22>>*/  {x: 1, y: 2, ...o};
+     /*<<fake:9:4>>*/ [x, y] = [1, 2];
+     /*<<fake:10:4>>*/ [x, y, ...rest] = [1, 2];
+     /*<<fake:12:4>>*/ ({x, y} = {x: 1, y: 2});
+     /*<<fake:13:4>>*/ ({x, y, ...rest} = {x: 1, y: 2});
+     /*<<fake:15:4>>*/ for([a, b, {c, d = e, [f]: [g, h, a, i, j]}] in 3) ;
+     /*<<fake:17:4>>*/ for([a, b, {c, d = e, [f]: [g, h, a, i, j]}] of 3) ; |}]
 
 let%expect_test "for loops" =
   (* GH#1017 *)
@@ -466,11 +476,9 @@ let%expect_test "for loops" =
 
   [%expect
     {|
-    /*<<fake:2:4>>*/ for(x in 3)  /*<<fake:2:15>>*/ ;
-    /*<<fake:3:4>>*/ for(x of 3)  /*<<fake:3:15>>*/ ;
-    /*<<fake:4:4>>*/ async function f(x){
-     /*<<fake:5:4>>*/ for await(x of 3)  /*<<fake:5:21>>*/ ;
-    /*<<fake:4:4>>*/ } |}]
+     /*<<fake:2:4>>*/ for(x in 3) ;
+     /*<<fake:3:4>>*/ for(x of 3) ;
+    async function f(x){ /*<<fake:5:4>>*/ for await(x of 3) ; /*<<fake:4:4>>*/ } |}]
 
 let%expect_test "string template" =
   (* GH#1017 *)
@@ -488,12 +496,12 @@ let%expect_test "string template" =
 
   [%expect
     {|
-    /*<<fake:2:4>>*/  /*<<fake:2:10>>*/ var s = `asdte`;
-    /*<<fake:3:4>>*/  /*<<fake:3:10>>*/ var s = `asd ${test} te`;
-    /*<<fake:5:4>>*/  /*<<fake:5:10>>*/ var
-    s =  /*<<fake:5:12>>*/ tag`asd ${test} te`;
-    /*<<fake:7:4>>*/  /*<<fake:7:10>>*/ var
-    s = `asd ${ /*<<fake:7:20>>*/ f(`space ${test} space`, 32)} te`; |}]
+    var s =  /*<<fake:2:10>>*/ `asdte`;
+    var s =  /*<<fake:3:10>>*/ `asd ${test} te`;
+    var s =  /*<<fake:5:10>>*/  /*<<fake:5:12>>*/ tag`asd ${test} te`;
+    var
+     s =
+        /*<<fake:7:10>>*/ `asd ${ /*<<fake:7:20>>*/ f(`space ${test} space`, 32)} te`; |}]
 
 let%expect_test "from keyword" =
   (* GH#1017 *)
@@ -513,13 +521,15 @@ let%expect_test "from keyword" =
      function from(field, get){
        /*<<fake:5:6>>*/ if(! get)
         /*<<fake:5:14>>*/ get =
-        function get(x){ /*<<fake:5:34>>*/ return x; /*<<fake:5:18>>*/ };
+        function get(x){
+          /*<<fake:5:34>>*/ return x /*<<fake:5:42>>*/ ;
+         /*<<fake:5:18>>*/ };
        /*<<fake:6:6>>*/ return  /*<<fake:6:13>>*/ this.compute
               ([field],
                function(state){
                  /*<<fake:6:50>>*/ return  /*<<fake:6:57>>*/ get
-                        ( /*<<fake:6:61>>*/ state.field(field));
-                /*<<fake:6:34>>*/ });
+                        ( /*<<fake:6:61>>*/ state.field(field)) /*<<fake:6:80>>*/ ;
+                /*<<fake:6:34>>*/ }) /*<<fake:6:83>>*/ ;
       /*<<fake:4:3>>*/ }}); |}]
 
 let%expect_test "new.target" =
@@ -529,7 +539,7 @@ let%expect_test "new.target" =
  |};
 
   [%expect {|
-    /*<<fake:2:4>>*/  /*<<fake:2:10>>*/ var s = new.target; |}]
+    var s =  /*<<fake:2:10>>*/ new.target; |}]
 
 let%expect_test "super" =
   (* GH#1017 *)
@@ -566,25 +576,23 @@ class x extends p {
 
   [%expect
     {|
-    /*<<fake:2:0>>*/ class x extends p {
-      constructor(){
-        /*<<fake:4:6>>*/  /*<<fake:4:6>>*/ super(a, b, c);
-       /*<<fake:3:4>>*/ }
-      foo(){
-        /*<<fake:8:6>>*/  /*<<fake:8:12>>*/ var s = super[d];
-        /*<<fake:9:6>>*/  /*<<fake:9:12>>*/ var s = super.d;
-       /*<<fake:6:4>>*/ }
-      static
-      bar(){
-        /*<<fake:14:6>>*/  /*<<fake:14:12>>*/ var s = super[d];
-        /*<<fake:15:6>>*/  /*<<fake:15:12>>*/ var s = super.d;
-       /*<<fake:12:11>>*/ }
-      x =  /*<<fake:17:5>>*/ 3;
-      static y =  /*<<fake:19:12>>*/ 5;
-      #z =  /*<<fake:21:6>>*/ 6;
-      static #t =  /*<<fake:23:13>>*/ 2;
-      static { /*<<fake:25:12>>*/  /*<<fake:25:18>>*/ var x = 3;}
-    } |}]
+    class x extends p {
+       constructor(){ /*<<fake:4:6>>*/ super(a, b, c); /*<<fake:3:4>>*/ }
+       foo(){
+        var s =  /*<<fake:8:12>>*/ super[d];
+        var s =  /*<<fake:9:12>>*/ super.d;
+        /*<<fake:6:4>>*/ }
+       static
+       bar(){
+        var s =  /*<<fake:14:12>>*/ super[d];
+        var s =  /*<<fake:15:12>>*/ super.d;
+        /*<<fake:12:11>>*/ }
+       x =  /*<<fake:17:5>>*/ 3;
+       static y =  /*<<fake:19:12>>*/ 5;
+       #z =  /*<<fake:21:6>>*/ 6;
+       static #t =  /*<<fake:23:13>>*/ 2;
+       static {var x =  /*<<fake:25:18>>*/ 3;}
+     } |}]
 
 let%expect_test "ite" =
   print
