@@ -31,10 +31,11 @@ let print_macro_transformed source =
       in
       let parsed = Util.parse_js source in
       let transformed, _ = Jsoo.Macro.f ~flags:false parsed in
-      let (_ : Jsoo.Source_map.t option) = Jsoo.Js_output.program pp transformed in
+      let (_ : Jsoo.Source_map.info) = Jsoo.Js_output.program pp transformed in
       print_endline (Buffer.contents buffer))
 
 let print_macro_transformed source =
+  Jsoo.Targetint.set_num_bits 32;
   try print_macro_transformed source with Failure s -> Format.printf "failure: %s%!" s
 
 let%expect_test "BLOCK()" =
