@@ -157,9 +157,10 @@ let%expect_test "static eval of string get" =
       param$0 = next$0;
      }
     }
-    //end |}]
+    //end
+    |}]
 
-let%expect_test "static eval of tags" =
+let%expect_test "static eval of tags (optimized switch)" =
   let program =
     compile_and_parse
       {|
@@ -191,18 +192,121 @@ let%expect_test "static eval of tags" =
        }
        var
         global_data = runtime.caml_get_global_data(),
-        Stdlib_Random = global_data.Stdlib__Random,
-        _a_ = [0, [1, 0]],
-        _b_ = [1, 0],
-        x = 1 < caml_call1(Stdlib_Random[5], 3) ? _a_ : _b_;
-       x[0];
-       var
-        foobar = 3,
-        export$0 = [0, foobar, foobar],
-        Test = [0, foobar, export$0];
+        Stdlib_Random = global_data.Stdlib__Random;
+       caml_call1(Stdlib_Random[5], 3);
+       var foobar = 3, export$ = [0, foobar, foobar], Test = [0, foobar, export$];
        runtime.caml_register_global(3, Test, "Test");
        return;
       }
       (globalThis));
     //end
     |}]
+
+let%expect_test "static eval of tags" =
+  let program =
+    compile_and_parse
+      {|
+
+      type t = A | B | C of t | D of t | E of t | F of t
+
+      let foobar =
+        let x = if Random.int 3 > 1 then C (D A) else D (A) in
+        match x with
+        | A -> 1
+        | B -> 2
+        | C _
+        | D _ -> 3
+        | E _ -> 5
+        | F _ -> 7
+
+      let export = [|foobar;foobar|]
+  |}
+  in
+  print_program program;
+  [%expect
+    {|
+    (function(globalThis){
+       "use strict";
+       var runtime = globalThis.jsoo_runtime;
+       function caml_call1(f, a0){
+        return (f.l >= 0 ? f.l : f.l = f.length) === 1
+                ? f(a0)
+                : runtime.caml_call_gen(f, [a0]);
+       }
+       var
+        global_data = runtime.caml_get_global_data(),
+        Stdlib_Random = global_data.Stdlib__Random;
+       caml_call1(Stdlib_Random[5], 3);
+       var foobar = 3, export$ = [0, foobar, foobar], Test = [0, foobar, export$];
+       runtime.caml_register_global(3, Test, "Test");
+       return;
+      }
+      (globalThis));
+    //end
+    |}]
+
+let%expect_test "static eval int prims" =
+  let program =
+    compile_and_parse
+      {|
+
+      let lt =
+        let x = if Random.int 3 > 1 then 1 else 2 in
+        x < 5
+
+      let le =
+         let x = if Random.int 3 > 1 then 1 else 2 in
+         x <= 5
+
+      let eq =
+        let x = if Random.int 3 > 1 then 1 else 2 in
+        x = 3
+
+      let neq =
+        let x = if Random.int 3 > 1 then 1 else 2 in
+       x <> 3
+
+      type ult = A | B | C | D
+
+      let ult =
+        let x = if Random.int 3 > 1 then A else D in
+        match x with
+        | A | D -> true
+        | B | C -> false
+
+      let export = [| lt; le; eq; neq; ult |]
+  |}
+  in
+  print_program program;
+  [%expect
+    {|
+    (function(globalThis){
+       "use strict";
+       var runtime = globalThis.jsoo_runtime;
+       function caml_call1(f, a0){
+        return (f.l >= 0 ? f.l : f.l = f.length) === 1
+                ? f(a0)
+                : runtime.caml_call_gen(f, [a0]);
+       }
+       var
+        global_data = runtime.caml_get_global_data(),
+        Stdlib_Random = global_data.Stdlib__Random;
+       caml_call1(Stdlib_Random[5], 3);
+       var lt = 1;
+       caml_call1(Stdlib_Random[5], 3);
+       var le = 1;
+       caml_call1(Stdlib_Random[5], 3);
+       var eq = 0;
+       caml_call1(Stdlib_Random[5], 3);
+       var neq = 1;
+       caml_call1(Stdlib_Random[5], 3);
+       var
+        ult = 1,
+        export$ = [0, lt, le, eq, neq, ult],
+        Test = [0, lt, le, eq, neq, ult, export$];
+       runtime.caml_register_global(1, Test, "Test");
+       return;
+      }
+      (globalThis));
+    //end
+    |}]
