@@ -142,7 +142,8 @@ let%expect_test _ =
      }
      return 1;
     }
-    //end |}];
+    //end
+    |}];
   with_temp_dir ~f:(fun () -> print_fun_decl (program ~enable:false) (Some "match_expr"));
   [%expect
     {|
@@ -164,4 +165,5 @@ let%expect_test _ =
      }
      return 1;
     }
-    //end |}]
+    //end
+    |}]
