@@ -57,7 +57,10 @@ let%expect_test _ =
   let tmp_filename = Filename.temp_file "out" "txt" in
   let chan = open_out_bin tmp_filename in
   let v1 = Op (Add, [ Literal (Numeral (SPlus, 5)); Literal (Numeral (SMinus, 7)) ]) in
-  let v2 = Op (Times, [ v1; v1 ]) (* shared *) in
+  let v2 =
+    Op (Times, [ v1; v1 ])
+    (* shared *)
+  in
   let v1_sz = write_out chan v1 in
   let v2_sz = write_out chan v2 in
   let v2_ns_sz = write_out_noshare chan v2 in
@@ -168,3 +171,19 @@ let%expect_test "test float" =
     {|
     "\132\149\166\190\000\000\000\017\000\000\000\004\000\000\000\012\000\000\000\011\160\160\012\031\133\235Q\184\030\t@\004\001\160\004\003@"
     3.140000 3.140000 3.140000 3.140000 |}]
+
+let%expect_test "test input with offset" =
+  let s = 3.14 in
+  let b = Bytes.create 1000 in
+  let prefix = "not-marshal" in
+  Bytes.blit_string prefix 0 b 0 (String.length prefix);
+  let _ =
+    Marshal.to_buffer
+      b
+      (String.length prefix)
+      (Bytes.length b - String.length prefix)
+      s
+      []
+  in
+  Printf.printf "%f" (Marshal.from_bytes b (String.length prefix));
+  [%expect {| 3.140000 |}]
