@@ -1,5 +1,4 @@
-(* TEST
-*)
+(* TEST *)
 
 open Printf
 
@@ -12,6 +11,8 @@ module type S = sig
   val make : int -> float -> t
   val create : int -> t
   val init : int -> (int -> float) -> t
+  val make_matrix : int -> int -> float -> t array
+  val init_matrix : int -> int -> (int -> int -> float) -> t array
   val append : t -> t -> t
   val concat : t list -> t
   val sub : t -> int -> int -> t
@@ -23,7 +24,9 @@ module type S = sig
   val iter : (float -> unit) -> t -> unit
   val iteri : (int -> float -> unit) -> t -> unit
   val map : (float -> float) -> t -> t
+  val map_inplace : (float -> float) -> t -> unit
   val mapi : (int -> float -> float) -> t -> t
+  val mapi_inplace : (int -> float -> float) -> t -> unit
   val fold_left : ('a -> float -> 'a) -> 'a -> t -> 'a
   val fold_right : (float -> 'a -> 'a) -> t -> 'a -> 'a
   val iter2 : (float -> float -> unit) -> t -> t -> unit
@@ -32,6 +35,10 @@ module type S = sig
   val exists : (float -> bool) -> t -> bool
   val mem : float -> t -> bool
   val mem_ieee : float -> t -> bool
+  val find_opt : (float -> bool) -> t -> float option
+  val find_index : (float-> bool) -> t -> int option
+  val find_map : (float -> 'a option) -> t -> 'a option
+  val find_mapi : (int -> float -> 'a option) -> t -> 'a option
   val sort : (float -> float -> int) -> t -> unit
   val stable_sort : (float -> float -> int) -> t -> unit
   val fast_sort : (float -> float -> int) -> t -> unit
@@ -100,9 +107,9 @@ module Test (A : S) : sig end = struct
   check_inval (fun i -> A.set a i 1.0) (-1);
   check_inval (fun i -> A.set a i 1.0) 1000;
   check_inval A.create (-1);
-  (* check_inval A.create (A.max_length + 1);*)
+  check_inval A.create (A.max_length + 1);
   check_inval (fun i -> A.make i 1.0) (-1);
-  (* check_inval (fun i -> A.make i 1.0) (A.max_length + 1); *)
+  check_inval (fun i -> A.make i 1.0) (A.max_length + 1);
 
   (* [length] *)
   let test_length l = assert (l = (A.length (A.create l))) in
@@ -118,7 +125,48 @@ module Test (A : S) : sig end = struct
   let a = A.init 1000 Float.of_int in
   check_i a;
   check_inval (fun i -> A.init i Float.of_int) (-1);
-  (* check_inval (fun i -> A.init i Float.of_int) (A.max_length + 1); *)
+  check_inval (fun i -> A.init i Float.of_int) (A.max_length + 1);
+
+  (* [make_matrix] *)
+  let check_make_matrix m n =
+    let a = A.make_matrix m n 42. in
+    assert (Array.length a = m);
+    for i = 0 to m-1 do
+      let row = Array.get a i in
+      assert (A.length row = n);
+      for j = 0 to n-1 do
+        assert (A.get row j = 42.);
+        A.set row j (Float.of_int (i*n + j));
+      done;
+    done;
+    (* check absence of sharing: *)
+    if n > 0 then begin
+      for i = 0 to m-1 do
+        assert (A.get (Array.get a i) 0 = Float.of_int (i*n));
+      done
+    end
+  in
+  check_make_matrix 0 0;
+  check_make_matrix 0 3;
+  check_make_matrix 5 0;
+  check_make_matrix 5 3;
+
+  (* [init_matrix] *)
+  let check_init_matrix m n =
+    let a = A.init_matrix m n (fun i j -> Float.of_int (i*n + j)) in
+    assert (Array.length a = m);
+    for i = 0 to m-1 do
+      let row = Array.get a i in
+      assert (A.length row = n);
+      for j = 0 to n-1 do
+        assert (A.get row j = Float.of_int (i*n + j));
+      done;
+    done;
+  in
+  check_init_matrix 0 0;
+  check_init_matrix 0 3;
+  check_init_matrix 5 0;
+  check_init_matrix 5 3;
 
   (* [append] *)
   let check m n =
@@ -368,6 +416,61 @@ module Test (A : S) : sig end = struct
   A.set a 0 nan;
   assert (not (A.mem_ieee nan a));
 
+  (* [find_opt], test result and order of evaluation *)
+  let a = A.init 777 Float.of_int in
+  let r = ref 0.0 in
+  let f x =
+    assert (x = !r);
+    r := x +. 1.0;
+    false
+  in
+  assert (Option.is_none (A.find_opt f a));
+  let f x = assert (x = 0.0); true in
+  assert (Option.is_some (A.find_opt f a));
+
+  (* [find_index], test result and order of evaluation *)
+  let a = A.init 777 Float.of_int in
+  let r = ref 0.0 in
+  let f x =
+    assert (x = !r);
+    r := x +. 1.0;
+    false
+  in
+  assert (Option.is_none (A.find_index f a));
+  let f x = assert (x = 0.0); true in
+  assert (Option.get (A.find_index f a) = 0);
+
+  (* [find_map], test result and order of evaluation *)
+  let a = A.init 777 Float.of_int in
+  let r = ref 0.0 in
+  let f x =
+    assert (x = !r);
+    r := x +. 1.0;
+    None
+  in
+  assert (Option.is_none (A.find_map f a));
+  let f x = assert (x = 0.0); Some "abc" in
+  assert (Option.get (A.find_map f a) = "abc");
+
+  (* [find_mapi], test result and order of evaluation *)
+  let a = A.init 777 Float.of_int in
+  let r = ref 0.0 in
+  let r_i = ref 0 in
+  let f i x =
+    assert (i = !r_i);
+    assert (x = !r);
+    r_i := !r_i + 1;
+    r := x +. 1.0;
+    None
+  in
+  assert (Option.is_none (A.find_mapi f a));
+  let f i x =
+    assert (i = 0);
+    assert (x = 0.0);
+    Some "abc"
+  in
+  assert (Option.get (A.find_mapi f a) = "abc");
+
   (* [sort] [fast_sort] [stable_sort] *)
   let check_sort sort cmp a =
     let rec check_sorted a i =
@@ -537,8 +640,21 @@ module Test (A : S) : sig end = struct
     assert (compare value value' = 0)
   in
   let l = [0.; 0.25; -4.; 3.14159265; nan; infinity; neg_infinity; neg_zero] in
-  (* js_of_ocaml doesn't marshal floats *)
-  if false then test_structured_io (A.of_list l);
+  test_structured_io (A.of_list l);
+
+  (* map_inplace *)
+  let a = A.init 4 (fun i -> Float.of_int (i + 1)) in
+  A.map_inplace (fun x -> 2. *. x) a;
+  let got = A.map_to_array Fun.id a in
+  let expected = [|2.; 4.; 6.; 8.|] in
+  assert (Array.for_all2 Float.equal got expected);
+
+  (* mapi_inplace *)
+  let a = A.init 4 (fun i -> Float.of_int (i + 1)) in
+  A.mapi_inplace (fun i x -> 1. +. (Float.of_int i) +. x) a;
+  let got = A.map_to_array Fun.id a in
+  let expected = [|2.; 4.; 6.; 8.|] in
+  assert (Array.for_all2 Float.equal got expected)
 
 end
 
