@@ -1,5 +1,6 @@
 (* TEST
-   include testing
+ include testing;
+ flags = "-no-strict-formats";
 *)
 
 (* these are not valid under -strict-formats, but we test them here
