@@ -248,8 +248,6 @@ let lwt_wrap f =
 
 module Html = Dom_html
 
-let json : < parse : Js.js_string Js.t -> 'a > Js.t = Js.Unsafe.pure_js_expr "JSON"
-
 let http_get url =
   XmlHttpRequest.get url
   >>= fun r ->
@@ -257,7 +255,7 @@ let http_get url =
   let msg = r.XmlHttpRequest.content in
   if cod = 0 || cod = 200 then Lwt.return msg else fst (Lwt.wait ())
 
-let getfile f = try Lwt.return (Sys_js.read_file ~name:f) with Not_found -> http_get f
+let getfile f = try Lwt.return (Sys_js.read_file ~name:f) with Sys_error _ -> http_get f
 
 let load_image src =
   let img = Html.createImg Html.document in
@@ -296,10 +294,10 @@ let _debug_msg _s = ()
                     *)
 
 let handle_drag element move stop click =
-  let fuzz = 4 in
+  let fuzz = 4. in
   element##.onmousedown :=
     Html.handler (fun ev ->
-        let x0 = ev##.clientX and y0 = ev##.clientY in
+        let x0 = Js.to_float ev##.clientX and y0 = Js.to_float ev##.clientY in
         (*
 debug_msg (Format.sprintf "Mouse down %d %d" x0 y0);
 *)
@@ -309,11 +307,13 @@ debug_msg (Format.sprintf "Mouse down %d %d" x0 y0);
             Html.document
             Html.Event.mousemove
             (Html.handler (fun ev ->
-                 let x = ev##.clientX and y = ev##.clientY in
+                 let x = Js.to_float ev##.clientX and y = Js.to_float ev##.clientY in
                  (*
 debug_msg (Format.sprintf "Mouse move %d %d %d %d" x0 y0 x y);
 *)
-                 if (not !started) && (abs (x - x0) > fuzz || abs (y - y0) > fuzz)
+                 if
+                   (not !started)
+                   && (abs_float (x -. x0) > fuzz || abs_float (y -. y0) > fuzz)
                  then (
                    started := true;
                    element##.style##.cursor := Js.string "move");
@@ -337,14 +337,14 @@ debug_msg (Format.sprintf "Mouse up %d %d %d %d" x0 y0 ev##clientX ev##clientY);
                     if !started
                     then (
                       element##.style##.cursor := Js.string "";
-                      stop ev##.clientX ev##.clientY)
-                    else click ev##.clientX ev##.clientY;
+                      stop (Js.to_float ev##.clientX) (Js.to_float ev##.clientY))
+                    else click (Js.to_float ev##.clientX) (Js.to_float ev##.clientY);
                     Js._true))
                Js._true);
         Js._true)
 
 let handle_touch_events element move stop cancel click =
-  let fuzz = 4 in
+  let fuzz = 4. in
   ignore
     (Html.addEventListener
        element
@@ -354,7 +354,8 @@ let handle_touch_events element move stop cancel click =
               (ev##.changedTouches##item 0)
               (fun touch ->
                 let id = touch##.identifier in
-                let x0 = touch##.clientX and y0 = touch##.clientY in
+                let x0 = Js.to_float touch##.clientX
+                and y0 = Js.to_float touch##.clientY in
                 (*
 debug_msg (Format.sprintf "Touch start %d %d" x0 y0);
 *)
@@ -370,12 +371,15 @@ debug_msg (Format.sprintf "Touch start %d %d" x0 y0);
                              (fun touch ->
                                if touch##.identifier = id
                                then (
-                                 let x = touch##.clientX and y = touch##.clientY in
+                                 let x = Js.to_float touch##.clientX
+                                 and y = Js.to_float touch##.clientY in
                                  (*
   debug_msg (Format.sprintf "Touch move %d %d %d %d" x0 y0 x y);
 *)
-                                 if (not !started)
-                                    && (abs (x - x0) > fuzz || abs (y - y0) > fuzz)
+                                 if
+                                   (not !started)
+                                   && (abs_float (x -. x0) > fuzz
+                                      || abs_float (y -. y0) > fuzz)
                                  then (
                                    started := true;
                                    element##.style##.cursor := Js.string "move");
@@ -399,7 +403,8 @@ debug_msg (Format.sprintf "Touch start %d %d" x0 y0);
                                 (fun touch ->
                                   if touch##.identifier = id
                                   then (
-                                    let x = touch##.clientX and y = touch##.clientY in
+                                    let x = Js.to_float touch##.clientX
+                                    and y = Js.to_float touch##.clientY in
                                     (*
 debug_msg (Format.sprintf "Touch end %d %d %d %d" x0 y0 x y);
 *)
@@ -524,12 +529,15 @@ let text_size font txt =
 
 (******)
 
+let of_json ~typ v =
+  match Sys.backend_type with
+  | Other "js_of_ocaml" -> Js._JSON##parse (Js.string v)
+  | _ -> Deriving_Json.from_string typ v
+
+(******)
+
 let default_language () =
-  (Js.Optdef.get
-     Dom_html.window##.navigator##.language
-     (fun () ->
-       Js.Optdef.get Dom_html.window##.navigator##.userLanguage (fun () -> Js.string "en")))
-  ##substring
+  (Js.Opt.get Dom_html.window##.navigator##.language (fun () -> Js.string "en"))##substring
     0
     2
 
@@ -538,7 +546,7 @@ let language =
     (Js.Optdef.case Html.window##.localStorage default_language (fun st ->
          Js.Opt.get (st##getItem (Js.string "hyp_lang")) default_language))
 
-let _ = Firebug.console##log !language
+let _ = Console.console##log !language
 
 let set_language lang =
   Js.Optdef.iter Html.window##.localStorage (fun st ->
@@ -546,7 +554,7 @@ let set_language lang =
   language := lang
 
 let load_messages () =
-  getfile "messages.json" >>= fun s -> Lwt.return (json##parse (Js.string s))
+  getfile "messages.json" >>= fun s -> Lwt.return (Js._JSON##parse (Js.string s))
 
 let local_messages msgs : messages Js.t = option (Js.Unsafe.get msgs !language)
 
@@ -577,7 +585,7 @@ let to_screen z = ((z.x +. 1.) *. r, (z.y +. 1.) *. r)
 *)
 let from_screen canvas x y =
   let rx, ry, dx, dy = screen_transform canvas in
-  let z = { x = (float x -. dx) /. rx; y = (float y -. dy) /. ry } in
+  let z = { x = (x -. dx) /. rx; y = (y -. dy) /. ry } in
   let n = norm z in
   if n <= 1. -. eps then z else sdiv z (n /. (1. -. eps))
 
@@ -635,7 +643,7 @@ type boxes =
 let shadow = false
 
 let draw canvas vertices edges nodes boxes =
-  Firebug.console##time (Js.string "draw");
+  Console.console##time (Js.string "draw");
   let c = canvas##getContext Html._2d_ in
   let ((rx, ry, dx, dy) as transf) = screen_transform canvas in
   c##clearRect
@@ -787,14 +795,14 @@ let draw canvas vertices edges nodes boxes =
           (Js.float (2. *. h))
     | `Txt (_, None, _) | `None -> ()
   done;
-  Firebug.console##timeEnd (Js.string "draw");
-  Firebug.console##log_2 !image_count !large_image_count
+  Console.console##timeEnd (Js.string "draw");
+  Console.console##log_2 !image_count !large_image_count
 
 let tree_url = "tree.json"
 
 let ( >> ) x f = f x
 
-type 'a tree = Node of 'a * 'a tree array
+type 'a tree = Node of 'a * 'a tree array [@@deriving json]
 
 let rec tree_vertice_count n =
   let (Node (_, l)) = n in
@@ -1067,17 +1075,22 @@ let tree_layout node_names root =
   compute_text_nodes node_names nodes;
   vertices, edges, nodes, boxes
 
+type js_string = Js.js_string Js.t
+
+let js_string_to_json _ _ : unit = assert false
+
+let js_string_of_json buf = Js.bytestring (Deriving_Json.Json_string.read buf)
+
+[@@@warning "-20-39"]
+
+type tree_info =
+  js_string tree * (js_string * (js_string * js_string) array * js_string) array
+[@@deriving json]
+
 let load_tree () =
   getfile tree_url
   >>= fun s ->
-  let info :
-      Js.js_string Js.t tree
-      * (Js.js_string Js.t
-        * (Js.js_string Js.t * Js.js_string Js.t) array
-        * Js.js_string Js.t)
-        array =
-    json##parse (Js.string s)
-  in
+  let info : tree_info = of_json ~typ:[%json: tree_info] s in
   let tree, node_names = info in
   randomize_tree tree;
   let node_names =
@@ -1091,17 +1104,18 @@ let load_tree () =
   Lwt.return (tree_layout node_names tree, node_names)
 
 type info =
-  { name : Js.js_string Js.t
-  ; url : Js.js_string Js.t
-  ; attribution : Js.js_string Js.t
+  { name : js_string
+  ; url : js_string
+  ; attribution : js_string
   ; width : int
   ; height : int
-  ; links : (Js.js_string Js.t * Js.js_string Js.t * Js.js_string Js.t) array
-  ; img_url : Js.js_string Js.t option
+  ; links : (js_string * js_string * js_string) array
+  ; img_url : js_string option
   }
+[@@deriving json]
 
 let load_image_info () : info array Lwt.t =
-  getfile "image_info.json" >>= fun s -> Lwt.return (json##parse (Js.string s))
+  getfile "image_info.json" >>= fun s -> Lwt.return (of_json ~typ:[%json: info array] s)
 
 let close_button over =
   let color = opt_style style##.buttonColor (Js.string "#888888") in
@@ -1225,7 +1239,7 @@ let show_image all_messages image_info name small_image =
   if !i >= 0
   then (
     let info = image_info.(!i) in
-    Firebug.console##log_2 name !i;
+    Console.console##log_2 name !i;
     let d = Html.document in
     let container = Html.createDiv d in
     container##.style##.margin := Js.string "10px";
@@ -1568,7 +1582,7 @@ let start _ =
     (redraw_funct :=
        fun () ->
          need_redraw := false;
-         Firebug.console##time (Js.string "transform");
+         Console.console##time (Js.string "transform");
          (*
           let transf = hyp_transf !tr' in
           for i = 0 to Array.length vertices - 1 do
@@ -1582,7 +1596,7 @@ let start _ =
            canvas##.width := w;
            canvas##.height := h);
          hyp_transf_vect !tr' vertices vertices';
-         Firebug.console##timeEnd (Js.string "transform");
+         Console.console##timeEnd (Js.string "transform");
          draw canvas vertices' edges nodes boxes);
     perform_redraw ();
     Html.window##.onresize :=
@@ -1619,11 +1633,10 @@ debug_msg (Format.sprintf "Resize %d %d" w h);
     let find_box boxes x y =
       let p = ref (-1) in
       for i = 0 to Array.length boxes.bw - 1 do
-        if Array.unsafe_get boxes.bw i > 0.
-           && abs_float (float x -. Array.unsafe_get boxes.bx i)
-              < Array.unsafe_get boxes.bw i
-           && abs_float (float y -. Array.unsafe_get boxes.by i)
-              < Array.unsafe_get boxes.bh i
+        if
+          Array.unsafe_get boxes.bw i > 0.
+          && abs_float (x -. Array.unsafe_get boxes.bx i) < Array.unsafe_get boxes.bw i
+          && abs_float (y -. Array.unsafe_get boxes.by i) < Array.unsafe_get boxes.bh i
         then p := i
       done;
       !p
@@ -1644,7 +1657,7 @@ debug_msg (Format.sprintf "Resize %d %d" w h);
     in
     canvas##.onmousemove :=
       Html.handler (fun ev ->
-          update_cursor ev##.clientX ev##.clientY;
+          update_cursor (Js.to_float ev##.clientX) (Js.to_float ev##.clientY);
           Js._false);
     handle_drag
       canvas
@@ -1679,7 +1692,7 @@ debug_msg (Format.sprintf "Resize %d %d" w h);
     handle_touch_events
       canvas
       (fun x0 y0 x1 y1 ->
-        Firebug.console##time (Js.string "transform");
+        Console.console##time (Js.string "transform");
         let z0 = from_screen canvas x0 y0 in
         let z1 = from_screen canvas x1 y1 in
         (*
@@ -1845,6 +1858,7 @@ debug_msg (Format.sprintf "Resize %d %d" w h);
       prev_buttons := Some buttons
     in
     make_buttons ();
+    (*
     let img = Html.createImg doc in
     img##.src := icon "ocsigen-powered.png";
     let a = Html.createA doc in
@@ -1857,15 +1871,11 @@ debug_msg (Format.sprintf "Resize %d %d" w h);
     logo##.style##.bottom := Js.string "0";
     Dom.appendChild logo a;
     Dom.appendChild doc##.body logo;
-    Lwt.return ());
-  Js._false
+*)
+    Lwt.return ())
 
-let start _ =
+let () =
   try
     ignore (Html.createCanvas Html.window##.document);
     start ()
-  with Html.Canvas_not_available ->
-    unsupported_messages ();
-    Js._false
-
-let _ = Html.window##.onload := Html.handler start
+  with Html.Canvas_not_available -> unsupported_messages ()
