@@ -294,10 +294,11 @@ let _divide all o =
   let k = ref 0 in
   for i = 0 to Array.length o.faces - 1 do
     let { v1; v2; v3 } = o.faces.(i) in
-    if all
-       || abs_float o.vertices.(v1).y = 1.
-       || abs_float o.vertices.(v2).y = 1.
-       || abs_float o.vertices.(v3).y = 1.
+    if
+      all
+      || abs_float o.vertices.(v1).y = 1.
+      || abs_float o.vertices.(v2).y = 1.
+      || abs_float o.vertices.(v3).y = 1.
     then (
       let w1 = midpoint v1 v2 in
       let w2 = midpoint v2 v3 in
@@ -592,9 +593,9 @@ let _texture = Js.string "black.jpg"
 
 let _texture = Js.string "../planet/land_ocean_ice_cloud_2048.jpg"
 
-let texture = Js.string "../planet/texture.jpg"
+let texture = Js.string "texture.jpg"
 
-let start _ =
+let () =
   Lwt.ignore_result
     (load_image texture
     >>= fun texture ->
@@ -683,23 +684,23 @@ let start _ =
     p##.innerHTML :=
       Js.string "Credit: <a href='http://visibleearth.nasa.gov/'>Visual Earth</a>, Nasa";
     add doc##.body p;
-    let mx = ref 0 in
-    let my = ref 0 in
+    let mx = ref 0. in
+    let my = ref 0. in
     canvas##.onmousedown :=
       Dom_html.handler (fun ev ->
-          mx := ev##.clientX;
-          my := ev##.clientY;
+          mx := Js.to_float ev##.clientX;
+          my := Js.to_float ev##.clientY;
           let c1 =
             Html.addEventListener
               Html.document
               Html.Event.mousemove
               (Dom_html.handler (fun ev ->
-                   let x = ev##.clientX and y = ev##.clientY in
-                   let dx = x - !mx and dy = y - !my in
-                   if dy != 0
-                   then m := matrix_mul (yz_rotation (2. *. float dy /. float width)) !m;
-                   if dx != 0
-                   then m := matrix_mul (xz_rotation (2. *. float dx /. float width)) !m;
+                   let x = Js.to_float ev##.clientX and y = Js.to_float ev##.clientY in
+                   let dx = x -. !mx and dy = y -. !my in
+                   if dy != 0.
+                   then m := matrix_mul (yz_rotation (2. *. dy /. float width)) !m;
+                   if dx != 0.
+                   then m := matrix_mul (xz_rotation (2. *. dx /. float width)) !m;
                    mx := x;
                    my := y;
                    Js._true))
@@ -776,7 +777,4 @@ if true then Lwt.return () else
       if (not !paused) && !follow then phi_rot := !phi_rot +. angle;
       loop t' (if !paused then phi else phi +. angle)
     in
-    loop (Js.to_float (new%js Js.date_now)##getTime) 0.);
-  Js._false
-
-let _ = Html.window##.onload := Html.handler start
+    loop (Js.to_float (new%js Js.date_now)##getTime) 0.)
