@@ -695,172 +695,20 @@ module Color = struct
       new%js Js.regExp
         (Js.bytestring "^hsla\\(\\s*\\d*,\\s*\\d*%,\\s*\\d*%,\\d*\\.?\\d*\\)$")
     in
-    if Js.to_bool (rgb_re##test s)
-       || Js.to_bool (rgba_re##test s)
-       || Js.to_bool (rgb_pct_re##test s)
-       || Js.to_bool (rgba_pct_re##test s)
-       || Js.to_bool (hsl_re##test s)
-       || Js.to_bool (hsla_re##test s)
+    if
+      Js.to_bool (rgb_re##test s)
+      || Js.to_bool (rgba_re##test s)
+      || Js.to_bool (rgb_pct_re##test s)
+      || Js.to_bool (rgba_pct_re##test s)
+      || Js.to_bool (hsl_re##test s)
+      || Js.to_bool (hsla_re##test s)
     then s
-    else if List.mem
-              (Js.to_string s)
-              [ "aliceblue"
-              ; "antiquewhite"
-              ; "aqua"
-              ; "aquamarine"
-              ; "azure"
-              ; "beige"
-              ; "bisque"
-              ; "black"
-              ; "blanchedalmond"
-              ; "blue"
-              ; "blueviolet"
-              ; "brown"
-              ; "burlywood"
-              ; "cadetblue"
-              ; "chartreuse"
-              ; "chocolate"
-              ; "coral"
-              ; "cornflowerblue"
-              ; "cornsilk"
-              ; "crimson"
-              ; "cyan"
-              ; "darkblue"
-              ; "darkcyan"
-              ; "darkgoldenrod"
-              ; "darkgray"
-              ; "darkgreen"
-              ; "darkgrey"
-              ; "darkkhaki"
-              ; "darkmagenta"
-              ; "darkolivegreen"
-              ; "darkorange"
-              ; "darkorchid"
-              ; "darkred"
-              ; "darksalmon"
-              ; "darkseagreen"
-              ; "darkslateblue"
-              ; "darkslategray"
-              ; "darkslategrey"
-              ; "darkturquoise"
-              ; "darkviolet"
-              ; "deeppink"
-              ; "deepskyblue"
-              ; "dimgray"
-              ; "dimgrey"
-              ; "dodgerblue"
-              ; "firebrick"
-              ; "floralwhite"
-              ; "forestgreen"
-              ; "fuchsia"
-              ; "gainsboro"
-              ; "ghostwhite"
-              ; "gold"
-              ; "goldenrod"
-              ; "gray"
-              ; "green"
-              ; "greenyellow"
-              ; "grey"
-              ; "honeydew"
-              ; "hotpink"
-              ; "indianred"
-              ; "indigo"
-              ; "ivory"
-              ; "khaki"
-              ; "lavender"
-              ; "lavenderblush"
-              ; "lawngreen"
-              ; "lemonchiffon"
-              ; "lightblue"
-              ; "lightcoral"
-              ; "lightcyan"
-              ; "lightgoldenrodyellow"
-              ; "lightgray"
-              ; "lightgreen"
-              ; "lightgrey"
-              ; "lightpink"
-              ; "lightsalmon"
-              ; "lightseagreen"
-              ; "lightskyblue"
-              ; "lightslategray"
-              ; "lightslategrey"
-              ; "lightsteelblue"
-              ; "lightyellow"
-              ; "lime"
-              ; "limegreen"
-              ; "linen"
-              ; "magenta"
-              ; "maroon"
-              ; "mediumaquamarine"
-              ; "mediumblue"
-              ; "mediumorchid"
-              ; "mediumpurple"
-              ; "mediumseagreen"
-              ; "mediumslateblue"
-              ; "mediumspringgreen"
-              ; "mediumturquoise"
-              ; "mediumvioletred"
-              ; "midnightblue"
-              ; "mintcream"
-              ; "mistyrose"
-              ; "moccasin"
-              ; "navajowhite"
-              ; "navy"
-              ; "oldlace"
-              ; "olive"
-              ; "olivedrab"
-              ; "orange"
-              ; "orangered"
-              ; "orchid"
-              ; "palegoldenrod"
-              ; "palegreen"
-              ; "paleturquoise"
-              ; "palevioletred"
-              ; "papayawhip"
-              ; "peachpuff"
-              ; "peru"
-              ; "pink"
-              ; "plum"
-              ; "powderblue"
-              ; "purple"
-              ; "red"
-              ; "rosybrown"
-              ; "royalblue"
-              ; "saddlebrown"
-              ; "salmon"
-              ; "sandybrown"
-              ; "seagreen"
-              ; "seashell"
-              ; "sienna"
-              ; "silver"
-              ; "skyblue"
-              ; "slateblue"
-              ; "slategray"
-              ; "slategrey"
-              ; "snow"
-              ; "springgreen"
-              ; "steelblue"
-              ; "tan"
-              ; "teal"
-              ; "thistle"
-              ; "tomato"
-              ; "turquoise"
-              ; "violet"
-              ; "wheat"
-              ; "white"
-              ; "whitesmoke"
-              ; "yellow"
-              ; "yellowgreen"
-              ]
-    then s
-    else raise (Invalid_argument (Js.to_string s ^ " is not a valid color"))
+    else
+      match name_of_string (Js.to_string s) with
+      | _ -> s
+      | exception _ -> raise (Invalid_argument (Js.to_string s ^ " is not a valid color"))
 
-  let name cn = Js.string (string_of_name cn)
-
-  let js = function
-    | Name n -> name n
-    | (RGB _ | RGB_percent _ | RGBA _ | RGBA_percent _ | HSL _ | HSLA _) as c ->
-        Js.string (string_of_t c)
+  let js c = Js.string (string_of_t c)
 
   let ml c =
     let s = Js.to_string c in
