@@ -21,10 +21,6 @@
 open Js
 open! Import
 
-external caml_js_on_ie : unit -> bool t = "caml_js_on_ie"
-
-let onIE = Js.to_bool (caml_js_on_ie ())
-
 external html_escape : js_string t -> js_string t = "caml_js_html_escape"
 
 external html_entities : js_string t -> js_string t opt = "caml_js_html_entities"
@@ -194,7 +190,7 @@ class type cssStyleDeclaration = object
 
   method minWidth : js_string t prop
 
-  method opacity : js_string t optdef prop
+  method opacity : js_string t prop
 
   method outline : js_string t prop
 
@@ -289,25 +285,25 @@ end
 and focusEvent = object
   inherit event
 
-  method relatedTarget : element t opt optdef readonly_prop
+  method relatedTarget : element t opt readonly_prop
 end
 
 and mouseEvent = object
   inherit event
 
-  method relatedTarget : element t opt optdef readonly_prop
+  method relatedTarget : element t opt readonly_prop
 
-  method clientX : int readonly_prop
+  method clientX : number_t readonly_prop
 
-  method clientY : int readonly_prop
+  method clientY : number_t readonly_prop
 
-  method screenX : int readonly_prop
+  method screenX : number_t readonly_prop
 
-  method screenY : int readonly_prop
+  method screenY : number_t readonly_prop
 
-  method offsetX : int readonly_prop
+  method offsetX : number_t readonly_prop
 
-  method offsetY : int readonly_prop
+  method offsetY : number_t readonly_prop
 
   method ctrlKey : bool t readonly_prop
 
@@ -319,15 +315,17 @@ and mouseEvent = object
 
   method button : int readonly_prop
 
+  method buttons : int readonly_prop
+
   method which : mouse_button optdef readonly_prop
 
   method fromElement : element t opt optdef readonly_prop
 
   method toElement : element t opt optdef readonly_prop
 
-  method pageX : int optdef readonly_prop
+  method pageX : number_t optdef readonly_prop
 
-  method pageY : int optdef readonly_prop
+  method pageY : number_t optdef readonly_prop
 end
 
 and keyboardEvent = object
@@ -347,27 +345,25 @@ and keyboardEvent = object
 
   method code : js_string t optdef readonly_prop
 
+  method isComposing : bool t readonly_prop
+
+  method repeat : bool t readonly_prop
+
+  method getModifierState : js_string t -> bool t meth
+
   method which : int optdef readonly_prop
 
   method charCode : int optdef readonly_prop
 
   method keyCode : int readonly_prop
 
-  method getModifierState : js_string t -> bool t meth
-
   method keyIdentifier : js_string t optdef readonly_prop
 end
 
-and mousewheelEvent = object
+and wheelEvent = object
   (* All modern browsers *)
   inherit mouseEvent
 
-  method wheelDelta : int readonly_prop
-
-  method wheelDeltaX : int optdef readonly_prop
-
-  method wheelDeltaY : int optdef readonly_prop
-
   method deltaX : number_t readonly_prop
 
   method deltaY : number_t readonly_prop
@@ -375,10 +371,18 @@ and mousewheelEvent = object
   method deltaZ : number_t readonly_prop
 
   method deltaMode : delta_mode readonly_prop
+
+  method wheelDelta : int readonly_prop
+
+  method wheelDeltaX : int optdef readonly_prop
+
+  method wheelDeltaY : int optdef readonly_prop
 end
 
+and mousewheelEvent = wheelEvent
+
 and mouseScrollEvent = object
-  (* Firefox *)
+  (* Deprecated *)
   inherit mouseEvent
 
   method detail : int readonly_prop
@@ -407,7 +411,7 @@ and touchEvent = object
 
   method metaKey : bool t readonly_prop
 
-  method relatedTarget : element t opt optdef readonly_prop
+  method relatedTarget : element t opt readonly_prop
 end
 
 and touchList = object
@@ -421,23 +425,23 @@ and touch = object
 
   method target : element t optdef readonly_prop
 
-  method screenX : int readonly_prop
+  method screenX : number_t readonly_prop
 
-  method screenY : int readonly_prop
+  method screenY : number_t readonly_prop
 
-  method clientX : int readonly_prop
+  method clientX : number_t readonly_prop
 
-  method clientY : int readonly_prop
+  method clientY : number_t readonly_prop
 
-  method pageX : int readonly_prop
+  method pageX : number_t readonly_prop
 
-  method pageY : int readonly_prop
+  method pageY : number_t readonly_prop
 end
 
 and submitEvent = object
   inherit event
 
-  method submitter : element t optdef readonly_prop
+  method submitter : element t readonly_prop
 end
 
 and dragEvent = object
@@ -460,6 +464,14 @@ and toggleEvent = object
   method oldState : js_string t readonly_prop
 end
 
+and mediaQueryListEvent = object
+  inherit event
+
+  method matches : js_string t readonly_prop
+
+  method media : bool t readonly_prop
+end
+
 and dataTransfer = object
   method dropEffect : js_string t prop
 
@@ -505,7 +517,7 @@ and eventTarget = object ('self)
 
   method onscroll : ('self t, event t) event_listener writeonly_prop
 
-  method onwheel : ('self t, mousewheelEvent t) event_listener writeonly_prop
+  method onwheel : ('self t, wheelEvent t) event_listener writeonly_prop
 
   method ondragstart : ('self t, dragEvent t) event_listener writeonly_prop
 
@@ -727,9 +739,9 @@ and element = object
 
   method offsetHeight : int readonly_prop
 
-  method scrollLeft : int prop
+  method scrollLeft : number_t prop
 
-  method scrollTop : int prop
+  method scrollTop : number_t prop
 
   method scrollWidth : int prop
 
@@ -759,9 +771,9 @@ and clientRect = object
 
   method left : number_t readonly_prop
 
-  method width : number_t optdef readonly_prop
+  method width : number_t readonly_prop
 
-  method height : number_t optdef readonly_prop
+  method height : number_t readonly_prop
 end
 
 and clientRectList = object
@@ -1187,7 +1199,7 @@ class type inputElement = object ('self)
 
   method select : unit meth
 
-  method files : File.fileList t optdef readonly_prop
+  method files : File.fileList t readonly_prop
 
   method placeholder : js_string t writeonly_prop
 
@@ -1407,9 +1419,9 @@ class type imageElement = object ('self)
 
   method height : int prop
 
-  method naturalWidth : int optdef readonly_prop
+  method naturalWidth : int readonly_prop
 
-  method naturalHeight : int optdef readonly_prop
+  method naturalHeight : int readonly_prop
 
   method complete : bool t prop
 
@@ -1994,7 +2006,35 @@ and canvasGradient = object
 end
 
 and textMetrics = object
+  method actualBoundingBoxAscent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxAscent *)
+
+  method actualBoundingBoxDescent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxDescent *)
+
+  method actualBoundingBoxLeft : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxLeft *)
+
+  method actualBoundingBoxRight : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxRight *)
+
+  method alphabeticBaseline : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/alphabeticBaseline *)
+
+  method fontBoundingBoxAscent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/fontBoundingBoxAscent *)
+
+  method fontBoundingBoxDescent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/fontBoundingBoxDescent *)
+
+  method hangingBaseline : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/hangingBaseline *)
+
+  method ideographicBaseline : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/ideographicBaseline *)
+
   method width : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/width *)
 end
 
 and imageData = object
@@ -2175,7 +2215,7 @@ class type location = object
 
   method hostname : js_string t prop
 
-  method origin : js_string t optdef readonly_prop
+  method origin : js_string t readonly_prop
 
   method port : js_string t prop
 
@@ -2192,19 +2232,7 @@ class type location = object
   method reload : unit meth
 end
 
-let location_origin (loc : location t) =
-  Optdef.case
-    loc##.origin
-    (fun () ->
-      let protocol = loc##.protocol in
-      let hostname = loc##.hostname in
-      let port = loc##.port in
-      if protocol##.length = 0 && hostname##.length = 0
-      then Js.string ""
-      else
-        let origin = protocol##concat_2 (Js.string "//") hostname in
-        if port##.length > 0 then origin##concat_2 (Js.string ":") loc##.port else origin)
-    (fun o -> o)
+let location_origin (loc : location t) = loc##.origin
 
 class type history = object
   method length : int readonly_prop
@@ -2241,11 +2269,11 @@ class type navigator = object
 
   method userAgent : js_string t readonly_prop
 
-  method language : js_string t optdef readonly_prop
-
-  method userLanguage : js_string t optdef readonly_prop
+  method language : js_string t opt readonly_prop
 
   method maxTouchPoints : int readonly_prop
+
+  method userLanguage : js_string t optdef readonly_prop
 end
 
 class type screen = object
@@ -2292,6 +2320,16 @@ class type _URL = object
   method revokeObjectURL : js_string t -> unit meth
 end
 
+class type mediaQueryList = object
+  method media : js_string t prop
+
+  method matches : bool readonly_prop
+
+  method onchange : (mediaQueryList t, mediaQueryListEvent t) event_listener prop
+
+  inherit eventTarget
+end
+
 class type window = object
   inherit eventTarget
 
@@ -2321,9 +2359,15 @@ class type window = object
 
   method blur : unit meth
 
-  method scroll : int -> int -> unit meth
+  method scrollX : number_t readonly_prop
 
-  method scrollBy : int -> int -> unit meth
+  method scrollY : number_t readonly_prop
+
+  method scroll : number_t -> number_t -> unit meth
+
+  method scrollTo : number_t -> number_t -> unit meth
+
+  method scrollBy : number_t -> number_t -> unit meth
 
   method sessionStorage : storage t optdef readonly_prop
 
@@ -2402,6 +2446,8 @@ class type window = object
   method _URL : _URL t readonly_prop
 
   method devicePixelRatio : number_t readonly_prop
+
+  method matchMedia : js_string t -> mediaQueryList t meth
 end
 
 let window : window t = Js.Unsafe.global
@@ -2524,14 +2570,15 @@ let rec unsafeCreateElementEx ?_type ?name doc elt =
         Js.Unsafe.coerce (doc##createElement (a##join (Js.string "")))
     | `Unknown ->
         createElementSyntax :=
-          if try
-               let el : inputElement Js.t =
-                 Js.Unsafe.coerce
-                   (document##createElement (Js.string "<input name=\"x\">"))
-               in
-               Js.equals el##.tagName##toLowerCase (Js.string "input")
-               && Js.equals el##.name (Js.string "x")
-             with _ -> false
+          if
+            try
+              let el : inputElement Js.t =
+                Js.Unsafe.coerce
+                  (document##createElement (Js.string "<input name=\"x\">"))
+              in
+              Js.equals el##.tagName##toLowerCase (Js.string "input")
+              && Js.equals el##.name (Js.string "x")
+            with _ -> false
           then `Extended
           else `Standard;
         unsafeCreateElementEx ?_type ?name doc elt
@@ -2714,18 +2761,8 @@ let createCanvas doc : canvasElement t =
 let html_element : htmlElement t constr = Js.Unsafe.global##._HTMLElement
 
 module CoerceTo = struct
-  let element : #Dom.node Js.t -> element Js.t Js.opt =
-    if not (Js.Optdef.test (def html_element))
-    then
-      (* ie < 9 does not have HTMLElement: we have to cheat to check
-         that something is an html element *)
-      fun e ->
-      if not (Js.Optdef.test (def (Js.Unsafe.coerce e)##.innerHTML))
-      then Js.null
-      else Js.some (Js.Unsafe.coerce e)
-    else
-      fun e ->
-      if Js.instanceof e html_element then Js.some (Js.Unsafe.coerce e) else Js.null
+  let element (e : #Dom.node Js.t) : element Js.t Js.opt =
+    if Js.instanceof e html_element then Js.some (Js.Unsafe.coerce e) else Js.null
 
   let unsafeCoerce tag (e : #element t) =
     if Js.equals e##.tagName##toLowerCase (Js.string tag)
@@ -2878,24 +2915,27 @@ end
 
 let eventTarget = Dom.eventTarget
 
-let eventRelatedTarget (e : #mouseEvent t) =
-  Optdef.get e##.relatedTarget (fun () ->
-      match Js.to_string e##._type with
-      | "mouseover" -> Optdef.get e##.fromElement (fun () -> assert false)
-      | "mouseout" -> Optdef.get e##.toElement (fun () -> assert false)
-      | _ -> Js.null)
+let eventRelatedTarget (e : #mouseEvent t) = e##.relatedTarget
 
 let eventAbsolutePosition' (e : #mouseEvent t) =
   let body = document##.body in
   let html = document##.documentElement in
-  ( e##.clientX + body##.scrollLeft + html##.scrollLeft
-  , e##.clientY + body##.scrollTop + html##.scrollTop )
+  ( Js.to_float e##.clientX
+    +. Js.to_float body##.scrollLeft
+    +. Js.to_float html##.scrollLeft
+  , Js.to_float e##.clientY
+    +. Js.to_float body##.scrollTop
+    +. Js.to_float html##.scrollTop )
 
 let eventAbsolutePosition (e : #mouseEvent t) =
   Optdef.case
     e##.pageX
     (fun () -> eventAbsolutePosition' e)
-    (fun x -> Optdef.case e##.pageY (fun () -> eventAbsolutePosition' e) (fun y -> x, y))
+    (fun x ->
+      Optdef.case
+        e##.pageY
+        (fun () -> eventAbsolutePosition' e)
+        (fun y -> Js.to_float x, Js.to_float y))
 
 let elementClientPosition (e : #element t) =
   let r = e##getBoundingClientRect in
@@ -2907,7 +2947,8 @@ let elementClientPosition (e : #element t) =
 let getDocumentScroll () =
   let body = document##.body in
   let html = document##.documentElement in
-  body##.scrollLeft + html##.scrollLeft, body##.scrollTop + html##.scrollTop
+  ( Js.to_float body##.scrollLeft +. Js.to_float html##.scrollLeft
+  , Js.to_float body##.scrollTop +. Js.to_float html##.scrollTop )
 
 let buttonPressed (ev : #mouseEvent Js.t) =
   Js.Optdef.case
@@ -3362,7 +3403,7 @@ module Keyboard_code = struct
     | 1 -> run_next (get_key_code evt) try_key_code_left
     | 2 -> run_next (get_key_code evt) try_key_code_right
     | 3 -> run_next (get_key_code evt) try_key_code_numpad
-    | _ -> make_unidentified
+    | _ -> fun v -> v
 
   let ( |> ) x f = f x
 
