@@ -189,7 +189,7 @@ class type cssStyleDeclaration = object
 
   method minWidth : js_string t prop
 
-  method opacity : js_string t optdef prop
+  method opacity : js_string t prop
 
   method outline : js_string t prop
 
@@ -290,27 +290,27 @@ end
 and focusEvent = object
   inherit event
 
-  method relatedTarget : element t opt optdef readonly_prop
+  method relatedTarget : element t opt readonly_prop
 end
 
 and mouseEvent = object
   inherit event
 
-  method relatedTarget : element t opt optdef readonly_prop
+  method relatedTarget : element t opt readonly_prop
 
   (* Relative to viewport *)
-  method clientX : int readonly_prop
+  method clientX : number_t readonly_prop
 
-  method clientY : int readonly_prop
+  method clientY : number_t readonly_prop
 
   (* Relative to the edge of the screen *)
-  method screenX : int readonly_prop
+  method screenX : number_t readonly_prop
 
-  method screenY : int readonly_prop
+  method screenY : number_t readonly_prop
 
-  method offsetX : int readonly_prop
+  method offsetX : number_t readonly_prop
 
-  method offsetY : int readonly_prop
+  method offsetY : number_t readonly_prop
 
   method ctrlKey : bool t readonly_prop
 
@@ -320,18 +320,20 @@ and mouseEvent = object
 
   method metaKey : bool t readonly_prop
 
-  method which : mouse_button optdef readonly_prop
+  method button : int readonly_prop
+
+  method buttons : int readonly_prop
 
   (* Legacy methods *)
-  method button : int readonly_prop
+  method which : mouse_button optdef readonly_prop
 
   method fromElement : element t opt optdef readonly_prop
 
   method toElement : element t opt optdef readonly_prop
 
-  method pageX : int optdef readonly_prop
+  method pageX : number_t optdef readonly_prop
 
-  method pageY : int optdef readonly_prop
+  method pageY : number_t optdef readonly_prop
 end
 
 and keyboardEvent = object
@@ -347,11 +349,19 @@ and keyboardEvent = object
 
   method location : int readonly_prop
 
-  (* Standardized but not fully supported properties *)
+  (* Chrome can send fake keyboard events without any of the expected
+     properties (https://chromium-review.googlesource.com/771674), so
+     we keep the [optdef] annotation for now *)
   method key : js_string t optdef readonly_prop
 
   method code : js_string t optdef readonly_prop
 
+  method isComposing : bool t readonly_prop
+
+  method repeat : bool t readonly_prop
+
+  method getModifierState : js_string t -> bool t meth
+
   (* Deprecated properties *)
   method which : int optdef readonly_prop
 
@@ -359,21 +369,12 @@ and keyboardEvent = object
 
   method keyCode : int readonly_prop
 
-  method getModifierState : js_string t -> bool t meth
-
   method keyIdentifier : js_string t optdef readonly_prop
 end
 
-and mousewheelEvent = object
-  (* All modern browsers *)
+and wheelEvent = object
   inherit mouseEvent
 
-  method wheelDelta : int readonly_prop
-
-  method wheelDeltaX : int optdef readonly_prop
-
-  method wheelDeltaY : int optdef readonly_prop
-
   method deltaX : number_t readonly_prop
 
   method deltaY : number_t readonly_prop
@@ -381,10 +382,19 @@ and mousewheelEvent = object
   method deltaZ : number_t readonly_prop
 
   method deltaMode : delta_mode readonly_prop
+
+  (* Deprecated *)
+  method wheelDelta : int readonly_prop
+
+  method wheelDeltaX : int optdef readonly_prop
+
+  method wheelDeltaY : int optdef readonly_prop
 end
 
+and mousewheelEvent = (* Deprecated *) wheelEvent
+
 and mouseScrollEvent = object
-  (* Firefox *)
+  (* Deprecated *)
   inherit mouseEvent
 
   method detail : int readonly_prop
@@ -413,7 +423,7 @@ and touchEvent = object
 
   method metaKey : bool t readonly_prop
 
-  method relatedTarget : element t opt optdef readonly_prop
+  method relatedTarget : element t opt readonly_prop
 end
 
 and touchList = object
@@ -425,25 +435,26 @@ end
 and touch = object
   method identifier : int readonly_prop
 
+  (* Not available on Safari *)
   method target : element t optdef readonly_prop
 
-  method screenX : int readonly_prop
+  method screenX : number_t readonly_prop
 
-  method screenY : int readonly_prop
+  method screenY : number_t readonly_prop
 
-  method clientX : int readonly_prop
+  method clientX : number_t readonly_prop
 
-  method clientY : int readonly_prop
+  method clientY : number_t readonly_prop
 
-  method pageX : int readonly_prop
+  method pageX : number_t readonly_prop
 
-  method pageY : int readonly_prop
+  method pageY : number_t readonly_prop
 end
 
 and submitEvent = object
   inherit event
 
-  method submitter : element t optdef readonly_prop
+  method submitter : element t readonly_prop
 end
 
 and dragEvent = object
@@ -466,6 +477,14 @@ and toggleEvent = object
   method oldState : js_string t readonly_prop
 end
 
+and mediaQueryListEvent = object
+  inherit event
+
+  method matches : js_string t readonly_prop
+
+  method media : bool t readonly_prop
+end
+
 and dataTransfer = object
   method dropEffect : js_string t prop
 
@@ -475,6 +494,7 @@ and dataTransfer = object
 
   method types : js_string t js_array t readonly_prop
 
+  (* Not standard *)
   method addElement : element t -> unit meth
 
   method clearData : js_string t -> unit meth
@@ -513,7 +533,7 @@ and eventTarget = object ('self)
 
   method onscroll : ('self t, event t) event_listener writeonly_prop
 
-  method onwheel : ('self t, mousewheelEvent t) event_listener writeonly_prop
+  method onwheel : ('self t, wheelEvent t) event_listener writeonly_prop
 
   method ondragstart : ('self t, dragEvent t) event_listener writeonly_prop
 
@@ -733,16 +753,15 @@ and element = object
 
   method offsetTop : int readonly_prop
 
-  (* Incorrect in IE until IE7 included *)
   method offsetParent : element t opt readonly_prop
 
   method offsetWidth : int readonly_prop
 
   method offsetHeight : int readonly_prop
 
-  method scrollLeft : int prop
+  method scrollLeft : number_t prop
 
-  method scrollTop : int prop
+  method scrollTop : number_t prop
 
   method scrollWidth : int prop
 
@@ -773,9 +792,9 @@ and clientRect = object
 
   method left : number_t readonly_prop
 
-  method width : number_t optdef readonly_prop
+  method width : number_t readonly_prop
 
-  method height : number_t optdef readonly_prop
+  method height : number_t readonly_prop
 end
 
 and clientRectList = object
@@ -916,7 +935,6 @@ class type selectElement = object ('self)
 
   method _type : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method selectedIndex : int prop
 
   method value : js_string t prop
@@ -933,7 +951,6 @@ class type selectElement = object ('self)
 
   method name : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method size : int prop
 
   method tabIndex : int prop
@@ -944,7 +961,6 @@ class type selectElement = object ('self)
 
   method required : bool t writeonly_prop
 
-  (* Not supported by IE 9/Safari *)
   method onchange : ('self t, event t) event_listener prop
 
   method oninput : ('self t, event t) event_listener prop
@@ -963,6 +979,7 @@ class type inputElement = object ('self)
 
   method accessKey : js_string t prop
 
+  (* deprecated *)
   method align : js_string t prop
 
   method alt : js_string t prop
@@ -975,12 +992,10 @@ class type inputElement = object ('self)
 
   method name : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method readOnly : bool t prop
 
   method required : bool t writeonly_prop
 
-  (* Not supported by IE 9/Safari *)
   method size : int prop
 
   method src : js_string t prop
@@ -989,18 +1004,17 @@ class type inputElement = object ('self)
 
   method _type : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
+  (* Deprecated *)
   method useMap : js_string t prop
 
   method value : js_string t prop
 
   method select : unit meth
 
-  method files : File.fileList t optdef readonly_prop
+  method files : File.fileList t readonly_prop
 
   method placeholder : js_string t writeonly_prop
 
-  (* Not supported by IE 9 *)
   method selectionDirection : js_string t prop
 
   method selectionStart : int prop
@@ -1033,7 +1047,6 @@ class type textAreaElement = object ('self)
 
   method name : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method readOnly : bool t prop
 
   method rows : int prop
@@ -1048,17 +1061,14 @@ class type textAreaElement = object ('self)
 
   method _type : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method value : js_string t prop
 
   method select : unit meth
 
   method required : bool t writeonly_prop
 
-  (* Not supported by IE 9/Safari *)
   method placeholder : js_string t writeonly_prop
 
-  (* Not supported by IE 9 *)
   method onselect : ('self t, event t) event_listener prop
 
   method onchange : ('self t, event t) event_listener prop
@@ -1081,12 +1091,10 @@ class type buttonElement = object
 
   method name : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method tabIndex : int prop
 
   method _type : js_string t readonly_prop
 
-  (* Cannot be changed under IE *)
   method value : js_string t prop
 end
 
@@ -1224,9 +1232,9 @@ class type imageElement = object ('self)
   method height : int prop
 
   (* Properties naturalWidth/Height not available in all browsers. *)
-  method naturalWidth : int optdef readonly_prop
+  method naturalWidth : int readonly_prop
 
-  method naturalHeight : int optdef readonly_prop
+  method naturalHeight : int readonly_prop
 
   method complete : bool t prop
 
@@ -1812,7 +1820,35 @@ and canvasGradient = object
 end
 
 and textMetrics = object
+  method actualBoundingBoxAscent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxAscent *)
+
+  method actualBoundingBoxDescent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxDescent *)
+
+  method actualBoundingBoxLeft : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxLeft *)
+
+  method actualBoundingBoxRight : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/actualBoundingBoxRight *)
+
+  method alphabeticBaseline : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/alphabeticBaseline *)
+
+  method fontBoundingBoxAscent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/fontBoundingBoxAscent *)
+
+  method fontBoundingBoxDescent : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/fontBoundingBoxDescent *)
+
+  method hangingBaseline : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/hangingBaseline *)
+
+  method ideographicBaseline : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/ideographicBaseline *)
+
   method width : number_t readonly_prop
+  (** https://developer.mozilla.org/docs/Web/API/TextMetrics/width *)
 end
 
 and imageData = object
@@ -2021,7 +2057,7 @@ class type location = object
 
   method hostname : js_string t prop
 
-  method origin : js_string t optdef readonly_prop
+  method origin : js_string t readonly_prop
 
   method port : js_string t prop
 
@@ -2078,11 +2114,12 @@ class type navigator = object
 
   method userAgent : js_string t readonly_prop
 
-  method language : js_string t optdef readonly_prop
-
-  method userLanguage : js_string t optdef readonly_prop
+  method language : js_string t opt readonly_prop
 
   method maxTouchPoints : int readonly_prop
+
+  (* Deprecated *)
+  method userLanguage : js_string t optdef readonly_prop
 end
 
 class type screen = object
@@ -2135,6 +2172,16 @@ class type _URL = object
   method revokeObjectURL : js_string t -> unit meth
 end
 
+class type mediaQueryList = object
+  method media : js_string t prop
+
+  method matches : bool readonly_prop
+
+  method onchange : (mediaQueryList t, mediaQueryListEvent t) event_listener prop
+
+  inherit eventTarget
+end
+
 (** Specification of window objects *)
 class type window = object
   inherit eventTarget
@@ -2165,9 +2212,19 @@ class type window = object
 
   method blur : unit meth
 
-  method scroll : int -> int -> unit meth
+  method scrollX : number_t readonly_prop
 
-  method scrollBy : int -> int -> unit meth
+  method scrollY : number_t readonly_prop
+
+  method scroll : number_t -> number_t -> unit meth
+
+  method scrollTo : number_t -> number_t -> unit meth
+
+  method scrollBy : number_t -> number_t -> unit meth
+
+  (* These two properties are not available on non-Web environments
+     (for instance, Web workers, node). So we keep the [optdef]
+     annotation for now. *)
 
   method sessionStorage : storage t optdef readonly_prop
 
@@ -2246,6 +2303,8 @@ class type window = object
   method _URL : _URL t readonly_prop
 
   method devicePixelRatio : number_t readonly_prop
+
+  method matchMedia : js_string t -> mediaQueryList t meth
 end
 
 val window : window t
@@ -2369,7 +2428,7 @@ module Event : sig
 
   val _DOMMouseScroll : mouseScrollEvent t typ
 
-  val wheel : mousewheelEvent t typ
+  val wheel : wheelEvent t typ
 
   val touchstart : touchEvent t typ
 
@@ -2554,7 +2613,7 @@ val removeEventListener : event_listener_id -> unit
 (** Remove the given event listener. *)
 
 val addMousewheelEventListenerWithOptions :
-     (#eventTarget t as 'a)
+     #eventTarget t
   -> ?capture:bool t
   -> ?once:bool t
   -> ?passive:bool t
@@ -2566,7 +2625,7 @@ val addMousewheelEventListenerWithOptions :
       means down / right. *)
 
 val addMousewheelEventListener :
-     (#eventTarget t as 'a)
+     #eventTarget t
   -> (mouseEvent t -> dx:int -> dy:int -> bool t)
   -> bool t
   -> event_listener_id
@@ -2591,13 +2650,13 @@ val buttonPressed : #mouseEvent Js.t -> mouse_button
 
 (** {2 Position helper functions} *)
 
-val eventAbsolutePosition : #mouseEvent t -> int * int
+val eventAbsolutePosition : #mouseEvent t -> float * float
 (** Returns the absolute position of the mouse pointer. *)
 
 val elementClientPosition : #element t -> int * int
 (** Position of an element relative to the viewport *)
 
-val getDocumentScroll : unit -> int * int
+val getDocumentScroll : unit -> float * float
 (** Viewport top/left position *)
 
 (** {2 Key event helper functions} *)
@@ -3210,8 +3269,6 @@ val _requestAnimationFrame : (unit -> unit) Js.callback -> unit
 
 val decode_html_entities : js_string t -> js_string t
 
-val onIE : bool
-
 val hasPushState : unit -> bool
 
 val hasPlaceholder : unit -> bool
