@@ -28,12 +28,12 @@
     let f_success pos =
       let coords = pos##.coords in
       let latitude = coords##.latitude in
-      Firebug.console##debug latitude ;
+      Console.console##debug latitude ;
     in
     let f_error err =
       let code = err##.code in
       let msg = err##.message in
-      if code = err##._TIMEOUT then Firebug.console##debug(msg)
+      if code = err##._TIMEOUT then Console.console##debug(msg)
     in
     geo##getCurrentPosition (Js.wrap_callback f_success) (Js.wrap_callback f_error) options
   ]}
