@@ -219,8 +219,6 @@ val nfkc : normalization t
 
 (** Specification of Javascript number objects. *)
 
-type number_t = float
-
 class type number = object
   method toString : js_string t meth
 
@@ -245,7 +243,9 @@ and js_string = object
 
   method charAt : int -> js_string t meth
 
-  method charCodeAt : int -> number_t meth
+  method charCodeAt : int -> number t meth
+
+  method codePointAt : int -> number t optdef meth
 
   (* This may return NaN... *)
   method concat : js_string t -> js_string t meth
@@ -265,7 +265,7 @@ and js_string = object
 
   method lastIndexOf_from : js_string t -> int -> int meth
 
-  method localeCompare : js_string t -> number_t meth
+  method localeCompare : js_string t -> number t meth
 
   method _match : regExp t -> match_result_handle t opt meth
 
@@ -328,6 +328,8 @@ and regExp = object
   method lastIndex : int prop
 end
 
+type number_t = number t
+
 (** Specification of the string constructor, considered as an object. *)
 class type string_constr = object
   method fromCharCode : int -> js_string t meth
