@@ -61,7 +61,7 @@ let with_error_log f x =
   Lwt.catch
     (fun () -> f x)
     (fun e ->
-      Firebug.console##log (Js.string (Printexc.to_string e));
+      Console.console##log (Js.string (Printexc.to_string e));
       Lwt.return ())
 
 let seq_loop evh ?(cancel_handler = false) ?use_capture ?passive target handler =
@@ -75,8 +75,9 @@ let seq_loop evh ?(cancel_handler = false) ?use_capture ?passive target handler
       if cancel_handler then Lwt.cancel !cur_handler;
       cancelled := true);
   let rec aux () =
-    if not !cancelled
-       (* In the case it has been cancelled
+    if
+      not !cancelled
+      (* In the case it has been cancelled
                          during the previous handler,
                          we do not reinstall the event handler *)
     then (
@@ -329,11 +330,11 @@ let mousewheel ?use_capture ?passive target =
          ?passive:(opt_map Js.bool passive)
          target
          (fun (ev : #Dom_html.event Js.t) ~dx ~dy ->
-           Firebug.console##log ev;
+           Console.console##log ev;
            cancel ();
            Lwt.wakeup w (ev, (dx, dy));
            Js.bool true)
-         (* true because we do not want to prevent default ->
+      (* true because we do not want to prevent default ->
                            the user can use the preventDefault function
                            above. *));
   t
