@@ -50,20 +50,20 @@ let raw_call name uri error_cb user_cb =
   script##.async := Js._true;
   ((Js.Unsafe.coerce script)##.onerror
   := fun x ->
-  finalize ();
-  error_cb x);
+    finalize ();
+    error_cb x);
   ((Js.Unsafe.coerce script)##.onload
   := fun x ->
-  Lwt.async (fun () ->
-      Lwt.bind (Lwt_js.sleep 1.) (fun () ->
-          if !executed
-          then Lwt.return_unit
-          else (
-            Firebug.console##warn
-              (Js.string "Jsonp: script loaded but callback not executed");
-            finalize ();
-            error_cb x;
-            Lwt.return_unit))));
+    Lwt.async (fun () ->
+        Lwt.bind (Lwt_js.sleep 1.) (fun () ->
+            if !executed
+            then Lwt.return_unit
+            else (
+              Console.console##warn
+                (Js.string "Jsonp: script loaded but callback not executed");
+              finalize ();
+              error_cb x;
+              Lwt.return_unit))));
   let init () = ignore (Dom.appendChild Dom_html.document##.body script) in
   init, finalize
 
