@@ -127,7 +127,20 @@ module Sys = struct
   module Config = struct
     external use_js_string : unit -> bool = "caml_jsoo_flags_use_js_string"
 
-    external effects : unit -> bool = "caml_jsoo_flags_effects"
+    type effects_backend =
+      [ `Disabled
+      | `Cps
+      | `Double_translation
+      ]
+
+    external effects_ : unit -> string = "caml_jsoo_flags_effects"
+
+    let effects () =
+      match effects_ () with
+      | "disabled" -> `Disabled
+      | "cps" -> `Cps
+      | "double-translation" -> `Double_translation
+      | _ -> assert false
   end
 
   let version = Runtime_version.s
@@ -164,7 +177,7 @@ end = struct
 
   let _ = Callback.register_exception "jsError" (Exn (Obj.magic [||]))
 
-  let raise_ : t -> 'a = Js.js_expr "(function (exn) { throw exn })"
+  external raise_ : t -> 'a = "caml_throw_js_exception"
 
   external of_exn : exn -> t option = "caml_js_error_option_of_exception"
 
@@ -189,6 +202,8 @@ module For_compatibility_only = struct
   external caml_list_to_js_array : 'a list -> 'a Js.js_array = "caml_list_to_js_array"
 
   external variable : string -> 'a = "caml_js_var"
+
+  external caml_string_of_array : 'a array -> string = "caml_string_of_array"
 end
 
 module Typed_array = struct
@@ -221,10 +236,25 @@ module Typed_array = struct
     external of_uint8Array : uint8Array -> t = "bigstring_of_typed_array"
   end
 
-  external of_uint8Array : uint8Array -> string = "caml_string_of_array"
+  external of_uint8Array : uint8Array -> string = "caml_string_of_uint8_array"
 end
 
 module Int64 = struct
   external create_int64_lo_mi_hi : int -> int -> int -> Int64.t
     = "caml_int64_create_lo_mi_hi"
 end
+
+module Effect : sig
+  external assume_no_perform : (unit -> 'a) -> 'a = "caml_assume_no_perform"
+  (** Passing a function [f] as argument of `assume_no_perform` guarantees that,
+      when compiling with `--effects=double-translation`, the direct-style
+      version of [f] is called, which is faster than the CPS version. As a
+      consequence, performing an effect in a transitive callee of [f] will
+      raise `Effect.Unhandled`, regardless of any effect handlers installed
+      before the call to `assume_no_perform`, unless a new effect handler was
+      installed in the meantime.
+
+      This behaviour is the same when double translation is disabled. *)
+end = struct
+  external assume_no_perform : (unit -> 'a) -> 'a = "caml_assume_no_perform"
+end
