@@ -28,7 +28,7 @@ let s x =
     if(x === undefined)
       return "undefined"
     if(typeof x === "function")
-      return "function#" + x.length + "#" + x.l
+      return "function#" + (x.l > 0 ? x.l : x.length) + "#" + x.l
     if(x.toString() == "[object Arguments]")
       return "(Arguments: " + Array.prototype.slice.call(x).toString() + ")";
     return x.toString()
@@ -86,8 +86,8 @@ let%expect_test "partial application, callback is called when all arguments are
     got 1, 2, 3, 4, 5, done
     Result: 0 |}]
 
-let%expect_test "partial application, 0 argument call is treated like 1 argument \
-                 (undefined)" =
+let%expect_test
+    "partial application, 0 argument call is treated like 1 argument (undefined)" =
   call_and_log (Js.wrap_callback cb5) {| (function(f){ return f(1)()(3)()(5) }) |};
   [%expect {|
     got 1, undefined, 3, undefined, 5, done
@@ -184,6 +184,19 @@ let%expect_test "wrap_callback_strict" =
     got 1, 2, undefined, done
     Result: 0 |}]
 
+(* Wrap callback unsafe *)
+let%expect_test "over application, extra arguments are dropped" =
+  call_and_log (Js.Unsafe.callback cb3) {| (function(f){ return f(1,2,3,4) }) |};
+  [%expect {|
+    got 1, 2, 3, done
+    Result: 0 |}]
+
+let%expect_test "partial application, extra arguments set to undefined" =
+  call_and_log (Js.Unsafe.callback cb3) {| (function(f){ return f(1,2) }) |};
+  [%expect {|
+    got 1, 2, undefined, done
+    Result: 0 |}]
+
 (* Wrap meth callback *)
 
 let%expect_test "over application, extra arguments are dropped" =
