@@ -174,7 +174,12 @@ let poly_fun_of_type_decl type_decl expr =
   fold_right_type_decl
     (fun name expr ->
       let name = name.txt in
-      Exp.fun_ nolabel None (pvar ("poly_" ^ name)) expr)
+      Ppxlib.Ast_builder.Default.pexp_fun
+        ~loc:Location.none
+        nolabel
+        None
+        (pvar ("poly_" ^ name))
+        expr)
     type_decl
     expr
 
@@ -245,9 +250,9 @@ let seqlist = function
 
 let check_record_fields =
   List.iter ~f:(function
-      | { pld_type = { ptyp_desc = Ptyp_poly _; _ }; _ } ->
-          Location.raise_errorf "%s cannot be derived for polymorphic records" deriver
-      | _ -> ())
+    | { pld_type = { ptyp_desc = Ptyp_poly _; _ }; _ } ->
+        Location.raise_errorf "%s cannot be derived for polymorphic records" deriver
+    | _ -> ())
 
 let maybe_tuple_type = function
   | [ y ] -> y
