@@ -250,7 +250,24 @@ let invoker ?(extra_types = []) uplift downlift body arguments =
   in
   let make_fun (label, pat) (label', typ) expr =
     assert (label' = label);
-    Exp.fun_ label None (Pat.constraint_ pat typ) expr
+    match expr.pexp_desc with
+    | ((Pexp_function (params, c, b)) [@if ast_version >= 502]) ->
+        let params =
+          { pparam_desc = Pparam_val (nolabel, None, Pat.constraint_ pat typ)
+          ; pparam_loc = { expr.pexp_loc with loc_ghost = true }
+          }
+          :: params
+        in
+        let c, b =
+          match c, b with
+          | ( None
+            , Pfunction_body
+                { pexp_desc = Pexp_constraint (e, ty); pexp_attributes = []; _ } ) ->
+              Some (Pconstraint ty), Pfunction_body e
+          | _ -> c, b
+        in
+        { expr with pexp_desc = Pexp_function (params, c, b) }
+    | _ -> Exp.fun_ label None (Pat.constraint_ pat typ) expr
   in
   let invoker =
     List.fold_right2
@@ -265,7 +282,20 @@ let invoker ?(extra_types = []) uplift downlift body arguments =
   let local_types =
     make_str res :: List.map (extra_types @ arguments) ~f:(fun x -> make_str (Arg.name x))
   in
-  let result = List.fold_right local_types ~init:invoker ~f:Exp.newtype in
+  let result =
+    match invoker.pexp_desc with
+    | ((Pexp_function (params, c, b)) [@if ast_version >= 502]) ->
+        { invoker with
+          pexp_desc =
+            Pexp_function
+              ( List.map local_types ~f:(fun t ->
+                    { pparam_desc = Pparam_newtype t; pparam_loc = Location.none })
+                @ params
+              , c
+              , b )
+        }
+    | _ -> List.fold_right local_types ~init:invoker ~f:Exp.newtype
+  in
   default_loc := default_loc';
   result
 
@@ -305,7 +335,7 @@ let method_call ~loc ~apply_loc obj (meth, meth_loc) args =
         | eobj :: eargs ->
             let eargs = inject_args eargs in
             Js.unsafe "meth_call" [ eobj; ocaml_str (unescape meth); eargs ])
-      (Arg.make () :: List.map args ~f:(fun (label, _) -> Arg.make ~label ()))
+      (List.map (nolabel :: List.map ~f:fst args) ~f:(fun label -> Arg.make ~label ()))
   in
   Exp.apply
     ~loc:apply_loc
@@ -455,7 +485,7 @@ let new_object constr args =
       (function
         | constr :: args -> Js.unsafe "new_obj" [ constr; inject_args args ]
         | _ -> assert false)
-      (Arg.make () :: List.map args ~f:(fun (label, _) -> Arg.make ~label ()))
+      (List.map (nolabel :: List.map ~f:fst args) ~f:(fun label -> Arg.make ~label ()))
   in
   let gloc = { constr.loc with loc_ghost = true } in
   Exp.apply
@@ -465,12 +495,10 @@ let new_object constr args =
 
 module S = Map.Make (String)
 
-(** We remove Pexp_poly as it should never be in the parsetree except after a method call.
-*)
-let format_meth body =
+let drop_pexp_poly body =
   match body.pexp_desc with
-  | Pexp_poly (e, _) -> e
-  | _ -> body
+  | Pexp_poly (e, ty) -> e, ty
+  | _ -> body, None
 
 (** Ensure basic sanity rules about fields of a literal object:
     - No duplicated declaration
@@ -506,7 +534,7 @@ type field_desc =
       string Asttypes.loc
       * Asttypes.private_flag
       * Asttypes.override_flag
-      * Parsetree.expression
+      * (Parsetree.expression * Parsetree.core_type option)
       * Arg.t list
   | Val of
       string Asttypes.loc * Prop_kind.t * Asttypes.override_flag * Parsetree.expression
@@ -520,6 +548,29 @@ let filter_map f l =
   in
   List.rev l
 
+let rec create_meth_ty exp =
+  match exp.pexp_desc with
+  | Pexp_fun (label, _, _, body) -> label :: create_meth_ty body
+  | Pexp_function _ -> [ nolabel ]
+  | Pexp_newtype (_, body) -> create_meth_ty body
+  | _ -> []
+[@@if ast_version < 502]
+
+let rec create_meth_ty exp =
+  match exp.pexp_desc with
+  | Pexp_function (params, _, body) -> (
+      List.filter_map params ~f:(function
+        | { pparam_desc = Pparam_newtype _; _ } -> None
+        | { pparam_desc = Pparam_val (label, _, _arg); _ } -> Some label)
+      @
+      match body with
+      | Pfunction_cases _ -> [ nolabel ]
+      | Pfunction_body e ->
+          (* TODO: should we recurse or not ? *)
+          create_meth_ty e)
+  | _ -> []
+[@@if ast_version >= 502]
+
 let preprocess_literal_object mappper fields :
     [ `Fields of field_desc list | `Error of _ ] =
   let check_name id names =
@@ -582,13 +633,18 @@ let preprocess_literal_object mappper fields :
         names, Val (id, kind, bang, body) :: fields
     | Pcf_method (id, priv, Cfk_concrete (bang, body)) ->
         let names = check_name id names in
-        let body = format_meth (mappper body) in
-        let rec create_meth_ty exp =
-          match exp.pexp_desc with
-          | Pexp_fun (label, _, _, body) -> Arg.make ~label () :: create_meth_ty body
-          | _ -> []
+        let body, body_ty = drop_pexp_poly (mappper body) in
+        let fun_ty =
+          List.map ~f:(fun label -> Arg.make ~label ()) (create_meth_ty body)
+        in
+
+        let body =
+          match body_ty with
+          | None -> body, None
+          | Some { ptyp_desc = Ptyp_poly _; _ } ->
+              raise_errorf ~loc:exp.pcf_loc "Polymorphic method not supported."
+          | Some ty -> body, Some ty
         in
-        let fun_ty = create_meth_ty body in
         names, Meth (id, priv, bang, body, fun_ty) :: fields
     | _ ->
         raise_errorf
@@ -640,14 +696,49 @@ let literal_object self_id (fields : field_desc list) =
   in
   let body = function
     | Val (_, _, _, body) -> body
-    | Meth (_, _, _, body, _) ->
-        Exp.fun_ ~loc:{ body.pexp_loc with loc_ghost = true } Nolabel None self_id body
+    | Meth (_, _, _, (body, ty), _) -> (
+        match body.pexp_desc, ty with
+        | ((Pexp_function (params, c, b), None) [@if ast_version >= 502]) ->
+            let params =
+              { pparam_desc = Pparam_val (nolabel, None, self_id)
+              ; pparam_loc = { body.pexp_loc with loc_ghost = true }
+              }
+              :: params
+            in
+            { body with pexp_desc = Pexp_function (params, c, b) }
+        | ((_, Some ty) [@if ast_version >= 502]) -> (
+            let e =
+              Exp.fun_
+                ~loc:{ body.pexp_loc with loc_ghost = true }
+                Nolabel
+                None
+                self_id
+                body
+            in
+            match e.pexp_desc with
+            | Pexp_function ([ param ], None, b) ->
+                { e with pexp_desc = Pexp_function ([ param ], Some (Pconstraint ty), b) }
+            | _ -> assert false)
+        | ((_, Some ty) [@if ast_version < 502]) ->
+            Exp.fun_
+              ~loc:{ body.pexp_loc with loc_ghost = true }
+              Nolabel
+              None
+              self_id
+              (Exp.constraint_ body ty)
+        | _, None ->
+            Exp.fun_
+              ~loc:{ body.pexp_loc with loc_ghost = true }
+              Nolabel
+              None
+              self_id
+              body)
   in
   let extra_types =
     List.concat
       (List.map fields ~f:(function
-          | Val _ -> []
-          | Meth (_, _, _, _, l) -> l))
+        | Val _ -> []
+        | Meth (_, _, _, _, l) -> l))
   in
   let invoker =
     invoker
@@ -691,8 +782,8 @@ let literal_object self_id (fields : field_desc list) =
                      ]))
           ])
       (List.map fields ~f:(function
-          | Val _ -> Arg.make ()
-          | Meth (_, _, _, _, _fun_ty) -> Arg.make ()))
+        | Val _ -> Arg.make ()
+        | Meth (_, _, _, _, _fun_ty) -> Arg.make ()))
   in
   let self = "self" in
   let gloc = { !default_loc with Location.loc_ghost = true } in
@@ -729,7 +820,23 @@ let literal_object self_id (fields : field_desc list) =
                (self :: List.map fields ~f:(fun f -> (name f).txt))
                ~init:fake_object
                ~f:(fun name fun_ ->
-                 Exp.fun_ ~loc:gloc nolabel None (Pat.var ~loc:gloc (mknoloc name)) fun_))
+                 match fun_.pexp_desc with
+                 | ((Pexp_function (params, c, b)) [@if ast_version >= 502]) ->
+                     let params =
+                       { pparam_desc =
+                           Pparam_val (nolabel, None, Pat.var ~loc:gloc (mknoloc name))
+                       ; pparam_loc = { fun_.pexp_loc with loc_ghost = true }
+                       }
+                       :: params
+                     in
+                     { fun_ with pexp_desc = Pexp_function (params, c, b) }
+                 | _ ->
+                     Exp.fun_
+                       ~loc:gloc
+                       nolabel
+                       None
+                       (Pat.var ~loc:gloc (mknoloc name))
+                       fun_))
             with
             pexp_attributes = [ merlin_hide ]
           }
@@ -746,13 +853,13 @@ let transform =
       let new_expr =
         match expr with
         (* obj##.var *)
-        | [%expr [%e? obj] ##. [%e? meth]] ->
+        | [%expr [%e? obj]##.[%e? meth]] ->
             let obj = self#expression obj in
             let prop = exp_to_string meth in
             let new_expr = prop_get ~loc:meth.pexp_loc obj prop in
             self#expression { new_expr with pexp_attributes }
         (* obj##.var := value *)
-        | [%expr [%e? [%expr [%e? obj] ##. [%e? meth]] as prop] := [%e? value]] ->
+        | [%expr [%e? [%expr [%e? obj]##.[%e? meth]] as prop] := [%e? value]] ->
             let obj = self#expression obj in
             let value = self#expression value in
             let prop_loc = prop.pexp_loc in
@@ -760,7 +867,7 @@ let transform =
             let new_expr = prop_set ~loc:meth.pexp_loc ~prop_loc obj prop value in
             self#expression { new_expr with pexp_attributes }
         (* obj##(meth arg1 arg2) .. *)
-        | [%expr [%e? obj] ## [%e? { pexp_desc = Pexp_apply (meth, args); _ }]] ->
+        | [%expr [%e? obj]##[%e? { pexp_desc = Pexp_apply (meth, args); _ }]] ->
             let meth_str = exp_to_string meth in
             let obj = self#expression obj in
             let args = List.map args ~f:(fun (s, e) -> s, self#expression e) in
@@ -774,7 +881,7 @@ let transform =
             in
             self#expression { new_expr with pexp_attributes }
         (* obj##meth arg1 arg2 .. *)
-        | { pexp_desc = Pexp_apply (([%expr [%e? obj] ## [%e? meth]] as prop), args)
+        | { pexp_desc = Pexp_apply (([%expr [%e? obj]##[%e? meth]] as prop), args)
           ; pexp_loc
           ; _
           } ->
@@ -791,7 +898,7 @@ let transform =
             in
             self#expression { new_expr with pexp_attributes }
         (* obj##meth *)
-        | [%expr [%e? obj] ## [%e? meth]] as expr ->
+        | [%expr [%e? obj]##[%e? meth]] as expr ->
             let obj = self#expression obj in
             let meth_str = exp_to_string meth in
             let new_expr =
