@@ -25,28 +25,39 @@ var caml_sys_fds = new Array(3);
 //Provides: caml_sys_close
 //Requires: caml_sys_fds
 function caml_sys_close(fd) {
-  var file = caml_sys_fds[fd];
-  if (file) file.close();
-  delete caml_sys_fds[fd];
+  var x = caml_sys_fds[fd];
+  if (x) {
+    x.file.close();
+    delete caml_sys_fds[fd];
+  }
   return 0;
 }
 
+//Provides: MlChanid
+function MlChanid(id) {
+  this.id = id;
+}
+
 //Provides: caml_sys_open
 //Requires: caml_raise_sys_error
 //Requires: MlFakeFd_out
 //Requires: resolve_fs_device
-//Requires: caml_jsbytes_of_string
 //Requires: fs_node_supported
 //Requires: caml_sys_fds
 //Requires: caml_sys_open_for_node
+//Requires: MlChanid
 function caml_sys_open_internal(file, idx) {
+  var chanid;
   if (idx === undefined) {
     idx = caml_sys_fds.length;
-  }
-  caml_sys_fds[idx] = file;
+    chanid = new MlChanid(idx);
+  } else if (caml_sys_fds[idx]) {
+    chanid = caml_sys_fds[idx].chanid;
+  } else chanid = new MlChanid(idx);
+  caml_sys_fds[idx] = { file: file, chanid: chanid };
   return idx | 0;
 }
-function caml_sys_open(name, flags, _perms) {
+function caml_sys_open(name, flags, perms) {
   var f = {};
   while (flags) {
     switch (flags[1]) {
@@ -58,6 +69,7 @@ function caml_sys_open(name, flags, _perms) {
         break;
       case 2:
         f.append = 1;
+        f.writeonly = 1;
         break;
       case 3:
         f.create = 1;
@@ -80,23 +92,14 @@ function caml_sys_open(name, flags, _perms) {
     }
     flags = flags[2];
   }
-  if (f.rdonly && f.wronly)
-    caml_raise_sys_error(
-      caml_jsbytes_of_string(name) +
-        " : flags Open_rdonly and Open_wronly are not compatible",
-    );
-  if (f.text && f.binary)
-    caml_raise_sys_error(
-      caml_jsbytes_of_string(name) +
-        " : flags Open_text and Open_binary are not compatible",
-    );
   var root = resolve_fs_device(name);
-  var file = root.device.open(root.rest, f);
+  var file = root.device.open(root.rest, f, perms);
   return caml_sys_open_internal(file, undefined);
 }
 (function () {
+  var is_node = fs_node_supported();
   function file(fd, flags) {
-    if (fs_node_supported()) {
+    if (is_node) {
       return caml_sys_open_for_node(fd, flags);
     } else return new MlFakeFd_out(fd, flags);
   }
@@ -105,11 +108,11 @@ function caml_sys_open(name, flags, _perms) {
     0,
   );
   caml_sys_open_internal(
-    file(1, { buffered: 2, wronly: 1, isCharacterDevice: true }),
+    file(1, { buffered: is_node ? 1 : 2, wronly: 1, isCharacterDevice: true }),
     1,
   );
   caml_sys_open_internal(
-    file(2, { buffered: 2, wronly: 1, isCharacterDevice: true }),
+    file(2, { buffered: is_node ? 1 : 2, wronly: 1, isCharacterDevice: true }),
     2,
   );
 })();
@@ -125,41 +128,64 @@ function caml_ml_set_channel_name(chanid, name) {
 }
 
 //Provides: caml_ml_channels
-var caml_ml_channels = new Array();
+//Requires: MlChanid
+class caml_ml_channels_state {
+  constructor() {
+    this.map = new globalThis.WeakMap();
+    this.opened = new globalThis.Set();
+  }
+
+  close(chanid) {
+    this.opened.delete(chanid);
+  }
+
+  get(chanid) {
+    return this.map.get(chanid);
+  }
+
+  set(chanid, val) {
+    if (val.opened) this.opened.add(chanid);
+    return this.map.set(chanid, val);
+  }
+
+  all() {
+    return this.opened.values();
+  }
+}
+
+var caml_ml_channels = new caml_ml_channels_state();
+
+//Provides: caml_ml_channel_get
+//Requires: caml_ml_channels
+function caml_ml_channel_get(id) {
+  return caml_ml_channels.get(id);
+}
 
 //Provides: caml_ml_channel_redirect
 //Requires: caml_ml_channel_get, caml_ml_channels
 function caml_ml_channel_redirect(captured, into) {
   var to_restore = caml_ml_channel_get(captured);
   var new_ = caml_ml_channel_get(into);
-  caml_ml_channels[captured] = new_; // XXX
+  caml_ml_channels.set(captured, new_);
   return to_restore;
 }
 
 //Provides: caml_ml_channel_restore
 //Requires: caml_ml_channels
 function caml_ml_channel_restore(captured, to_restore) {
-  caml_ml_channels[captured] = to_restore; // XXX
+  caml_ml_channels.set(captured, to_restore);
   return 0;
 }
 
-//Provides: caml_ml_channel_get
-//Requires: caml_ml_channels
-function caml_ml_channel_get(id) {
-  return caml_ml_channels[id]; // XXX
-}
-
 //Provides: caml_ml_out_channels_list
 //Requires: caml_ml_channels
+//Requires: caml_ml_channel_get
 function caml_ml_out_channels_list() {
   var l = 0;
-  for (var c = 0; c < caml_ml_channels.length; c++) {
-    if (
-      caml_ml_channels[c] &&
-      caml_ml_channels[c].opened &&
-      caml_ml_channels[c].out
-    )
-      l = [0, caml_ml_channels[c].fd, l];
+  var keys = caml_ml_channels.all();
+  for (var k of keys) {
+    var chan = caml_ml_channel_get(k);
+    if (chan.opened && chan.out) l = [0, k, l];
   }
   return l;
 }
@@ -168,58 +194,66 @@ function caml_ml_out_channels_list() {
 //Requires: caml_ml_channels, caml_sys_fds
 //Requires: caml_raise_sys_error
 //Requires: caml_sys_open
+//Requires: caml_io_buffer_size
 function caml_ml_open_descriptor_out(fd) {
-  var file = caml_sys_fds[fd];
-  if (file.flags.rdonly) caml_raise_sys_error("fd " + fd + " is readonly");
+  var fd_desc = caml_sys_fds[fd];
+  if (fd_desc === undefined)
+    caml_raise_sys_error("fd " + fd + " doesn't exist");
+  var file = fd_desc.file;
+  var chanid = fd_desc.chanid;
   var buffered = file.flags.buffered !== undefined ? file.flags.buffered : 1;
   var channel = {
     file: file,
-    offset: file.flags.append ? file.length() : 0,
+    offset: file.offset,
     fd: fd,
     opened: true,
     out: true,
     buffer_curr: 0,
-    buffer: new Uint8Array(65536),
+    buffer: new Uint8Array(caml_io_buffer_size),
     buffered: buffered,
   };
-  caml_ml_channels[channel.fd] = channel;
-  return channel.fd;
+  caml_ml_channels.set(chanid, channel);
+  return chanid;
 }
 
 //Provides: caml_ml_open_descriptor_in
 //Requires: caml_ml_channels, caml_sys_fds
 //Requires: caml_raise_sys_error
 //Requires: caml_sys_open
+//Requires: caml_io_buffer_size
 function caml_ml_open_descriptor_in(fd) {
-  var file = caml_sys_fds[fd];
-  if (file.flags.wronly) caml_raise_sys_error("fd " + fd + " is writeonly");
+  var fd_desc = caml_sys_fds[fd];
+  if (fd_desc === undefined)
+    caml_raise_sys_error("fd " + fd + " doesn't exist");
+  var file = fd_desc.file;
+  var chanid = fd_desc.chanid;
   var refill = null;
   var channel = {
     file: file,
-    offset: file.flags.append ? file.length() : 0,
+    offset: file.offset,
     fd: fd,
     opened: true,
     out: false,
     buffer_curr: 0,
     buffer_max: 0,
-    buffer: new Uint8Array(65536),
+    buffer: new Uint8Array(caml_io_buffer_size),
     refill: refill,
   };
-  caml_ml_channels[channel.fd] = channel;
-  return channel.fd;
+  caml_ml_channels.set(chanid, channel);
+  return chanid;
 }
 
 //Provides: caml_ml_open_descriptor_in_with_flags
 //Requires: caml_ml_open_descriptor_in
 //Version: >= 5.1
-function caml_ml_open_descriptor_in_with_flags(fd, flags) {
+function caml_ml_open_descriptor_in_with_flags(fd, _flags) {
   return caml_ml_open_descriptor_in(fd);
 }
 
 //Provides: caml_ml_open_descriptor_out_with_flags
 //Requires: caml_ml_open_descriptor_out
 //Version: >= 5.1
-function caml_ml_open_descriptor_out_with_flags(fd, flags) {
+function caml_ml_open_descriptor_out_with_flags(fd, _flags) {
   return caml_ml_open_descriptor_out(fd);
 }
 
@@ -253,10 +287,12 @@ function caml_ml_is_binary_mode(chanid) {
 //Provides: caml_ml_close_channel
 //Requires: caml_ml_flush, caml_ml_channel_get
 //Requires: caml_sys_close
+//Requires: caml_ml_channels
 function caml_ml_close_channel(chanid) {
   var chan = caml_ml_channel_get(chanid);
   if (chan.opened) {
     chan.opened = false;
+    caml_ml_channels.close(chanid);
     caml_sys_close(chan.fd);
     chan.fd = -1;
     chan.buffer = new Uint8Array(0);
@@ -270,7 +306,7 @@ function caml_ml_close_channel(chanid) {
 //Requires: caml_ml_channel_get
 function caml_ml_channel_size(chanid) {
   var chan = caml_ml_channel_get(chanid);
-  return chan.file.length();
+  return chan.file.length() | 0;
 }
 
 //Provides: caml_ml_channel_size_64
@@ -299,6 +335,7 @@ function caml_ml_set_channel_refill(chanid, f) {
 
 //Provides: caml_refill
 //Requires: caml_ml_string_length, caml_uint8_array_of_string
+//Requires: caml_raise_sys_error
 function caml_refill(chan) {
   if (chan.refill != null) {
     var str = chan.refill();
@@ -316,8 +353,10 @@ function caml_refill(chan) {
       chan.buffer_max += str_a.length;
     }
   } else {
+    if (chan.fd === -1) {
+      caml_raise_sys_error("Bad file descriptor");
+    }
     var nread = chan.file.read(
-      chan.offset,
       chan.buffer,
       chan.buffer_max,
       chan.buffer.length - chan.buffer_max,
@@ -370,7 +409,7 @@ function caml_ml_input_block(chanid, ba, i, l) {
 }
 
 //Provides: caml_input_value
-//Requires: caml_marshal_data_size, caml_input_value_from_bytes, caml_create_bytes, caml_ml_channel_get, caml_bytes_of_array
+//Requires: caml_marshal_data_size, caml_input_value_from_bytes, caml_create_bytes, caml_ml_channel_get, caml_bytes_of_uint8_array
 //Requires: caml_refill, caml_failwith, caml_raise_end_of_file
 //Requires: caml_marshal_header_size
 function caml_input_value(chanid) {
@@ -395,12 +434,12 @@ function caml_input_value(chanid) {
   if (r === 0) caml_raise_end_of_file();
   else if (r < caml_marshal_header_size)
     caml_failwith("input_value: truncated object");
-  var len = caml_marshal_data_size(caml_bytes_of_array(header), 0);
+  var len = caml_marshal_data_size(caml_bytes_of_uint8_array(header), 0);
   var buf = new Uint8Array(len + caml_marshal_header_size);
   buf.set(header, 0);
   var r = block(buf, caml_marshal_header_size, len);
   if (r < len) caml_failwith("input_value: truncated object " + r + "  " + len);
-  var res = caml_input_value_from_bytes(caml_bytes_of_array(buf), 0);
+  var res = caml_input_value_from_bytes(caml_bytes_of_uint8_array(buf), 0);
   return res;
 }
 
@@ -428,10 +467,8 @@ function caml_ml_input_char(chanid) {
 }
 
 //Provides: caml_ml_input_int
-//Requires: caml_raise_end_of_file
-//Requires: caml_ml_input_char, caml_ml_channel_get
+//Requires: caml_ml_input_char
 function caml_ml_input_int(chanid) {
-  var chan = caml_ml_channel_get(chanid);
   var res = 0;
   for (var i = 0; i < 4; i++) {
     res = ((res << 8) + caml_ml_input_char(chanid)) | 0;
@@ -451,6 +488,7 @@ function caml_seek_in(chanid, pos) {
   ) {
     chan.buffer_curr = chan.buffer_max - (chan.offset - pos);
   } else {
+    chan.file.seek(pos, 0);
     chan.offset = pos;
     chan.buffer_curr = 0;
     chan.buffer_max = 0;
@@ -475,13 +513,13 @@ function caml_ml_seek_in_64(chanid, pos) {
 //Requires: caml_ml_channel_get
 function caml_pos_in(chanid) {
   var chan = caml_ml_channel_get(chanid);
-  return (chan.offset - (chan.buffer_max - chan.buffer_curr)) | 0;
+  return chan.offset - (chan.buffer_max - chan.buffer_curr);
 }
 
 //Provides: caml_ml_pos_in
 //Requires: caml_pos_in
 function caml_ml_pos_in(chanid) {
-  return caml_pos_in(chanid);
+  return caml_pos_in(chanid) | 0;
 }
 
 //Provides: caml_ml_pos_in_64
@@ -519,15 +557,19 @@ function caml_ml_input_scan_line(chanid) {
 
 //Provides: caml_ml_flush
 //Requires: caml_raise_sys_error, caml_ml_channel_get
-//Requires: caml_subarray_to_jsbytes
+//Requires: caml_sub_uint8_array_to_jsbytes
 function caml_ml_flush(chanid) {
   var chan = caml_ml_channel_get(chanid);
   if (!chan.opened) caml_raise_sys_error("Cannot flush a closed channel");
   if (!chan.buffer || chan.buffer_curr === 0) return 0;
   if (chan.output) {
-    chan.output(caml_subarray_to_jsbytes(chan.buffer, 0, chan.buffer_curr));
+    chan.output(
+      caml_sub_uint8_array_to_jsbytes(chan.buffer, 0, chan.buffer_curr),
+    );
   } else {
-    chan.file.write(chan.offset, chan.buffer, 0, chan.buffer_curr);
+    for (var pos = 0; pos < chan.buffer_curr; ) {
+      pos += chan.file.write(chan.buffer, pos, chan.buffer_curr - pos);
+    }
   }
   chan.offset += chan.buffer_curr;
   chan.buffer_curr = 0;
@@ -625,6 +667,7 @@ function caml_output_value(chanid, v, flags) {
 function caml_seek_out(chanid, pos) {
   caml_ml_flush(chanid);
   var chan = caml_ml_channel_get(chanid);
+  chan.file.seek(pos, 0);
   chan.offset = pos;
   return 0;
 }
@@ -651,7 +694,7 @@ function caml_pos_out(chanid) {
 //Provides: caml_ml_pos_out
 //Requires: caml_pos_out
 function caml_ml_pos_out(chanid) {
-  return caml_pos_out(chanid);
+  return caml_pos_out(chanid) | 0;
 }
 
 //Provides: caml_ml_pos_out_64
@@ -661,12 +704,10 @@ function caml_ml_pos_out_64(chanid) {
 }
 
 //Provides: caml_ml_output_int
-//Requires: caml_ml_output
-//Requires: caml_string_of_array
+//Requires: caml_ml_output_ta
 function caml_ml_output_int(chanid, i) {
   var arr = [(i >> 24) & 0xff, (i >> 16) & 0xff, (i >> 8) & 0xff, i & 0xff];
-  var s = caml_string_of_array(arr);
-  caml_ml_output(chanid, s, 0, 4);
+  caml_ml_output_ta(chanid, new Uint8Array(arr), 0, 4);
   return 0;
 }
 
