@@ -47,80 +47,93 @@ var caml_marshal_constants = {
 };
 
 //Provides: UInt8ArrayReader
-//Requires: caml_string_of_array, caml_jsbytes_of_string
-function UInt8ArrayReader(s, i) {
-  this.s = s;
-  this.i = i;
-}
-UInt8ArrayReader.prototype = {
-  read8u: function () {
+//Requires: caml_string_of_uint8_array
+class UInt8ArrayReader {
+  constructor(s, i) {
+    this.s = s;
+    this.i = i;
+  }
+
+  read8u() {
     return this.s[this.i++];
-  },
-  read8s: function () {
+  }
+
+  read8s() {
     return (this.s[this.i++] << 24) >> 24;
-  },
-  read16u: function () {
+  }
+
+  read16u() {
     var s = this.s,
       i = this.i;
     this.i = i + 2;
     return (s[i] << 8) | s[i + 1];
-  },
-  read16s: function () {
+  }
+
+  read16s() {
     var s = this.s,
       i = this.i;
     this.i = i + 2;
     return ((s[i] << 24) >> 16) | s[i + 1];
-  },
-  read32u: function () {
+  }
+
+  read32u() {
     var s = this.s,
       i = this.i;
     this.i = i + 4;
     return ((s[i] << 24) | (s[i + 1] << 16) | (s[i + 2] << 8) | s[i + 3]) >>> 0;
-  },
-  read32s: function () {
+  }
+
+  read32s() {
     var s = this.s,
       i = this.i;
     this.i = i + 4;
     return (s[i] << 24) | (s[i + 1] << 16) | (s[i + 2] << 8) | s[i + 3];
-  },
-  readstr: function (len) {
+  }
+
+  readstr(len) {
     var i = this.i;
     this.i = i + len;
-    return caml_string_of_array(this.s.subarray(i, i + len));
-  },
-  readuint8array: function (len) {
+    return caml_string_of_uint8_array(this.s.subarray(i, i + len));
+  }
+
+  readuint8array(len) {
     var i = this.i;
     this.i = i + len;
     return this.s.subarray(i, i + len);
-  },
-};
-
-//Provides: MlStringReader
-//Requires: caml_string_of_jsbytes, caml_jsbytes_of_string
-function MlStringReader(s, i) {
-  this.s = caml_jsbytes_of_string(s);
-  this.i = i;
+  }
 }
-MlStringReader.prototype = {
-  read8u: function () {
+
+//Provides: JsStringReader
+//Requires: caml_string_of_jsbytes
+class JsStringReader {
+  constructor(s, i) {
+    this.s = s;
+    this.i = i;
+  }
+
+  read8u() {
     return this.s.charCodeAt(this.i++);
-  },
-  read8s: function () {
+  }
+
+  read8s() {
     return (this.s.charCodeAt(this.i++) << 24) >> 24;
-  },
-  read16u: function () {
+  }
+
+  read16u() {
     var s = this.s,
       i = this.i;
     this.i = i + 2;
     return (s.charCodeAt(i) << 8) | s.charCodeAt(i + 1);
-  },
-  read16s: function () {
+  }
+
+  read16s() {
     var s = this.s,
       i = this.i;
     this.i = i + 2;
     return ((s.charCodeAt(i) << 24) >> 16) | s.charCodeAt(i + 1);
-  },
-  read32u: function () {
+  }
+
+  read32u() {
     var s = this.s,
       i = this.i;
     this.i = i + 4;
@@ -131,8 +144,9 @@ MlStringReader.prototype = {
         s.charCodeAt(i + 3)) >>>
       0
     );
-  },
-  read32s: function () {
+  }
+
+  read32s() {
     var s = this.s,
       i = this.i;
     this.i = i + 4;
@@ -142,13 +156,15 @@ MlStringReader.prototype = {
       (s.charCodeAt(i + 2) << 8) |
       s.charCodeAt(i + 3)
     );
-  },
-  readstr: function (len) {
+  }
+
+  readstr(len) {
     var i = this.i;
     this.i = i + len;
     return caml_string_of_jsbytes(this.s.slice(i, i + len));
-  },
-  readuint8array: function (len) {
+  }
+
+  readuint8array(len) {
     var b = new Uint8Array(len);
     var s = this.s;
     var i = this.i;
@@ -157,73 +173,8 @@ MlStringReader.prototype = {
     }
     this.i = i + len;
     return b;
-  },
-};
-
-//Provides: BigStringReader
-//Requires: caml_string_of_array, caml_ba_get_1
-function BigStringReader(bs, i) {
-  this.s = bs;
-  this.i = i;
+  }
 }
-BigStringReader.prototype = {
-  read8u: function () {
-    return caml_ba_get_1(this.s, this.i++);
-  },
-  read8s: function () {
-    return (caml_ba_get_1(this.s, this.i++) << 24) >> 24;
-  },
-  read16u: function () {
-    var s = this.s,
-      i = this.i;
-    this.i = i + 2;
-    return (caml_ba_get_1(s, i) << 8) | caml_ba_get_1(s, i + 1);
-  },
-  read16s: function () {
-    var s = this.s,
-      i = this.i;
-    this.i = i + 2;
-    return ((caml_ba_get_1(s, i) << 24) >> 16) | caml_ba_get_1(s, i + 1);
-  },
-  read32u: function () {
-    var s = this.s,
-      i = this.i;
-    this.i = i + 4;
-    return (
-      ((caml_ba_get_1(s, i) << 24) |
-        (caml_ba_get_1(s, i + 1) << 16) |
-        (caml_ba_get_1(s, i + 2) << 8) |
-        caml_ba_get_1(s, i + 3)) >>>
-      0
-    );
-  },
-  read32s: function () {
-    var s = this.s,
-      i = this.i;
-    this.i = i + 4;
-    return (
-      (caml_ba_get_1(s, i) << 24) |
-      (caml_ba_get_1(s, i + 1) << 16) |
-      (caml_ba_get_1(s, i + 2) << 8) |
-      caml_ba_get_1(s, i + 3)
-    );
-  },
-  readstr: function (len) {
-    var i = this.i;
-    var arr = new Array(len);
-    for (var j = 0; j < len; j++) {
-      arr[j] = caml_ba_get_1(this.s, i + j);
-    }
-    this.i = i + len;
-    return caml_string_of_array(arr);
-  },
-  readuint8array: function (len) {
-    var i = this.i;
-    var offset = this.offset(i);
-    this.i = i + len;
-    return this.s.data.subarray(offset, offset + len);
-  },
-};
 
 //Provides: caml_float_of_bytes
 //Requires: caml_int64_float_of_bits, caml_int64_of_bytes
@@ -231,22 +182,18 @@ function caml_float_of_bytes(a) {
   return caml_int64_float_of_bits(caml_int64_of_bytes(a));
 }
 
-//Provides: caml_input_value_from_string mutable
-//Requires: MlStringReader, caml_input_value_from_reader
-//Version: < 4.12
-function caml_input_value_from_string(s, ofs) {
-  var reader = new MlStringReader(s, typeof ofs === "number" ? ofs : ofs[0]);
-  return caml_input_value_from_reader(reader, ofs);
-}
-
 //Provides: caml_input_value_from_bytes mutable
-//Requires: MlStringReader, caml_input_value_from_reader, caml_string_of_bytes
+//Requires: JsStringReader, UInt8ArrayReader
+//Requires: caml_input_value_from_reader
+//Requires: caml_ml_bytes_content
 function caml_input_value_from_bytes(s, ofs) {
-  var reader = new MlStringReader(
-    caml_string_of_bytes(s),
-    typeof ofs === "number" ? ofs : ofs[0],
-  );
-  return caml_input_value_from_reader(reader, ofs);
+  var c = caml_ml_bytes_content(s);
+  var ofs = typeof ofs === "number" ? ofs : ofs[0];
+  var reader =
+    c instanceof Uint8Array
+      ? new UInt8ArrayReader(c, ofs)
+      : new JsStringReader(c, ofs);
+  return caml_input_value_from_reader(reader);
 }
 
 //Provides: caml_int64_unmarshal
@@ -331,7 +278,8 @@ var caml_custom_ops = {
 //Requires: caml_float_of_bytes, caml_custom_ops
 //Requires: UInt8ArrayReader
 //Requires: caml_decompress_input
-function caml_input_value_from_reader(reader, ofs) {
+//Requires: caml_set_oo_id
+function caml_input_value_from_reader(reader) {
   function readvlq(overflow) {
     var c = reader.read8u();
     var n = c & 0x7f;
@@ -343,6 +291,7 @@ function caml_input_value_from_reader(reader, ofs) {
     }
     return n;
   }
+  var old_pos = reader.i;
   var magic = reader.read32u();
   switch (magic) {
     case 0x8495a6be /* Intext_magic_number_small */:
@@ -378,7 +327,11 @@ function caml_input_value_from_reader(reader, ofs) {
       caml_failwith("caml_input_value_from_reader: bad object");
       break;
   }
+  if (header_len !== reader.i - old_pos) {
+    caml_failwith("caml_input_value_from_reader: invalid header");
+  }
   var stack = [];
+  var objects = [];
   var intern_obj_table = num_objects > 0 ? [] : null;
   var obj_counter = 0;
   function intern_rec(reader) {
@@ -390,6 +343,7 @@ function caml_input_value_from_reader(reader, ofs) {
         var v = [tag];
         if (size === 0) return v;
         if (intern_obj_table) intern_obj_table[obj_counter++] = v;
+        if (tag === 248) objects.push(v);
         stack.push(v, size);
         return v;
       } else return code & 0x3f;
@@ -429,6 +383,7 @@ function caml_input_value_from_reader(reader, ofs) {
             var v = [tag];
             if (size === 0) return v;
             if (intern_obj_table) intern_obj_table[obj_counter++] = v;
+            if (tag === 248) objects.push(v);
             stack.push(v, size);
             return v;
           case 0x13: //cst.CODE_BLOCK64:
@@ -530,7 +485,6 @@ function caml_input_value_from_reader(reader, ofs) {
                 reader.read32s();
                 break;
             }
-            var old_pos = reader.i;
             var size = [0];
             var v = ops.deserialize(reader, size);
             if (expected_size !== undefined) {
@@ -565,6 +519,10 @@ function caml_input_value_from_reader(reader, ofs) {
     if (d < size) stack.push(v, size);
     v[d] = intern_rec(reader);
   }
+  while (objects.length > 0) {
+    var x = objects.pop();
+    if (x[2] >= 0) caml_set_oo_id(x);
+  }
   return res;
 }
 
@@ -623,46 +581,25 @@ function caml_marshal_data_size(s, ofs) {
 }
 
 //Provides: MlObjectTable
-var MlObjectTable;
-if (typeof globalThis.Map === "undefined") {
-  MlObjectTable = (function () {
-    /* polyfill (using linear search) */
-    function NaiveLookup(objs) {
-      this.objs = objs;
-    }
-    NaiveLookup.prototype.get = function (v) {
-      for (var i = 0; i < this.objs.length; i++) {
-        if (this.objs[i] === v) return i;
-      }
-    };
-    NaiveLookup.prototype.set = function () {
-      // Do nothing here. [MlObjectTable.store] will push to [this.objs] directly.
-    };
-
-    return function MlObjectTable() {
-      this.objs = [];
-      this.lookup = new NaiveLookup(this.objs);
-    };
-  })();
-} else {
-  MlObjectTable = function MlObjectTable() {
+class MlObjectTable {
+  constructor() {
     this.objs = [];
     this.lookup = new globalThis.Map();
-  };
+  }
+
+  store(v) {
+    this.lookup.set(v, this.objs.length);
+    this.objs.push(v);
+  }
+
+  recall(v) {
+    var i = this.lookup.get(v);
+    return i === undefined
+      ? undefined
+      : this.objs.length - i; /* index is relative */
+  }
 }
 
-MlObjectTable.prototype.store = function (v) {
-  this.lookup.set(v, this.objs.length);
-  this.objs.push(v);
-};
-
-MlObjectTable.prototype.recall = function (v) {
-  var i = this.lookup.get(v);
-  return i === undefined
-    ? undefined
-    : this.objs.length - i; /* index is relative */
-};
-
 //Provides: caml_output_val
 //Requires: caml_int64_to_bytes, caml_failwith
 //Requires: caml_int64_bits_of_float
@@ -671,40 +608,46 @@ MlObjectTable.prototype.recall = function (v) {
 //Requires: MlObjectTable, caml_list_to_js_array, caml_custom_ops
 //Requires: caml_invalid_argument,caml_string_of_jsbytes, caml_is_continuation_tag
 var caml_output_val = (function () {
-  function Writer() {
-    this.chunk = [];
-  }
-  Writer.prototype = {
-    chunk_idx: 20,
-    block_len: 0,
-    obj_counter: 0,
-    size_32: 0,
-    size_64: 0,
-    write: function (size, value) {
+  class Writer {
+    constructor() {
+      this.chunk = [];
+      this.chunk_idx = 20;
+      this.block_len = 0;
+      this.obj_counter = 0;
+      this.size_32 = 0;
+      this.size_64 = 0;
+    }
+
+    write(size, value) {
       for (var i = size - 8; i >= 0; i -= 8)
         this.chunk[this.chunk_idx++] = (value >> i) & 0xff;
-    },
-    write_at: function (pos, size, value) {
+    }
+
+    write_at(pos, size, value) {
       var pos = pos;
       for (var i = size - 8; i >= 0; i -= 8)
         this.chunk[pos++] = (value >> i) & 0xff;
-    },
-    write_code: function (size, code, value) {
+    }
+
+    write_code(size, code, value) {
       this.chunk[this.chunk_idx++] = code;
       for (var i = size - 8; i >= 0; i -= 8)
         this.chunk[this.chunk_idx++] = (value >> i) & 0xff;
-    },
-    write_shared: function (offset) {
+    }
+
+    write_shared(offset) {
       if (offset < 1 << 8)
         this.write_code(8, 0x04 /*cst.CODE_SHARED8*/, offset);
       else if (offset < 1 << 16)
         this.write_code(16, 0x05 /*cst.CODE_SHARED16*/, offset);
       else this.write_code(32, 0x06 /*cst.CODE_SHARED32*/, offset);
-    },
-    pos: function () {
+    }
+
+    pos() {
       return this.chunk_idx;
-    },
-    finalize: function () {
+    }
+
+    finalize() {
       this.block_len = this.chunk_idx - 20;
       this.chunk_idx = 0;
       this.write(32, 0x8495a6be);
@@ -713,8 +656,8 @@ var caml_output_val = (function () {
       this.write(32, this.size_32);
       this.write(32, this.size_64);
       return this.chunk;
-    },
-  };
+    }
+  }
   return function (v, flags) {
     flags = caml_list_to_js_array(flags);
 
@@ -769,9 +712,8 @@ var caml_output_val = (function () {
           for (var i = 0; i < name.length; i++)
             writer.write(8, name.charCodeAt(i));
           writer.write(8, 0);
-          var old_pos = writer.pos();
           ops.serialize(writer, v, sz_32_64);
-          if (ops.fixed_length !== writer.pos() - old_pos)
+          if (ops.fixed_length !== sz_32_64[0])
             caml_failwith(
               "output_value: incorrect fixed sizes specified by " + name,
             );
@@ -863,27 +805,27 @@ var caml_output_val = (function () {
     }
     if (intern_obj_table) writer.obj_counter = intern_obj_table.objs.length;
     writer.finalize();
-    return writer.chunk;
+    return new Uint8Array(writer.chunk);
   };
 })();
 
 //Provides: caml_output_value_to_string mutable
-//Requires: caml_output_val, caml_string_of_array
+//Requires: caml_output_val, caml_string_of_uint8_array
 function caml_output_value_to_string(v, flags) {
-  return caml_string_of_array(caml_output_val(v, flags));
+  return caml_string_of_uint8_array(caml_output_val(v, flags));
 }
 
 //Provides: caml_output_value_to_bytes mutable
-//Requires: caml_output_val, caml_bytes_of_array
+//Requires: caml_output_val, caml_bytes_of_uint8_array
 function caml_output_value_to_bytes(v, flags) {
-  return caml_bytes_of_array(caml_output_val(v, flags));
+  return caml_bytes_of_uint8_array(caml_output_val(v, flags));
 }
 
 //Provides: caml_output_value_to_buffer
-//Requires: caml_output_val, caml_failwith, caml_blit_bytes
+//Requires: caml_output_val, caml_failwith, caml_blit_bytes, caml_bytes_of_uint8_array
 function caml_output_value_to_buffer(s, ofs, len, v, flags) {
   var t = caml_output_val(v, flags);
   if (t.length > len) caml_failwith("Marshal.to_buffer: buffer overflow");
-  caml_blit_bytes(t, 0, s, ofs, t.length);
+  caml_blit_bytes(caml_bytes_of_uint8_array(t), 0, s, ofs, t.length);
   return 0;
 }
