@@ -48,24 +48,7 @@
 
 //Provides: caml_str_repeat
 function caml_str_repeat(n, s) {
-  if (n === 0) return "";
-  if (s.repeat) {
-    return s.repeat(n);
-  } // ECMAscript 6 and Firefox 24+
-  var r = "",
-    l = 0;
-  for (;;) {
-    if (n & 1) r += s;
-    n >>= 1;
-    if (n === 0) return r;
-    s += s;
-    l++;
-    if (l === 9) {
-      s.slice(0, 1); // flatten the string
-      // then, the flattening of the whole string will be faster,
-      // as it will be composed of larger pieces
-    }
-  }
+  return s.repeat(n);
 }
 
 //Provides: caml_subarray_to_jsbytes
@@ -82,113 +65,18 @@ function caml_subarray_to_jsbytes(a, i, len) {
   return s;
 }
 
-//Provides: caml_utf8_of_utf16
-function caml_utf8_of_utf16(s) {
-  for (var b = "", t = b, c, d, i = 0, l = s.length; i < l; i++) {
-    c = s.charCodeAt(i);
-    if (c < 0x80) {
-      for (var j = i + 1; j < l && (c = s.charCodeAt(j)) < 0x80; j++);
-      if (j - i > 512) {
-        t.slice(0, 1);
-        b += t;
-        t = "";
-        b += s.slice(i, j);
-      } else t += s.slice(i, j);
-      if (j === l) break;
-      i = j;
-    }
-    if (c < 0x800) {
-      t += String.fromCharCode(0xc0 | (c >> 6));
-      t += String.fromCharCode(0x80 | (c & 0x3f));
-    } else if (c < 0xd800 || c >= 0xdfff) {
-      t += String.fromCharCode(
-        0xe0 | (c >> 12),
-        0x80 | ((c >> 6) & 0x3f),
-        0x80 | (c & 0x3f),
-      );
-    } else if (
-      c >= 0xdbff ||
-      i + 1 === l ||
-      (d = s.charCodeAt(i + 1)) < 0xdc00 ||
-      d > 0xdfff
-    ) {
-      // Unmatched surrogate pair, replaced by \ufffd (replacement character)
-      t += "\xef\xbf\xbd";
-    } else {
-      i++;
-      c = (c << 10) + d - 0x35fdc00;
-      t += String.fromCharCode(
-        0xf0 | (c >> 18),
-        0x80 | ((c >> 12) & 0x3f),
-        0x80 | ((c >> 6) & 0x3f),
-        0x80 | (c & 0x3f),
-      );
-    }
-    if (t.length > 1024) {
-      t.slice(0, 1);
-      b += t;
-      t = "";
-    }
-  }
-  return b + t;
-}
-
-//Provides: caml_utf16_of_utf8
-function caml_utf16_of_utf8(s) {
-  for (var b = "", t = "", c, c1, c2, v, i = 0, l = s.length; i < l; i++) {
-    c1 = s.charCodeAt(i);
-    if (c1 < 0x80) {
-      for (var j = i + 1; j < l && (c1 = s.charCodeAt(j)) < 0x80; j++);
-      if (j - i > 512) {
-        t.slice(0, 1);
-        b += t;
-        t = "";
-        b += s.slice(i, j);
-      } else t += s.slice(i, j);
-      if (j === l) break;
-      i = j;
-    }
-    v = 1;
-    if (++i < l && ((c2 = s.charCodeAt(i)) & -64) === 128) {
-      c = c2 + (c1 << 6);
-      if (c1 < 0xe0) {
-        v = c - 0x3080;
-        if (v < 0x80) v = 1;
-      } else {
-        v = 2;
-        if (++i < l && ((c2 = s.charCodeAt(i)) & -64) === 128) {
-          c = c2 + (c << 6);
-          if (c1 < 0xf0) {
-            v = c - 0xe2080;
-            if (v < 0x800 || (v >= 0xd7ff && v < 0xe000)) v = 2;
-          } else {
-            v = 3;
-            if (
-              ++i < l &&
-              ((c2 = s.charCodeAt(i)) & -64) === 128 &&
-              c1 < 0xf5
-            ) {
-              v = c2 - 0x3c82080 + (c << 6);
-              if (v < 0x10000 || v > 0x10ffff) v = 3;
-            }
-          }
-        }
-      }
-    }
-    if (v < 4) {
-      // Invalid sequence
-      i -= v;
-      t += "\ufffd";
-    } else if (v > 0xffff)
-      t += String.fromCharCode(0xd7c0 + (v >> 10), 0xdc00 + (v & 0x3ff));
-    else t += String.fromCharCode(v);
-    if (t.length > 1024) {
-      t.slice(0, 1);
-      b += t;
-      t = "";
-    }
-  }
-  return b + t;
+//Provides: caml_sub_uint8_array_to_jsbytes
+//Weakdef
+// Pre ECMAScript 5, [apply] would not support array-like object.
+// In such setup, Typed_array would be implemented as polyfill, and [f.apply] would
+// fail here. Mark the primitive as Weakdef, so that people can override it easily.
+function caml_sub_uint8_array_to_jsbytes(a, i, len) {
+  var f = String.fromCharCode;
+  if (i === 0 && len <= 4096 && len === a.length) return f.apply(null, a);
+  var s = "";
+  for (; 0 < len; i += 1024, len -= 1024)
+    s += f.apply(null, a.subarray(i, i + Math.min(len, 1024)));
+  return s;
 }
 
 //Provides: jsoo_is_ascii
@@ -330,7 +218,7 @@ function caml_bytes_get(s, i) {
 //Provides: caml_string_set
 //Requires: caml_failwith
 //If: js-string
-function caml_string_set(s, i, c) {
+function caml_string_set(_s, _i, _c) {
   caml_failwith("caml_string_set");
 }
 
@@ -387,54 +275,70 @@ function caml_bytes_set(s, i, c) {
   return caml_bytes_unsafe_set(s, i, c);
 }
 
+//Provides: jsoo_text_encoder
+var jsoo_text_encoder = new TextEncoder();
+
+//Provides: jsoo_text_decoder
+var jsoo_text_decoder = new TextDecoder();
+
 //Provides: caml_bytes_of_utf16_jsstring
-//Requires: jsoo_is_ascii, caml_utf8_of_utf16, MlBytes
+//Requires: MlBytes, jsoo_text_encoder
+//Requires: jsoo_is_ascii
 function caml_bytes_of_utf16_jsstring(s) {
-  var tag = 9 /* BYTES | ASCII */;
-  if (!jsoo_is_ascii(s))
-    (tag = 8) /* BYTES | NOT_ASCII */, (s = caml_utf8_of_utf16(s));
-  return new MlBytes(tag, s, s.length);
+  if (jsoo_is_ascii(s)) {
+    return new MlBytes(9, s, s.length);
+  } else {
+    var a = jsoo_text_encoder.encode(s);
+    return new MlBytes(4, a, a.length);
+  }
 }
 
 //Provides: MlBytes
-//Requires: caml_convert_string_to_bytes, jsoo_is_ascii, caml_utf16_of_utf8
-function MlBytes(tag, contents, length) {
-  this.t = tag;
-  this.c = contents;
-  this.l = length;
-}
-MlBytes.prototype.toString = function () {
-  switch (this.t) {
-    case 9: /*BYTES | ASCII*/
-    case 8 /*BYTES | NOT_ASCII*/:
-      return this.c;
-    case 4: /* ARRAY */
-    case 2 /* PARTIAL */:
-      // biome-ignore lint/suspicious/noFallthroughSwitchClause:
-      caml_convert_string_to_bytes(this);
-    // fallthrough
-    case 0 /*BYTES | UNKOWN*/:
-      if (jsoo_is_ascii(this.c)) this.t = 9; /*BYTES | ASCII*/
-      else this.t = 8; /*BYTES | NOT_ASCII*/
-      return this.c;
+//Requires: caml_convert_string_to_bytes, jsoo_is_ascii
+//Requires: caml_uint8_array_of_bytes
+//Requires: jsoo_text_decoder
+class MlBytes {
+  constructor(tag, contents, length) {
+    this.t = tag;
+    this.c = contents;
+    this.l = length;
   }
-};
-MlBytes.prototype.toUtf16 = function () {
-  var r = this.toString();
-  if (this.t === 9) return r;
-  return caml_utf16_of_utf8(r);
-};
-MlBytes.prototype.slice = function () {
-  var content = this.t === 4 ? this.c.slice() : this.c;
-  return new MlBytes(this.t, content, this.l);
-};
+
+  toString() {
+    switch (this.t) {
+      case 9: /*BYTES | ASCII*/
+      case 8 /*BYTES | NOT_ASCII*/:
+        return this.c;
+      case 4: /* ARRAY */
+      case 2 /* PARTIAL */:
+        // biome-ignore lint/suspicious/noFallthroughSwitchClause:
+        caml_convert_string_to_bytes(this);
+      // fallthrough
+      case 0 /*BYTES | UNKOWN*/:
+        if (jsoo_is_ascii(this.c)) this.t = 9; /*BYTES | ASCII*/
+        else this.t = 8; /*BYTES | NOT_ASCII*/
+        return this.c;
+    }
+  }
+
+  toUtf16() {
+    if (this.t === 9) return this.c;
+    var a = caml_uint8_array_of_bytes(this);
+    return jsoo_text_decoder.decode(a);
+  }
+
+  slice() {
+    var content = this.t === 4 ? this.c.slice() : this.c;
+    return new MlBytes(this.t, content, this.l);
+  }
+}
 
 //Provides: caml_convert_string_to_bytes
-//Requires: caml_str_repeat, caml_subarray_to_jsbytes
+//Requires: caml_str_repeat, caml_sub_uint8_array_to_jsbytes
 function caml_convert_string_to_bytes(s) {
   /* Assumes not BYTES */
   if (s.t === 2 /* PARTIAL */) s.c += caml_str_repeat(s.l - s.c.length, "\0");
-  else s.c = caml_subarray_to_jsbytes(s.c, 0, s.c.length);
+  else s.c = caml_sub_uint8_array_to_jsbytes(s.c, 0, s.c.length);
   s.t = 0; /*BYTES | UNKOWN*/
 }
 
@@ -480,7 +384,7 @@ function caml_create_string(len) {
 //Provides: caml_create_string const
 //Requires: caml_invalid_argument
 //If: js-string
-function caml_create_string(len) {
+function caml_create_string(_len) {
   caml_invalid_argument("String.create");
 }
 
@@ -497,6 +401,20 @@ function caml_string_of_array(a) {
   return caml_string_of_jsbytes(caml_subarray_to_jsbytes(a, 0, a.length));
 }
 
+//Provides: caml_string_of_uint8_array
+//Requires: caml_sub_uint8_array_to_jsbytes
+//If: js-string
+function caml_string_of_uint8_array(a) {
+  return caml_sub_uint8_array_to_jsbytes(a, 0, a.length);
+}
+
+//Provides: caml_string_of_uint8_array
+//Requires: caml_bytes_of_uint8_array
+//If: !js-string
+function caml_string_of_uint8_array(a) {
+  return caml_bytes_of_uint8_array(a.slice());
+}
+
 //Provides: caml_bytes_of_array
 //Requires: MlBytes
 function caml_bytes_of_array(a) {
@@ -506,6 +424,12 @@ function caml_bytes_of_array(a) {
   return new MlBytes(4, a, a.length);
 }
 
+//Provides: caml_bytes_of_uint8_array
+//Requires: MlBytes
+function caml_bytes_of_uint8_array(a) {
+  return new MlBytes(4, a, a.length);
+}
+
 //Provides: caml_bytes_compare mutable
 //Requires: caml_convert_string_to_bytes
 function caml_bytes_compare(s1, s2) {
@@ -596,7 +520,7 @@ function caml_fill_bytes(s, i, l, c) {
 }
 
 //Provides: caml_blit_bytes
-//Requires: caml_subarray_to_jsbytes, caml_convert_bytes_to_array
+//Requires: caml_sub_uint8_array_to_jsbytes, caml_convert_bytes_to_array
 function caml_blit_bytes(s1, i1, s2, i2, len) {
   if (len === 0) return 0;
   if (
@@ -605,7 +529,7 @@ function caml_blit_bytes(s1, i1, s2, i2, len) {
   ) {
     s2.c =
       s1.t === 4 /* ARRAY */
-        ? caml_subarray_to_jsbytes(s1.c, i1, len)
+        ? caml_sub_uint8_array_to_jsbytes(s1.c, i1, len)
         : i1 === 0 && s1.c.length === len
           ? s1.c
           : s1.c.slice(i1, i1 + len);
@@ -613,7 +537,7 @@ function caml_blit_bytes(s1, i1, s2, i2, len) {
   } else if (s2.t === 2 /* PARTIAL */ && i2 === s2.c.length) {
     s2.c +=
       s1.t === 4 /* ARRAY */
-        ? caml_subarray_to_jsbytes(s1.c, i1, len)
+        ? caml_sub_uint8_array_to_jsbytes(s1.c, i1, len)
         : i1 === 0 && s1.c.length === len
           ? s1.c
           : s1.c.slice(i1, i1 + len);
@@ -649,19 +573,19 @@ function caml_ml_bytes_length(s) {
   return s.l;
 }
 
-//Provides: caml_string_concat
+//Provides: caml_string_concat const
 //If: js-string
 function caml_string_concat(a, b) {
   return a + b;
 }
 
-//Provides: caml_string_concat
+//Provides: caml_string_concat const
 //Requires: caml_convert_string_to_bytes, MlBytes
 //If: !js-string
 function caml_string_concat(s1, s2) {
   s1.t & 6 && caml_convert_string_to_bytes(s1);
   s2.t & 6 && caml_convert_string_to_bytes(s2);
-  return new MlBytes(s1.t, s1.c + s2.c, s1.l + s2.l);
+  return new MlBytes(0, s1.c + s2.c, s1.l + s2.l);
 }
 
 //Provides: caml_string_unsafe_get const
@@ -728,20 +652,35 @@ function caml_jsbytes_of_string(x) {
   return x;
 }
 
+//Provides: jsoo_text_decoder_buff
+var jsoo_text_decoder_buff = new ArrayBuffer(1024);
+
 //Provides: caml_jsstring_of_string const
-//Requires: jsoo_is_ascii, caml_utf16_of_utf8
+//Requires: jsoo_is_ascii
+//Requires: jsoo_text_decoder
+//Requires: jsoo_text_decoder_buff
 //If: js-string
 function caml_jsstring_of_string(s) {
   if (jsoo_is_ascii(s)) return s;
-  return caml_utf16_of_utf8(s);
+  var a =
+    s.length <= jsoo_text_decoder_buff.length
+      ? new Uint8Array(jsoo_text_decoder_buff, 0, s.length)
+      : new Uint8Array(s.length);
+  for (var i = 0; i < s.length; i++) {
+    a[i] = s.charCodeAt(i);
+  }
+  return jsoo_text_decoder.decode(a);
 }
 
 //Provides: caml_string_of_jsstring const
-//Requires: jsoo_is_ascii, caml_utf8_of_utf16, caml_string_of_jsbytes
+//Requires: caml_string_of_array
+//Requires: jsoo_text_encoder
+//Requires: jsoo_is_ascii, caml_string_of_jsbytes
 //If: js-string
 function caml_string_of_jsstring(s) {
   if (jsoo_is_ascii(s)) return caml_string_of_jsbytes(s);
-  else return caml_string_of_jsbytes(caml_utf8_of_utf16(s));
+  var a = jsoo_text_encoder.encode(s);
+  return caml_string_of_array(a);
 }
 
 //Provides: caml_bytes_of_jsbytes const
@@ -861,7 +800,6 @@ function caml_ml_bytes_content(s) {
 }
 
 //Provides: caml_is_ml_string
-//Requires: jsoo_is_ascii
 //If: js-string
 function caml_is_ml_string(s) {
   // biome-ignore lint/suspicious/noControlCharactersInRegex: expected
