@@ -1,11 +1,13 @@
 //Provides: MlMutex
-function MlMutex() {
-  this.locked = false;
+class MlMutex {
+  constructor() {
+    this.locked = false;
+  }
 }
 
 //Provides: caml_ml_mutex_new
 //Requires: MlMutex
-function caml_ml_mutex_new(unit) {
+function caml_ml_mutex_new(_unit) {
   return new MlMutex();
 }
 
