@@ -49,13 +49,22 @@ let read_cmi ~dir cmi =
   try with_name (Js_of_ocaml_compiler.Stdlib.String.uncapitalize_ascii cmi)
   with Not_found -> (
     try with_name (Js_of_ocaml_compiler.Stdlib.String.capitalize_ascii cmi)
-    with Not_found ->
-      Format.eprintf
-        "Could not find cmi %s or %s in %s@."
-        (Js_of_ocaml_compiler.Stdlib.String.capitalize_ascii cmi)
-        (Js_of_ocaml_compiler.Stdlib.String.uncapitalize_ascii cmi)
-        dir;
-      raise Not_found)
+    with Not_found -> (
+      match cmi with
+      (* HACK: here a list of known "hidden" cmi from the OCaml distribution. *)
+      | "Dynlink_config.cmi"
+      | "Dynlink_types.cmi"
+      | "Dynlink_platform_intf.cmi"
+      | "Dynlink_common.cmi"
+      | "Dynlink_symtable.cmi"
+      | "Dynlink_compilerlibs.cmi" -> raise Not_found
+      | cmi ->
+          Format.eprintf
+            "Could not find cmi %s or %s in %s@."
+            (Js_of_ocaml_compiler.Stdlib.String.capitalize_ascii cmi)
+            (Js_of_ocaml_compiler.Stdlib.String.uncapitalize_ascii cmi)
+            dir;
+          raise Not_found))
 
 let cmis_of_cma ~dir cma_path =
   let cma_path =
@@ -75,19 +84,19 @@ let cmis_of_package pkg : string list =
       try Findlib.package_property [ "byte" ] pkg "archive"
       with exc -> if String.equal pkg "stdlib" then "stdlib.cma" else raise exc
     in
-    let l = String.split_char ~sep:' ' archive in
+    let l = String.split_on_char ~sep:' ' archive in
     List.iter l ~f:(function
-        | "" -> ()
-        | x ->
-            if Filename.check_suffix x ".cmo"
-            then
-              let u = Filename.chop_suffix x ".cmo" in
-              add (read_cmi ~dir (u ^ ".cmi"))
-            else if Filename.check_suffix x ".cma"
-            then List.iter (cmis_of_cma ~dir x) ~f:add
-            else if Filename.check_suffix x ".cmi"
-            then add (read_cmi ~dir (Filename.chop_suffix x ".cmi"))
-            else Format.eprintf "Wrong extension for archive %s@." x);
+      | "" -> ()
+      | x ->
+          if Filename.check_suffix x ".cmo"
+          then
+            let u = Filename.chop_suffix x ".cmo" in
+            add (read_cmi ~dir (u ^ ".cmi"))
+          else if Filename.check_suffix x ".cma"
+          then List.iter (cmis_of_cma ~dir x) ~f:add
+          else if Filename.check_suffix x ".cmi"
+          then add (read_cmi ~dir (Filename.chop_suffix x ".cmi"))
+          else Format.eprintf "Wrong extension for archive %s@." x);
     !fs
   with exn ->
     Format.eprintf "Error for package %s@." pkg;
@@ -107,7 +116,7 @@ let cmis files =
       match kind file with
       | `Pkg pkg -> cmis_of_package pkg @ fs
       | `Cmi s -> (
-          match String.split_char ~sep:':' s with
+          match String.split_on_char ~sep:':' s with
           | [ s ] -> read_cmi ~dir:"." s :: fs
           | [ pkg; s ] ->
               let dir = Findlib.package_directory pkg in
