@@ -145,7 +145,7 @@ let resize ~container ~textbox () =
   textbox##.style##.height := Js.string "auto";
   textbox##.style##.height
   := Js.string (Printf.sprintf "%dpx" (max 18 textbox##.scrollHeight));
-  container##.scrollTop := container##.scrollHeight;
+  container##.scrollTop := Js.float (float container##.scrollHeight);
   Lwt.return ()
 
 let setup_printers () =
@@ -276,11 +276,11 @@ let setup_share_button ~output =
 
 let setup_js_preview () =
   let ph = by_id "last-js" in
-  let runcode : string -> 'a = Js.Unsafe.global##.toplevelEval in
-  Js.Unsafe.global##.toplevelEval
-  := fun bc ->
-  ph##.innerHTML := Js.string bc;
-  runcode bc
+  let runcode : string -> 'a = !Js_of_ocaml_compiler_dynlink.eval_ref in
+  Js_of_ocaml_compiler_dynlink.eval_ref :=
+    fun bc ->
+      ph##.innerHTML := Js.string bc;
+      runcode bc
 
 let current_position = ref 0
 
@@ -365,8 +365,9 @@ let run _ =
     let content = Js.to_string textbox##.value##trim in
     let content' =
       let len = String.length content in
-      if try content <> "" && content.[len - 1] <> ';' && content.[len - 2] <> ';'
-         with _ -> true
+      if
+        try content <> "" && content.[len - 1] <> ';' && content.[len - 2] <> ';'
+        with _ -> true
       then content ^ ";;"
       else content
     in
@@ -376,7 +377,7 @@ let run _ =
     JsooTop.execute true ~pp_code:sharp_ppf ~highlight_location caml_ppf content';
     resize ~container ~textbox ()
     >>= fun () ->
-    container##.scrollTop := container##.scrollHeight;
+    container##.scrollTop := Js.float (float container##.scrollHeight);
     textbox##focus;
     Lwt.return_unit
   in
@@ -445,7 +446,7 @@ let run _ =
        match exc with
        | Js_error.Exn e ->
            let e = Js_error.to_error e in
-           Firebug.console##log e##.stack
+           Console.console##log e##.stack
        | _ -> ());
   Lwt.async (fun () ->
       resize ~container ~textbox ()
@@ -480,7 +481,7 @@ let run _ =
   with
   | Not_found -> ()
   | exc ->
-      Firebug.console##log_3
+      Console.console##log_3
         (Js.string "exception")
         (Js.string (Printexc.to_string exc))
         exc
